package com.indicee.api.examples;

import java.io.IOException;
import java.io.InputStream;
import java.net.URLEncoder;
import java.security.NoSuchAlgorithmException;
import java.util.ArrayList;
import java.util.List;

import javax.net.ssl.SSLContext;
import javax.xml.parsers.DocumentBuilderFactory;

import oauth.signpost.commonshttp.CommonsHttpOAuthConsumer;

import org.apache.http.HttpResponse;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.conn.ClientConnectionManager;
import org.apache.http.conn.scheme.PlainSocketFactory;
import org.apache.http.conn.scheme.Scheme;
import org.apache.http.conn.scheme.SchemeRegistry;
import org.apache.http.conn.ssl.SSLSocketFactory;
import org.apache.http.impl.client.DefaultHttpClient;
import org.apache.http.impl.conn.SingleClientConnManager;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

/**
 * Delete all the datasets in the user's current data library using 
 * <a href="http://dev.indicee.com/api/modelapi/modeldataset/model-dataset-list">datasets/list.xml</a> and 
 * <a href="http://dev.indicee.com/api/modelapi/modeldataset/model-dataset-delete">datasets/delete.xml</a>.
 * @author thaggie
 */
public class DeleteAllData {
	
	private static final String HOST            = "https://secure.indicee.com";
	private static final String CONSUMER_KEY    = "XXXXXXXX-XXXX-XXXX-XXXX-XXXXXXXXXXXX";
	private static final String CONSUMER_SECRET = "XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX";
	private static final String TOKEN_KEY       = "XXXXXXXX-XXXX-XXXX-XXXX-XXXXXXXXXXXX";
	private static final String TOKEN_SECRET    = "XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX";

	public static void main(String[] args) throws Exception {
        CommonsHttpOAuthConsumer commonsHttpConsumer = new CommonsHttpOAuthConsumer(CONSUMER_KEY, CONSUMER_SECRET);
        commonsHttpConsumer.setTokenWithSecret(TOKEN_KEY, TOKEN_SECRET);
        
        Document document = getItems(commonsHttpConsumer, "datasets");
        List<String> datasetNames = parseDatasetNames(document);
        for (String datasetName : datasetNames) {
        	System.out.println("Deleteing: " + datasetName);
        	deleteItem(commonsHttpConsumer, "datasets", datasetName);
        }
	}

	private static List<String> parseDatasetNames(Document document) {
		List<String> datasetNames = new ArrayList<String>();
		NodeList elements = document.getElementsByTagName("dataset");
		for (int i = 0; i < elements.getLength(); i++) {
			Node node = elements.item(i);
			NodeList children = node.getChildNodes();
			for (int j = 0; j < children.getLength(); j++) {
				Node childNode = children.item(j);
				if (childNode.getNodeName().equals("name")) {
					String datasetName = childNode.getTextContent();
					datasetNames.add(datasetName);
				}
			}
		}
		return datasetNames;
	}

	private static DefaultHttpClient createHttpClient() throws Exception {
		SchemeRegistry registry = new SchemeRegistry();

		registry.register(new Scheme("http", 80, PlainSocketFactory.getSocketFactory()));

		try {
			SSLSocketFactory sf = new SSLSocketFactory(SSLContext.getDefault(),
					SSLSocketFactory.ALLOW_ALL_HOSTNAME_VERIFIER);
			registry.register(new Scheme("https", 443, sf));
		} catch (NoSuchAlgorithmException e) {
			throw new Exception("Problem creating ssl connection", e);
		}

		ClientConnectionManager conman = new SingleClientConnManager(registry);
		DefaultHttpClient httpClient = new DefaultHttpClient(conman);
		return httpClient;
	}
	
	private static Document getItems(CommonsHttpOAuthConsumer commonsHttpConsumer, String type)
			throws Exception {
		HttpGet httpGet = new HttpGet(HOST + "/api/models/" + type + "/list.xml");

		// init a http client
		DefaultHttpClient httpClient = createHttpClient();
		
		commonsHttpConsumer.sign(httpGet);

		// send the data to get the response
		HttpResponse httpResponse = httpClient.execute(httpGet);
		int status = httpResponse.getStatusLine().getStatusCode();
		if (status != 200) {
			return null;
		}

		InputStream body = httpResponse.getEntity().getContent();
		try {
			return DocumentBuilderFactory.newInstance().newDocumentBuilder()
					.parse(body);
		} catch (SAXException e) {
			e.printStackTrace();
			return null;
		} catch (IOException e) {
			e.printStackTrace();
			return null;
		}
	}

	private static boolean deleteItem(CommonsHttpOAuthConsumer commonsHttpConsumer, String type, String name)
			throws Exception {
		name = URLEncoder.encode(name, "utf-8").replaceAll("\\+", "%20");
		String url = HOST + "/api/models/" + type + "/" + name + "/delete.xml";
		HttpPost httpPost = new HttpPost(url);

		// init a http client
		DefaultHttpClient httpClient = createHttpClient();

		commonsHttpConsumer.sign(httpPost);

		// send the data to get the response
		HttpResponse httpResponse = httpClient.execute(httpPost);
		int status = httpResponse.getStatusLine().getStatusCode();
		if (status == 200) {
			return true;
		} else {
			return false;
		}
	}
}
