package com.indicee.api.connector.util;

import java.io.InputStream;

import com.indicee.api.connector.MimeType;

public class InputStreamWrapper {

    private final String fileName;

    private final InputStream inputSteam;

    private final MimeType mimeType;
    
    private final String charset;

    /**
     * Creates a new InputStreamWrapper. This wrapper is used to send an input
     * stream as a file to Indicee.
     * 
     * @param inputSteam
     *            The input stream.
     * @param mimeType
     *            The mine type of the input stream
     * @param fileName
     *            The name of this InputStream
     */
    public InputStreamWrapper(InputStream inputSteam, MimeType mimeType, String fileName) {
    	this(inputSteam, mimeType, fileName, null);
    }
    
    /**
     * Creates a new InputStreamWrapper. This wrapper is used to send an input
     * stream as a file to Indicee.
     * 
     * @param inputSteam
     *            The input stream.
     * @param mimeType
     *            The mine type of the input stream
     * @param fileName
     *            The name of this InputStream
     * @param charset
     *            The character set
     */
    public InputStreamWrapper(InputStream inputSteam, MimeType mimeType, String fileName, String charset) {
        this.inputSteam = inputSteam;
        this.mimeType = mimeType;
        this.fileName = fileName;
        this.charset = charset;
    }

    /**
     * Returns the file name
     * 
     * @return The file name
     */
    public String getFileName() {
        return fileName;
    }

    /**
     * Returns the {@link InputStream} associated to this
     * {@link InputStreamWrapper}.
     */
    public InputStream getInputSteam() {
        return inputSteam;
    }

    /**
     * Returns the MineType of this.
     * 
     * @return the MineType
     */
    public MimeType getMimeType() {
        return mimeType;
    }
    
    /**
     * Returns the character set the input stream is encoded with.
     * @return
     */
    public String getCharset() {
		return charset;
	}
}
