package com.indicee.api.objects;

import java.io.Serializable;
import java.util.Date;

public abstract class IndiceeObject implements Serializable {

    private static final long serialVersionUID = 7213662872623706430L;

    @SuppressWarnings("deprecation")
    protected static String dateToString(Date date) {
        return (date.getYear() + 1900) + "-" + pad(date.getMonth() + 1) + "-" + pad(date.getDate()) + "T"
                + pad(date.getHours()) + ":" + pad(date.getMinutes()) + ":" + pad(date.getSeconds()) + "-"
                + pad(date.getTimezoneOffset() / 60) + ":" + pad(date.getTimezoneOffset() % 60);
    }

    private static String pad(int num) {
        String number = String.valueOf(num);
        return number.length() == 2 ? number : "0" + number;
    }

    /**
     * Replaces & " < with &amp; &quot; &lt; 
     * @param string
     * @return the escaped string
     */
    public static String htmlEncode(String string) {
        return string == null ? "" : string.replaceAll("&", "&amp;").replaceAll("\"", "&quot;").replaceAll("<", "&lt;");
    }

    public String toFormatedString() {
        return toString("\n", "\t", "");
    }

    @Override
    public String toString() {
        return toString("", "", "");
    }

    public abstract String toString(String newLine, String tab, String indent);
    
    
	protected static final void openElement(String indent, String newLine, String element, StringBuilder sb) {
		sb.append(indent).append("<").append(element).append(">").append(newLine);
	}

	protected static final void closeElement(String indent, String newLine, String element, StringBuilder sb) {
		sb.append(indent).append("</").append(element).append(">").append(newLine);
	}
	
	protected static final void textElement(String indent, String newLine, String element, String value, StringBuilder sb) {
		if (value != null) {
			sb.append(indent).append("<").append(element).append(">").append(htmlEncode(value)).append("</").append(element).append(">").append(newLine);
		}
	}
	
	protected static final void textElement(String indent, String newLine, String element, Date value, StringBuilder sb) {
		if (value != null) {
			textElement(indent, newLine, element, dateToString(value), sb);
		}		
	}
	
	protected static final void textElement(String indent, String newLine, String element, Object value, StringBuilder sb) {
		if (value != null) {
			textElement(indent, newLine, element, String.valueOf(value), sb);
		}
	}

}
