package com.indicee.api.objects;

/**
 * Represents a page of records of data
 */
public class Page {
	// records per page
	private long rpp;
	// the current page
	private long cursor;
	
	public Page(long rpp, long cursor){
		this.rpp = rpp;
		this.cursor = cursor;
	}
	
	/**
	 * Constructor, the string format is rpp=xx&cursor=xx where xx are longs
	 * @param nextPage - a string representing the page.
	 */
	public Page(String nextPage){
		parse(nextPage);
	}
	
	/**
	 * Constructor, initially set to defaults
	 */
	public Page() {
		this.rpp = -1L;
		this.rpp = -1L;
	}

	private void parse(String nextPage) {
		if(nextPage==null || nextPage.length()<1) {
			this.rpp=0L;
			this.cursor=0L;
			return;
		}
		for (String param: nextPage.substring(1).split("&")) {
			String[] map = param.split("=");
			if(map[0].equals("rpp")) {
				this.rpp = Long.parseLong(map[1]);
			}else if(map[0].equals("cursor")) {
				this.cursor = Long.parseLong(map[1]);
			}
		}
	}

	/**
	 * Get the number of records to include in each page of data.
	 * @return long
	 */
	public long getRpp() {
		return rpp;
	}

	/**
	 * Get the current page number.
	 * @return long
	 */
	public long getCursor() {
		return cursor;
	}

	/**
	 * Set the current page by a parsable string, the string format is rpp=xx&cursor=xx where xx are longs
	 * @param page
	 */
	public void setPage(String page) {
		parse(page);
	}
	
	/**
	 * Test if it's the last page.
	 * @return boolean
	 */
	public boolean isDone() {
		return rpp==0;
	}
	
	/**
	 * Test if the page is default.
	 * @return boolean
	 */
	public boolean isDefault() {
		return rpp==-1L;
	}
}
