package com.indicee.api.objects.account;

import java.util.Date;
import java.util.List;

import com.indicee.api.objects.IndiceeObject;

public class Account extends IndiceeObject {

    private static final long serialVersionUID = 5768163777585012189L;

    private List<Attribute> attributes;

    private String organization;
    private Date createdDate;
    private String displayName;
    private String email;
    private String firstName;
    private long id;
    private String industry;
    private String lastName;
    private Long parent;
    private String password;
    private String phone;
    private String status;
    private String title;

    public Attribute getAttribute(String name){
    	for (Attribute attribute : attributes)
    		if (attribute.getName().equals(name))
    			return attribute;
    	return null;
    }
    
    public List<Attribute> getAttributes() {
        return attributes;
    }

    public String getOrganization() {
        return organization;
    }

    public Date getCreatedDate() {
        return createdDate;
    }

    public String getDisplayName() {
        return displayName;
    }

    public String getEmail() {
        return email;
    }

    public String getFirstName() {
        return firstName;
    }

    public long getId() {
        return id;
    }

    public String getIndustry() {
        return industry;
    }

    public String getLastName() {
        return lastName;
    }

    public Long getParent() {
        return parent;
    }

    public String getPassword() {
        return password;
    }

    public String getPhone() {
        return phone;
    }

    public String getStatus() {
        return status;
    }

    public String getTitle() {
        return title;
    }

    public void setAttribute(String name, String value){
    	setAttribute(name, value,null);
    }
    
    public void setAttribute(String name, String value, Boolean hierarchy){
    	Attribute attr = getAttribute(name);
    	if(attr!=null){
    		attr.setValue(value);
    		if(hierarchy!=null){
    			attr.setHierarchy(hierarchy);
    		}
    	}else{
    		if(hierarchy==null){
    			hierarchy=false;
    		}
    		attr = new Attribute(name, value, hierarchy);
    		attributes.add(attr);
    	}
    }
    
    public void setAttributes(List<Attribute> attributes) {
        this.attributes = attributes;
    }

    public void setOrganization(String organization) {
        this.organization = organization;
    }

    public void setCreatedDate(Date createdDate) {
        this.createdDate = createdDate;
    }

    public void setDisplayName(String displayName) {
        this.displayName = displayName;
    }

    public void setEmail(String email) {
        this.email = email;
    }

    public void setFirstName(String firstName) {
        this.firstName = firstName;
    }

    public void setId(long id) {
        this.id = id;
    }

    public void setIndustry(String industry) {
        this.industry = industry;
    }

    public void setLastName(String lastName) {
        this.lastName = lastName;
    }

    public void setParent(long parent) {
        this.parent = parent;
    }

    public void setParent(Long parent) {
        this.parent = parent;
    }

    public void setPassword(String password) {
        this.password = password;
    }

    public void setPhone(String phone) {
        this.phone = phone;
    }

    public void setStatus(String status) {
        this.status = status;
    }

    public void setTitle(String title) {
        this.title = title;
    }

    @Override
    public String toString(String newLine, String tab, String indent) {
        StringBuilder sb = new StringBuilder();
        sb.append(indent).append("<account>").append(newLine);
        sb.append(tab).append(indent).append("<id>").append(id).append("</id>").append(newLine);
        if (createdDate != null)
            sb.append(tab).append(indent).append("<created_time>").append(dateToString(createdDate)).append(
                    "</created_time>").append(newLine);
        if (displayName != null)
            sb.append(tab).append(indent).append("<display_name>").append(displayName).append("</display_name>")
                    .append(newLine);
        if (email != null)
            sb.append(tab).append(indent).append("<email>").append(email).append("</email>").append(newLine);
        if (firstName != null)
            sb.append(tab).append(indent).append("<first_name>").append(firstName).append("</first_name>").append(
                    newLine);
        if (industry != null)
            sb.append(tab).append(indent).append("<industry>").append(industry).append("</industry>").append(newLine);
        if (lastName != null)
            sb.append(tab).append(indent).append("<last_name>").append(lastName).append("</last_name>").append(newLine);
        if (organization != null)
            sb.append(tab).append(indent).append("<organization>").append(organization).append("</organization>").append(newLine);
        if (title != null)
            sb.append(tab).append(indent).append("<title>").append(title).append("</title>").append(newLine);
        if (parent != null)
            sb.append(tab).append(indent).append("<parent>").append(parent).append("</parent>").append(newLine);
        if (password != null)
            sb.append(tab).append(indent).append("<password>").append(password).append("</password>").append(newLine);
        if (status != null)
            sb.append(tab).append(indent).append("<status>").append(status).append("</status>").append(newLine);
        sb.append(indent + tab).append("<attributes>").append(newLine);
        if (attributes != null)
            for (Attribute attribute : attributes) {
                sb.append(attribute.toString(newLine, tab, indent + tab + tab)).append(newLine);
            }
        sb.append(indent + tab).append("</attributes>").append(newLine);
        sb.append(indent).append("</account>");
        return sb.toString();
    }
}
