package com.indicee.api.objects.model;

public class Field extends StandardModelObject {

    public enum AggregateType {
        Avg("Avg"), Count("Count"), Latest("Latest"), Sum("Sum"),DistinctCount("DistinctCount"),Min("Min"),Max("Max") ;
        public static AggregateType get(String name) {
            for (AggregateType aggregateType : AggregateType.values()) {
                if (aggregateType.value.equals(name)) {
                    return aggregateType;
                }
            }
            return null;
        }

        private String value;

        private AggregateType(String value) {
            this.value = value;
        }
    }

    public enum TrackingType {
        Type1("Type1");
        public static TrackingType get(String name) {
            for (TrackingType trackingType : TrackingType.values()) {
                if (trackingType.value.equals(name)) {
                    return trackingType;
                }
            }
            return null;
        }

        private String value;

        private TrackingType(String value) {
            this.value = value;
        }
    }

    private static final long serialVersionUID = 4726890561018893502L;

    private AggregateType aggregateType;
    private String dataFormat;
    private DataType dataType;
    private String formula;
    private TrackingType trackingType;

    public Field() {
    }
    
    @Override
    public void setName(String name) {
    	// validate - field names cannot contain [ or ] as this is how they're used in formulas
    	if (name != null && (name.contains("[") || name.contains("]"))) {
    		throw new IllegalArgumentException("\"" + name + "\" is an invalid field name, field names cannot contain \"[\" or \"]\"");
    	}
    	super.setName(name);
    }

    public AggregateType getAggregateType() {
        return aggregateType;
    }

    public String getDataFormat() {
        return dataFormat;
    }

    public DataType getDataType() {
        return dataType;
    }

    public String getFormula() {
        return formula;
    }

    public TrackingType getTrackingType() {
        return trackingType;
    }

    public void setAggregateType(AggregateType aggregateType) {
        this.aggregateType = aggregateType;
    }

    public void setDataFormat(String dataformat) {
        this.dataFormat = dataformat;
    }

    public void setDataType(DataType dataType) {
        this.dataType = dataType;
    }

    public void setFormula(String formula) {
        this.formula = formula;
    }

    public void setTrackingType(TrackingType trackingType) {
        this.trackingType = trackingType;
    }

    @Override
    public String toString(String newLine, String tab, String indent) {
        StringBuilder sb = new StringBuilder();
        sb.append(indent).append("<field>").append(newLine);
        sb.append(printStandardEntities(newLine, tab, indent + tab));
        if (trackingType != null) {
            sb.append(tab).append(indent).append("<tracking_type>").append(trackingType.value).append(
                    "</tracking_type>").append(newLine);
        }
        sb.append(tab).append(indent).append("<data_type>").append(dataType).append("</data_type>").append(
                newLine);
        if (aggregateType != null) {
            sb.append(tab).append(indent).append("<aggregate_type>").append(aggregateType.value).append(
                    "</aggregate_type>").append(newLine);
        }
        if (formula != null) {
            sb.append(tab).append(indent).append("<formula>").append(htmlEncode(formula)).append("</formula>").append(newLine);
        }
        sb.append(indent).append("</field>");
        return sb.toString();
    }

}
