package com.indicee.api.objects.report;

import java.io.Serializable;
import java.util.Date;
import java.util.List;

import com.indicee.api.objects.IndiceeQualifiedObject;
import com.indicee.api.objects.dashboard.CachingMetadata;
import com.indicee.api.objects.report.axes.Axes;
import com.indicee.api.objects.report.data.Data;
import com.indicee.api.objects.report.view.View;

public class Report extends IndiceeQualifiedObject implements Serializable {

    private static final long serialVersionUID = 2970302563525595882L;
    private Axes axes;
    private List<Column> columns;
    private Date createdDate;
    private Date generatedOn;
    private long timeToGenerate;
    private String createdDateString;
    private Boolean deleted;
    private Metadatas metadatas;
    private Data reportData;
    private View reportView;
    private VisualizationOptions visualizationOptions;
	private CachingMetadata cachingMetadata;

    public Axes getAxes() {
        return axes;
    }

    public List<Column> getColumns() {
        return columns;
    }

    public Date getCreatedDate() {
        return createdDate;
    }

    public String getCreatedDateString() {
        return createdDateString;
    }

    public Boolean getDeleted() {
        return deleted;
    }

    public Metadatas getMetadatas() {
        return metadatas;
    }

    public Data getReportData() {
        return reportData;
    }

    public View getReportView() {
        return reportView;
    }

    public VisualizationOptions getVisualizationOptions() {
        return visualizationOptions;
    }

    public Boolean isDeleted() {
        return deleted;
    }

    public void setAxes(Axes axes) {
        this.axes = axes;
    }

    public void setColumns(List<Column> columns) {
        this.columns = columns;
    }

    public void setCreatedDate(Date dateTime) {
        this.createdDate = dateTime;
    }

    public void setCreatedDateString(String createdDateString) {
        this.createdDateString = createdDateString;
    }

    public void setDeleted(Boolean deleted) {
        this.deleted = deleted;
    }

    public void setMetadatas(Metadatas metadatas) {
        this.metadatas = metadatas;
    }

    public void setReportData(Data reportData) {
        this.reportData = reportData;
    }

    public void setReportView(View reportView) {
        this.reportView = reportView;
    }

    public void setVisualizationOptions(VisualizationOptions visualizationOptions) {
        this.visualizationOptions = visualizationOptions;
    }

    @Override
    public String toString(String newLine, String tab, String indent) {
        StringBuilder sb = new StringBuilder();
        sb.append("<report>").append(newLine);
        
        sb.append(tab).append(printQualifiedEntities(newLine,tab,indent)).append(newLine);
        
        if (deleted != null) {
            sb.append(tab).append("<deleted>").append(deleted).append("</deleted>").append(newLine);
        }
        sb.append(tab).append("<created_datetime>").append(createdDateString != null ? createdDateString : "").append(
                "</created_datetime>").append(newLine);

        if (cachingMetadata != null) {
        	sb.append(cachingMetadata.toString(newLine, tab, tab)).append(newLine);
        }
        if (metadatas != null) {
            sb.append(metadatas.toString(newLine, tab, tab)).append(newLine);
            }
        if (visualizationOptions != null) {
            sb.append(visualizationOptions.toString(newLine, tab, tab)).append(newLine);
        }
        if (axes != null) {
            sb.append(axes.toString(newLine, tab, tab)).append(newLine);
            }
        if (reportView != null) {
            sb.append(reportView.toString(newLine, tab, tab)).append(newLine);
        }
        if (columns != null && !columns.isEmpty()) {
            sb.append(tab).append(indent).append("<header>").append(newLine);
            for (Column column : columns) {
                sb.append(column.toString(newLine, tab, indent + tab + tab)).append(newLine);
            }
            sb.append(tab).append(indent).append("</header>").append(newLine);
        }
        if (reportData != null) {
            sb.append(reportData.toString(newLine, tab, tab)).append(newLine);
        }

        sb.append("</report>").append(newLine);
        return sb.toString();
    }

	public Date getGeneratedOn() {
		return generatedOn;
	}

	public void setGeneratedOn(Date generatedOn) {
		this.generatedOn = generatedOn;
	}

	public long getTimeToGenerate() {
		return timeToGenerate;
	}

	public void setTimeToGenerate(long timeToGenerate) {
		this.timeToGenerate = timeToGenerate;
	}

	public void setCachingMetadata(CachingMetadata cachingMetadata) {
		this.cachingMetadata = cachingMetadata;
	}
	
	public CachingMetadata getCachingMetadata() {
		return cachingMetadata;
	}
}
