package com.indicee.api.objects.report.view;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

import com.indicee.api.objects.report.Collection;
import com.indicee.api.objects.report.view.attributes.AttributesOrigin;
import com.indicee.api.objects.report.view.axes.ViewAxis;
import com.indicee.api.objects.report.view.filter.Filter;

public class View implements Serializable {

    private static final long serialVersionUID = 4844209819566270771L;
    private List<AttributesOrigin> attributeOrigins=new ArrayList<AttributesOrigin>();
    private List<Collection> collections=new ArrayList<Collection>();
    private List<Filter> fieldFilters=new ArrayList<Filter>();
    private List<ViewAxis> reportViewAxes=new ArrayList<ViewAxis>();
    private List<String> preferredJoinEntities = new ArrayList<String>();

    public View() {
    }

    public View(View reportView) {
        if (reportView.getReportViewAxes() != null) {
            reportViewAxes = new ArrayList<ViewAxis>();
            for(ViewAxis axis : reportView.getReportViewAxes()){
                reportViewAxes.add(new ViewAxis(axis));
            }
        }
    }

    public List<AttributesOrigin> getAttributeOrigins() {
        return attributeOrigins;
    }

    public List<Collection> getCollections() {
        return collections;
    }
    
	public List<String> getPreferredJoinEntities() {
		 return preferredJoinEntities;
	}

    public List<Filter> getFieldFilters() {
        return fieldFilters;
    }

    public List<ViewAxis> getReportViewAxes() {
        return reportViewAxes;
    }

    public void setAttributeOrigins(List<AttributesOrigin> attributeOrigins) {
        this.attributeOrigins = attributeOrigins;
    }

    public void setCollections(List<Collection> collections) {
        this.collections = collections;
    }

	public void setPreferredJoinEntities(List<String> preferredJoinEntities) {
		this.preferredJoinEntities = preferredJoinEntities;
	}

    public void setFieldFilters(List<Filter> fieldFilters) {
        this.fieldFilters = fieldFilters;
    }

    public void setReportViewAxes(List<ViewAxis> reportViewAxes) {
        this.reportViewAxes = reportViewAxes;
    }

    public String toFormatedString() {
        return toString("\n", "\t");
    }

    @Override
    public String toString() {
        return toString("", "");
    }

    private String toString(String newLine, String tab) {
        return toString(newLine, tab, "");
    }

    public String toString(String newLine, String tab, String indent) {
        StringBuilder sb = new StringBuilder();
        sb.append(indent).append("<view>").append(newLine);
        sb.append(indent).append(tab).append("<axes>").append(newLine);
        for (ViewAxis axis : reportViewAxes) {
            sb.append(axis.toString(newLine, tab, indent + tab + tab)).append(newLine);
        }
        sb.append(indent).append(tab).append("</axes>").append(newLine);

        sb.append(indent).append(tab).append("<fieldfilters>").append(newLine);
        if (fieldFilters != null && !fieldFilters.isEmpty()) {
            for (Filter filter : fieldFilters) {
                sb.append(filter.toString(newLine, tab, indent + tab + tab)).append(newLine);
            }
        }
        sb.append(indent).append(tab).append("</fieldfilters>").append(newLine);

        sb.append(indent).append(tab).append("<collections>").append(newLine);
        if (collections != null && !collections.isEmpty()) {
            for (Collection collection : collections) {
                sb.append(collection.toString(newLine, tab, indent + tab + tab)).append(newLine);
            }
        }
        sb.append(indent).append(tab).append("</collections>").append(newLine);

        
        if (preferredJoinEntities != null && preferredJoinEntities.size() > 0) {
        	sb.append(indent).append(tab).append("<preferredJoinEntities>").append(newLine);
        	for (String preferredJoinEntity : preferredJoinEntities) {
        		sb.append(indent).append(tab).append(tab).append("<entity name=\"").append(preferredJoinEntity).append("\"/>").append(newLine);
        	}
        	
        	sb.append(indent).append(tab).append("</preferredJoinEntities>").append(newLine);
        }
        
        if (attributeOrigins != null) {
            sb.append(indent).append(tab).append("<attributes>").append(newLine);
            if (!attributeOrigins.isEmpty()) {
                for (AttributesOrigin attributeOrigin : attributeOrigins) {
                    sb.append(attributeOrigin.toString(newLine, tab, indent + tab + tab)).append(newLine);
                }
            }
            sb.append(indent).append(tab).append("</attributes>").append(newLine);
        }
        sb.append(indent).append("</view>");
        return sb.toString();
    }

}
