package com.indicee.api.objects.report.view.attributes;

import java.io.Serializable;
import java.util.ArrayList;

import com.indicee.api.objects.IndiceeObject;

public class Attribute implements Serializable {
    private static final long serialVersionUID = 6684538237393611532L;
    private String name;
    private String type;
    private String value;
    private ArrayList<String> valueList;

    public Attribute() {
    }

    public Attribute(Attribute attribute) {
        name = attribute.getName();
        type = attribute.getType();
        value = attribute.getValue();
        valueList = attribute.getValueList() == null ? null : new ArrayList<String>(attribute.getValueList());
    }

    public Attribute(String name, String type, String value) {
        this.name = name;
        this.type = type;
        this.value = value;
        this.valueList = null;
    }

    public Attribute(String name, String type, ArrayList<String> valueList) {
        this.name = name;
        this.type = type;
        this.value = null;
        this.valueList = valueList;
    }
    
    public String getName() {
        return name;
    }

    public String getType() {
        return type;
    }

    public String getValue() {
        return value;
    }

    public void setName(String name) {
        this.name = name;
    }

    public void setType(String type) {
        this.type = type;
    }

    public void setValue(String value) {
        this.value = value;
    }

    public ArrayList<String> getValueList() {
		return valueList;
	}
    
    public void setValueList(ArrayList<String> valueList) {
		this.valueList = valueList;
	}
    
    public String toString(String newLine, String tab, String indent) {
        StringBuilder sb = new StringBuilder();
        sb.append(indent).append("<attribute name=\"" + name + "\" type=\"" + type + "\" ");
        
        if (valueList != null) {
        	sb.append(">");
        	sb.append(newLine);
        	String itemIndent = indent+tab;
        	for (String item : valueList) {
        		sb.append(itemIndent);
        		sb.append("<item value=\"" + IndiceeObject.htmlEncode(item) + "\"/>");
        		sb.append(newLine);
        	}
        	sb.append(indent).append("</attribute>");
        } else {
        	sb.append("value=\"" + IndiceeObject.htmlEncode(value) + "\"/>");
        }
        return sb.toString();
    }
}
