package com.indicee.api.objects.report.view.axes;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

import com.indicee.api.objects.IndiceeObject;
import com.indicee.api.objects.report.view.attributes.AttributesOrigin;

public class AxisSort extends IndiceeObject implements Serializable {

    private static final long serialVersionUID = -82301717445356292L;

    private boolean ascending;
    private List<AttributesOrigin> attributes;
    private boolean breakHierarchies;
    private QueryValue queryValue;

    public AxisSort() {
    }

    public AxisSort(AxisSort axisSorting) {
        ascending = axisSorting.isAscending();
        breakHierarchies = axisSorting.isBreakHierarchies();
        queryValue = axisSorting.getQueryValue().clone();
        if (axisSorting.getAttributes() != null) {
            attributes = new ArrayList<AttributesOrigin>();
            for (AttributesOrigin attribute : axisSorting.getAttributes()) {
                attributes.add(new AttributesOrigin(attribute));
            }
        }
    }

    public List<AttributesOrigin> getAttributes() {
        return attributes;
    }

    public QueryValue getQueryValue() {
        return queryValue;
    }

    public boolean isAscending() {
        return ascending;
    }

    public boolean isBreakHierarchies() {
        return breakHierarchies;
    }

    public void setAscending(boolean ascending) {
        this.ascending = ascending;
    }

    public void setAttributes(List<AttributesOrigin> attributes) {
        this.attributes = attributes;
    }

    public void setBreakHierarchies(boolean breakHierarchies) {
        this.breakHierarchies = breakHierarchies;
    }

    public void setQueryValue(QueryValue queryValue) {
        this.queryValue = queryValue;
    }

    @Override
    public String toString(String newLine, String tab, String indent) {
        StringBuilder sb = new StringBuilder();
        sb.append(indent).append(
                "<axissort ascending=\"" + ascending + "\" breakhierarchies=\"" + breakHierarchies + "\">").append(
                newLine);
        sb.append(queryValue.toString(newLine, tab, indent + tab)).append(newLine);
        sb.append(indent + tab).append("<attributes>").append(newLine);
        if (attributes != null) {
            for (AttributesOrigin attribute : attributes) {
                sb.append(attribute.toString(newLine, tab, indent + tab + tab)).append(newLine);
            }
        }
        sb.append(indent + tab).append("</attributes>").append(newLine);
        sb.append(indent).append("</axissort>");
        return sb.toString();
    }
}
