package com.indicee.api.objects.report.view.axes;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

import com.indicee.api.objects.IndiceeObject;
import com.indicee.api.objects.report.view.attributes.Attribute;

public class GroupFilter extends IndiceeObject implements Serializable {

    private static final long serialVersionUID = 3005773789021749033L;

    private String comparator;

    private List<Attribute> comparatorAttributes;

    private QueryValue queryValue;

    public GroupFilter() {
    }

    public GroupFilter(GroupFilter groupFilter) {
        comparator = groupFilter.getComparator();
        queryValue = groupFilter.getQueryValue().clone();
        if (groupFilter.getComparatorAttributes() != null) {
            comparatorAttributes = new ArrayList<Attribute>();
            for (Attribute comparatorAttribute : groupFilter.getComparatorAttributes()) {
                comparatorAttributes.add(new Attribute(comparatorAttribute));
            }
        }
    }

    public String getComparator() {
        return comparator;
    }

    public List<Attribute> getComparatorAttributes() {
        return comparatorAttributes;
    }

    public QueryValue getQueryValue() {
        return queryValue;
    }

    public void setComparator(String comparator) {
        this.comparator = comparator;
    }

    public void setComparatorAttributes(List<Attribute> comparatorAttributes) {
        this.comparatorAttributes = comparatorAttributes;
    }

    public void setQueryValue(QueryValue queryValue) {
        this.queryValue = queryValue;
    }

    @Override
    public String toString(String newLine, String tab, String indent) {
        StringBuilder sb = new StringBuilder();
        sb.append(indent).append("<groupfilter comparator=\"" + comparator + "\">").append(newLine);
        sb.append(queryValue.toString(newLine, tab, indent + tab)).append(newLine);
        if (comparatorAttributes != null && !comparatorAttributes.isEmpty()) {
            sb.append(indent + tab).append("<comparatorattributes>").append(newLine);
            for (Attribute attribute : comparatorAttributes) {
                sb.append(attribute.toString(newLine, tab, indent + tab + tab)).append(newLine);
            }
            sb.append(indent + tab).append("</comparatorattributes>").append(newLine);
        }
        sb.append(indent).append("</groupfilter>");
        return sb.toString();
    }
}
