package com.indicee.api.objects.report.view.axes;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

import com.indicee.api.objects.report.view.attributes.AttributesOrigin;
public class GroupingAxisLayer extends Layer implements Serializable, Cloneable {

    private static final long serialVersionUID = 6303669075793800208L;
    private List<AttributesOrigin> attributes;
    private boolean drillable;
    private List<GroupFilter> groupFilters;
    private LayerGroupSorting groupSorting;
    private String hierarchy;
    private String hierarchyDisplay;
    private boolean leadingTotals;
    private LayerQueryGrouping queryGrouping;
    private boolean trailingTotals;

    public GroupingAxisLayer() {
    }

    public GroupingAxisLayer(GroupingAxisLayer groupingAxisLayer) {
        leadingTotals = groupingAxisLayer.isLeadingTotals();
        trailingTotals = groupingAxisLayer.isTrailingTotals();
        hierarchy = groupingAxisLayer.getHierarchy();
        hierarchyDisplay = groupingAxisLayer.getHierarchyDisplay();
        drillable = groupingAxisLayer.isDrillable();
        queryGrouping = new LayerQueryGrouping(groupingAxisLayer.getQueryGrouping());
        groupSorting = new LayerGroupSorting(groupingAxisLayer.getGroupSorting());
        if (groupingAxisLayer.getGroupFilters() != null) {
            groupFilters = new ArrayList<GroupFilter>();
            for (GroupFilter groupFilter : groupingAxisLayer.getGroupFilters()) {
                groupFilters.add(new GroupFilter(groupFilter));
            }
        }
    }

    @Override
    public GroupingAxisLayer clone() {
        return new GroupingAxisLayer(this);
    }

    public List<AttributesOrigin> getAttributes() {
        return attributes;
    }
    public List<GroupFilter> getGroupFilters() {
        return groupFilters;
    }

    public LayerGroupSorting getGroupSorting() {
        return groupSorting;
    }

    public String getHierarchy() {
        return hierarchy;
    }

    public String getHierarchyDisplay() {
        return hierarchyDisplay;
    }

    public LayerQueryGrouping getQueryGrouping() {
        return queryGrouping;
    }

    public boolean isDrillable() {
        return drillable;
    }

    public boolean isLeadingTotals() {
        return leadingTotals;
    }

    public boolean isTrailingTotals() {
        return trailingTotals;
    }

    public void setAttributes(List<AttributesOrigin> attributes) {
        this.attributes = attributes;
    }

    public void setDrillable(boolean drillable) {
        this.drillable = drillable;
    }

    public void setGroupFilters(List<GroupFilter> groupFilters) {
        this.groupFilters = groupFilters;
    }

    public void setGroupSorting(LayerGroupSorting groupSorting) {
        this.groupSorting = groupSorting;
    }

    public void setHierarchy(String hierarchy) {
        this.hierarchy = hierarchy;
    }

    public void setHierarchyDisplay(String hierarchyDisplay) {
        this.hierarchyDisplay = hierarchyDisplay;
    }

    public void setLeadingTotals(boolean leadingTotals) {
        this.leadingTotals = leadingTotals;
    }

    public void setQueryGrouping(LayerQueryGrouping queryGrouping) {
        this.queryGrouping = queryGrouping;
    }

    public void setTrailingTotals(boolean trailingTotals) {
        this.trailingTotals = trailingTotals;
    }

    @Override
    public String toString(String newLine, String tab, String indent) {
        StringBuilder sb = new StringBuilder();
        sb.append(indent).append(
                "<groupingaxislayer leadingtotals=\"" + leadingTotals + "\" trailingtotals=\"" + trailingTotals
                        + "\" hierarchy=\"" + hierarchy + "\" hierarchydisplay=\"" + 
                        htmlEncode(hierarchyDisplay)
                        + "\" drillable=\"" + drillable + "\">").append(newLine);
        if (queryGrouping != null) {
            sb.append(queryGrouping.toString(newLine, tab, indent + tab)).append(newLine);
        }
        if (groupSorting != null) {
            sb.append(groupSorting.toString(newLine, tab, indent + tab)).append(newLine);
        }
        if (groupFilters != null && !groupFilters.isEmpty()) {
            sb.append(indent + tab).append("<groupfilters>").append(newLine);
            for (GroupFilter groupFilter : groupFilters) {
                sb.append(groupFilter.toString(newLine, tab, indent + tab + tab)).append(newLine);
            }
            sb.append(indent + tab).append("</groupfilters>").append(newLine);
        }
        sb.append(indent + tab).append("<attributes>").append(newLine);
        if (attributes != null) {
            for (AttributesOrigin originAttribute : attributes) {
                sb.append(originAttribute.toString(newLine, tab, indent + tab + tab)).append(newLine);
            }
        }
        sb.append(indent + tab).append("</attributes>").append(newLine);
        sb.append(indent).append("</groupingaxislayer>");
        return sb.toString();
    }
}
