package com.indicee.api.objects.report.view.axes;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

import com.indicee.api.objects.IndiceeObject;

public class ViewAxis extends IndiceeObject implements Serializable {

    private static final long serialVersionUID = -1136348447395899713L;

    private List<AxisSort> axisSortings;
    private List<Layer> layers;
    private String name;

    public ViewAxis() {
    }

    public ViewAxis(List<Layer> layers) {
        this.layers = layers;
    }

    public ViewAxis(ViewAxis axis) {
        if (axis.getLayers() != null) {
            layers = new ArrayList<Layer>();
            for (Layer layer : axis.getLayers()) {
                layers.add(layer.clone());
            }
        }
        if (axis.getAxisSortings() != null) {
            axisSortings = new ArrayList<AxisSort>();
            for (AxisSort axisSorting : axis.getAxisSortings()) {
                axisSortings.add(new AxisSort(axisSorting));
            }
        }
    }

    public List<AxisSort> getAxisSortings() {
        return axisSortings;
    }

    public List<Layer> getLayers() {
        return layers;
    }

    public String getName() {
        return name;
    }

    public void setAxisSortings(List<AxisSort> axisSortings) {
        this.axisSortings = axisSortings;
    }

    public void setLayers(List<Layer> layers) {
        this.layers = layers;
    }

    public void setName(String name) {
        this.name = name;
    }

    @Override
    public String toFormatedString() {
        return toString("\n", "\t", "");
    }

    @Override
    public String toString() {
        return toString("", "", "");
    }

    @Override
    public String toString(String newLine, String tab, String indent) {
        StringBuilder sb = new StringBuilder();
        sb.append(indent).append("<axis name=\"" + name + "\">").append(newLine);
        sb.append(indent + tab).append("<layers>").append(newLine);
        for (Layer layer : layers) {
            sb.append(layer.toString(newLine, tab, indent + tab + tab)).append(newLine);
        }
        sb.append(indent + tab).append("</layers>").append(newLine);
        if (axisSortings != null && !axisSortings.isEmpty()) {
            sb.append(indent + tab).append("<axissorting>").append(newLine);
            for (AxisSort axisSort : axisSortings) {
                sb.append(axisSort.toString(newLine, tab, indent + tab + tab)).append(newLine);
            }
            sb.append(indent + tab).append("</axissorting>").append(newLine);
        }
        sb.append(indent).append("</axis>");
        return sb.toString();
    }
}
