/**
 * 
 */
package com.indicee.api.objects.report.view.filter;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

import com.indicee.api.objects.report.view.attributes.AttributesOrigin;

/**
 * @author thaggie
 *
 */
public class EntityKeysFilter extends Filter implements Serializable {

	private static final long serialVersionUID = 1L;

	private ArrayList<ArrayList<FieldValue>> keyValueSets;
	
	public EntityKeysFilter() {
		
	}
	
	public EntityKeysFilter(String entity, ArrayList<ArrayList<FieldValue>> keyValueSets, ArrayList<AttributesOrigin> originAttributes) {
		super();
		setEntity(entity);
		this.keyValueSets = keyValueSets;
		this.attributeOrigins = originAttributes;
	}

	public ArrayList<ArrayList<FieldValue>> getKeyValueSets() {
		return keyValueSets;
	}

	public void setKeyValueSets(ArrayList<ArrayList<FieldValue>> keyValueSets) {
		this.keyValueSets = keyValueSets;
	}
	
	/**
	 * @see com.indicee.api.objects.report.view.filter.Filter#toString(java.lang.String, java.lang.String, java.lang.String)
	 */
	@Override
	public String toString(String newLine, String tab, String indent) {
		StringBuilder sb = new StringBuilder();
		sb.append(indent);
		sb.append("<entitykeysfilter entity=\"");
		sb.append(htmlEncode(getEntity()));
		sb.append("\">");
		sb.append(newLine);
		
		if (keyValueSets != null) {
			String keyValueSetIndent = indent + tab;
			String keyValueIndent = keyValueSetIndent + tab;
			for (List<FieldValue> keyValueSet : keyValueSets) {
				if (keyValueSet.size() > 0) {
					sb.append(keyValueSetIndent);
					sb.append("<keyvalueset>");
					sb.append(newLine);
					for (FieldValue fieldValue : keyValueSet) {
						sb.append(fieldValue.toString(newLine, tab, keyValueIndent));
					}
					sb.append(keyValueSetIndent);
					sb.append("</keyvalueset>");
					sb.append(newLine);
				}
			}
		}
		
		if (attributeOrigins != null && attributeOrigins.size() > 0) {
			String attributesIndent = indent + tab;
			String attributesOriginIndent = attributesIndent + tab;
			sb.append(attributesIndent);
			sb.append("<attributes>");
			sb.append(newLine);
			for (AttributesOrigin originAttribute : attributeOrigins) {
				sb.append(originAttribute.toString(newLine, tab, attributesOriginIndent));
			}
			sb.append(attributesIndent);
			sb.append("</attributes>");
			sb.append(newLine);
		}
		sb.append(indent);
		sb.append("</entitykeysfilter>");
		sb.append(newLine);
		
		return sb.toString();
	}

}
