package com.indicee.api.objects.report.view.filter;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

import com.indicee.api.objects.report.view.attributes.AttributesOrigin;

public class FieldFilter extends Filter implements Serializable {

    private static final long serialVersionUID = 2527038416303587156L;
    private List<FieldFilterValue> values;

    public FieldFilter() {
    }

    public FieldFilter(FieldFilter filter) {
        super(filter);
        if (filter.values != null) {
            values = new ArrayList<FieldFilterValue>();
            for (FieldFilterValue value : filter.values) {
                values.add(new FieldFilterValue(value));
            }
        }
    }

    public List<FieldFilterValue> getValues() {
        return values;
    }

    public void setValues(List<FieldFilterValue> values) {
        this.values = values;
    }

    @Override
    public String toString(String newLine, String tab, String indent) {
        StringBuilder sb = new StringBuilder();
        if(getNot()) {
        	sb.append(indent).append("<not>");
        }else {
        	sb.append(indent);
        }
        
        if (values.size() == 1) {
            sb.append(
                    "<fieldfilter field=\"" + getField() + "\" inclusive=\"" + isInclusive() + "\" value=\""
                            + htmlEncode(values.get(0).getValue()) + "\"");
            if (getEntity() != null && !getEntity().isEmpty()) {
                sb.append(" entity=\"" + getEntity() + "\"");
            }
            sb.append("/>");
        } else {
            sb.append("<fieldfilter field=\"" + getField() + "\" inclusive=\"" + isInclusive() + "\"");
            if (getEntity() != null && !getEntity().isEmpty()) {
                sb.append(" entity=\"" + getEntity() + "\"");
            }
            sb.append(">").append(newLine);
            sb.append(indent + tab).append("<values>").append(newLine);
            for (FieldFilterValue value : values) {
                sb.append(value.toString(newLine, tab, indent + tab + tab)).append(newLine);
            }
            sb.append(indent + tab).append("</values>").append(newLine);
            
            if (attributeOrigins != null) {
                sb.append(indent).append(tab).append("<attributes>").append(newLine);
                if (!attributeOrigins.isEmpty()) {
                    for (AttributesOrigin attributeOrigin : attributeOrigins) {
                        sb.append(attributeOrigin.toString(newLine, tab, indent + tab + tab)).append(newLine);
                    }
                }
                sb.append(indent).append(tab).append("</attributes>").append(newLine);
            }
            
            sb.append(indent).append("</fieldfilter>");
        }
        
        if(getNot()) {
        	sb.append("</not>");
        }
        return sb.toString();
    }
}
