package com.indicee.api.objects.report.view.filter;

import java.io.Serializable;
import java.util.Date;

public class RelativeTimePeriod implements Serializable {

    private static final long serialVersionUID = 8546877811043053071L;
    private int rangeEnd;
    private int rangeStart;
    private Long referenceTime;
    private String timePeriod;

    public RelativeTimePeriod() {
    }

    /**
     * Creates a relative Time period filter for the time between the given two
     * dates.
     * 
     * @param startDate
     * @param endDate
     */
    public RelativeTimePeriod(Date startDate, Date endDate) {
        setTimePeriod("DayOfMonthPeriod");

        //need a constant reference date. The actual date doesn't matter.
        //Date referenceDate = new Date(2010 - 1900, 0, 1);
        //Date referenceDate = new Date();
        Date referenceDate = startDate;

        //calculate the days till the end date.
        long diff = endDate.getTime() - referenceDate.getTime();
        long days = diff / (1000 * 60 * 60 * 24);
        setRangeEnd((int) days);

        //calcualte the days till the start date.
        diff = startDate.getTime() - referenceDate.getTime();
        days = diff / (1000 * 60 * 60 * 24);
        setRangeStart((int) days);

        //sets the reference time. This value needs to be in 'ticks'. Ticks are (millis * 10000 +0)
        setReferenceTime(referenceDate.getTime() * 10000L);
    }

    public int getRangeEnd() {
        return rangeEnd;
    }

    public int getRangeStart() {
        return rangeStart;
    }

    public Long getReferenceTime() {
        return referenceTime;
    }

    public String getTimePeriod() {
        return timePeriod;
    }

    public void setRangeEnd(int rangeEnd) {
        this.rangeEnd = rangeEnd;
    }

    public void setRangeStart(int rangeStart) {
        this.rangeStart = rangeStart;
    }

    public void setReferenceTime(Long referenceTime) {
        this.referenceTime = referenceTime;
    }

    public void setTimePeriod(String timePeriod) {
        this.timePeriod = timePeriod;
    }

    public String toString(String newLine, String tab, String indent) {
        StringBuilder sb = new StringBuilder();
        sb.append(indent).append(
                "<relativetimeperiod timeperiod=\"" + timePeriod + "\" rangestart=\"" + rangeStart + "\" rangeend=\""
                        + rangeEnd + "\"");
        
        if (referenceTime != null) {
        	sb.append(" referencetime=\"" + referenceTime + "\"");
        }
        sb.append("/>");
        return sb.toString();
    }
}
