package com.indicee.api.parser;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.w3c.dom.Document;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import com.indicee.api.objects.account.Account;
import com.indicee.api.objects.account.Attribute;

public final class AccountParser extends IndiceeParser {

    private enum AccountElement {
        Account("account"), Attribute("attribute"), Attributes("attributes"), CreatedTime("created_time"), DisplayName(
                "display_name"), Email("email"), FirstName("first_name"), ID("id"), Industry("industry"), LastName(
                "last_name"), Organization("organization"), Parent("parent"), Password("password"), Phone("phone"), Status(
                "status"), Title("title");

        public static AccountElement get(String name) {
            for (AccountElement item : AccountElement.values()) {
                if (item.value.equals(name)) {
                    return item;
                }
            }
            return null;
        }

        private final String value;

        private AccountElement(String value) {
            this.value = value;
        }
    }

    public static Account createAccount(Document userDocument) {
        return createAccount(userDocument.getDocumentElement());
    }

    private static Account createAccount(Node node) {
        Account user = new Account();
        for (int i = 0; i < node.getChildNodes().getLength(); i++) {
            Node child = node.getChildNodes().item(i);
            AccountElement elem = AccountElement.get(child.getNodeName());
            if (elem != null) {
                switch (elem) {
                case ID:
                    user.setId(getInteger(child));
                    break;
                case FirstName:
                    user.setFirstName(getString(child));
                    break;
                case LastName:
                    user.setLastName(getString(child));
                    break;
                case Organization:
                    user.setOrganization(getString(child));
                    break;
                case DisplayName:
                    user.setDisplayName(getString(child));
                    break;
                case Email:
                    user.setEmail(getString(child));
                    break;
                case Industry:
                    user.setIndustry(getString(child));
                    break;
                case Phone:
                    user.setPhone(getString(child));
                    break;
                case Status:
                    user.setStatus(getString(child));
                    break;
                case Title:
                    user.setTitle(getString(child));
                    break;
                case Password:
                    user.setPassword(getString(child));
                    break;
                case Attributes:
                    user.setAttributes(getAttributes(child));
                    break;
                case CreatedTime:
                    user.setCreatedDate(getDate(child));
                    break;
                default:
                	break;
                }
            }
        }
        return user;
    }

    public static Account createAccount(String userDocument) {
        return createAccount(getDocumentFromString(userDocument));
    }

    public static List<Account> createAccounts(Document usersDocument) {
        List<Account> users = new ArrayList<Account>();
        NodeList userNodes = usersDocument.getElementsByTagName(AccountElement.Account.value);
        for (int i = 0; i < userNodes.getLength(); i++) {
            users.add(createAccount(userNodes.item(i)));
        }
        return users;
    }

    public static List<Account> createAccounts(String usersDocument) {
        return createAccounts(getDocumentFromString(usersDocument));
    }

    public static List<Attribute> getAttributes(Node attributeNodes) {
        List<Attribute> attributes = new ArrayList<Attribute>();
        NodeList childNodes = attributeNodes.getChildNodes();
        for (int i = 0; i < childNodes.getLength(); i++) {
        	Node childNode = childNodes.item(i);
            NamedNodeMap attributeNodeAttributes = childNode.getAttributes();
            Attribute attribute = new Attribute();
            Node name = attributeNodeAttributes.getNamedItem("name");
            if (name != null) {
            	attribute.setName(name.getNodeValue());
            }
            Node value = attributeNodeAttributes.getNamedItem("value");
            if (value != null) {
            	attribute.setValue(value.getNodeValue());
        	}
            Node hierarchical = attributeNodeAttributes.getNamedItem("hierarchical");
            if (hierarchical != null) {
            	attribute.setHierarchy(Boolean.parseBoolean(hierarchical.getNodeValue()));
            }
            attributes.add(attribute);
        }
        return attributes;
    }

    public static Map<String, String> getParameters(Account user) {
        Map<String, String> parameter = new HashMap<String, String>();
        if (user.getFirstName() != null) {
            parameter.put(AccountElement.FirstName.value, user.getFirstName());
        }
        if (user.getLastName() != null) {
            parameter.put(AccountElement.LastName.value, user.getLastName());
        }
        if (user.getDisplayName() != null) {
            parameter.put(AccountElement.DisplayName.value, user.getDisplayName());
        }
        if (user.getOrganization() != null) {
            parameter.put(AccountElement.Organization.value, user.getOrganization());
        }
        if (user.getIndustry() != null) {
            parameter.put(AccountElement.Industry.value, user.getIndustry());
        }
        if (user.getTitle() != null) {
            parameter.put(AccountElement.Title.value, user.getTitle());
        }
        if (user.getPhone() != null) {
            parameter.put(AccountElement.Phone.value, user.getPhone());
        }
        if (user.getEmail() != null) {
            parameter.put(AccountElement.Email.value, user.getEmail());
        }
        if (user.getPassword() != null) {
            parameter.put(AccountElement.Password.value, user.getPassword());
        }
        if (user.getStatus() != null) {
            parameter.put(AccountElement.Status.value, user.getStatus());
        }
        if (user.getParent() != null) {
            parameter.put(AccountElement.Parent.value, user.getParent().toString());
        }
        return parameter;
    }
}
