package com.indicee.api.parser;

import java.util.Map;

import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

public class ApplicationDataParser extends IndiceeParser {

    private enum DataElement {
        Data("data"), Key("key"), Value("value");

        public static DataElement get(String name) {
            for (DataElement item : DataElement.values()) {
                if (item.value.equals(name)) {
                    return item;
                }
            }
            return null;
        }

        private final String value;

        private DataElement(String value) {
            this.value = value;
        }
    }

   public static NodeParser<String> valueParser = new NodeParser<String>(DataElement.Data.value) {
	   public String parse(Node dataItemNode) {
	        for (int i = 0; i < dataItemNode.getChildNodes().getLength(); i++) {
	            Node child = dataItemNode.getChildNodes().item(i);
	            DataElement elem = DataElement.get(child.getNodeName());
	            if (elem != null) {
	                switch (elem) {
	                case Value:
	                    return getString(child);
                    default:
                    	break;
	                }
	            }
	        }
	        return null;
	    }
	};

	public static NodeParser<Void> createMapParser(Map<String,String> map) {
		return new MapParser(map);
	}
	
	private static final class MapParser extends NodeParser<Void> {
		private final Map<String,String> map;
		
		public MapParser(Map<String,String> map) {
			super(DataElement.Data.value);
			this.map = map;
		}
		
		@Override
		public Void parse(Node dataItemNode) {
			NodeList childNodes = dataItemNode.getChildNodes();
			int nChildNodes = childNodes.getLength();
			String key = null;
			String value = null;
	        for (int i = 0; i < nChildNodes; ++i) {
	            Node child = childNodes.item(i);
	            DataElement elem = DataElement.get(child.getNodeName());
	            if (elem != null) {
	                switch (elem) {
	                case Key:
	                    key = getString(child);
	                    break;
	                case Value:
	                    value = getString(child);
	                    break;
                    default: 
                    	break;
	                }
	            }
	        }
            if (key != null) {
            	map.put(key, value);
            }
			return null;
		}
	}
}
