package com.indicee.api.parser;

import java.util.ArrayList;
import java.util.List;

import org.w3c.dom.Document;
import org.w3c.dom.Node;

import com.indicee.api.objects.auth.Consumer;
import com.indicee.api.objects.auth.Token;

public class AuthParser extends IndiceeParser {

    private enum ConsumerElement {
        Consumer("consumer"), Description("description"), Key("key"), Name("name"), Secret("secret"), Website("website");

        public static ConsumerElement get(String name) {
            for (ConsumerElement fieldChild : ConsumerElement.values()) {
                if (fieldChild.value.equals(name)) {
                    return fieldChild;
                }
            }
            return null;
        }

        private final String value;

        private ConsumerElement(String value) {
            this.value = value;
        }
    }

    private enum TokenElement {
        ConsumerKey("consumerKey"), Key("key"), OwnerID("ownerID"), Secret("secret"), Token("token");

        public static TokenElement get(String name) {
            for (TokenElement fieldChild : TokenElement.values()) {
                if (fieldChild.value.equals(name)) {
                    return fieldChild;
                }
            }
            return null;
        }

        private final String value;

        private TokenElement(String value) {
            this.value = value;
        }
    }

    public static Consumer createConsumer(Document consumerDocument) {
        return getConsumer(consumerDocument.getDocumentElement());
    }

    public static List<Consumer> createConsumerList(Document consumersListDocument) {
        List<Consumer> consumers = new ArrayList<Consumer>();
        for (int i = 0; i < consumersListDocument.getDocumentElement().getChildNodes().getLength(); i++) {
            Node child = consumersListDocument.getDocumentElement().getChildNodes().item(i);
            ConsumerElement elem = ConsumerElement.get(child.getNodeName());
            if (elem != null && elem.equals(ConsumerElement.Consumer)) {
                consumers.add(getConsumer(child));
            }
        }
        return consumers;
    }

    public static Token createToken(Document tokenDocument) {
        return getToken(tokenDocument.getDocumentElement());
    }

    public static List<Token> createTokenList(Document tokensListDocument) {
        List<Token> consumers = new ArrayList<Token>();
        for (int i = 0; i < tokensListDocument.getDocumentElement().getChildNodes().getLength(); i++) {
            Node child = tokensListDocument.getDocumentElement().getChildNodes().item(i);
            TokenElement elem = TokenElement.get(child.getNodeName());
            if (elem != null && elem.equals(TokenElement.Token)) {
                consumers.add(getToken(child));
            }
        }
        return consumers;
    }

    private static Consumer getConsumer(Node consumerNode) {
        Consumer consumer = new Consumer();
        for (int i = 0; i < consumerNode.getChildNodes().getLength(); i++) {
            Node child = consumerNode.getChildNodes().item(i);
            ConsumerElement elem = ConsumerElement.get(child.getNodeName());
            if (elem != null) {
                switch (elem) {
                case Name:
                    consumer.setName(getString(child));
                    break;
                case Key:
                    consumer.setKey(getString(child));
                    break;
                case Secret:
                    consumer.setSecret(getString(child));
                    break;
                case Website:
                    consumer.setWebsite(getString(child));
                    break;
                case Description:
                    consumer.setDescription(getString(child));
                    break;
                default:
                	break;
                }
            }
        }
        return consumer;
    }

    private static Token getToken(Node tokenNode) {
        Token token = new Token();
        for (int i = 0; i < tokenNode.getChildNodes().getLength(); i++) {
            Node child = tokenNode.getChildNodes().item(i);
            TokenElement elem = TokenElement.get(child.getNodeName());
            if (elem != null) {
                switch (elem) {
                case ConsumerKey:
                    token.setConsumerKey(getString(child));
                    break;
                case Key:
                    token.setKey(getString(child));
                    break;
                case Secret:
                    token.setSecret(getString(child));
                    break;
                case OwnerID:
                    token.setOwnerID(getLong(child));
                    break;
                default:
                	break;
                }
            }
        }
        return token;
    }

}
