package com.indicee.api.parser;

import java.util.ArrayList;
import java.util.List;

import org.w3c.dom.Document;
import org.w3c.dom.Node;

import com.indicee.api.objects.contribution.Ancillary;
import com.indicee.api.objects.contribution.Contribution;
import com.indicee.api.objects.contribution.Status;

public final class ContributionParser extends IndiceeParser {

    private enum ContributionElement {
        Contribution("contribution"), ContributionID("contribution_id"), Contributions("contributions"), OwnerID("owner_id"),
        Size("size"), Status("status"), UniqueName("unique_name"), RowsRead("rows_read") , RowsProcessed("rows_processed"),
        Ancillaries("ancillaries"), Name("name"), Dataset("dataset_qname"),CreationDate("creation_date"), JobID("job_id");

        public static ContributionElement get(String name) {
            for (ContributionElement item : ContributionElement.values()) {
                if (item.value.equals(name)) {
                    return item;
                }
            }
            return null;
        }

        private final String value;

        private ContributionElement(String value) {
            this.value = value;
        }
    }
    
    private enum AncillaryElement {
        Ancillary("ancillary"), Name("name"), DateValue("date_value"), DoubleValue("double_value"), StringValue("string_value");

        public static AncillaryElement get(String name) {
            for (AncillaryElement item : AncillaryElement.values()) {
                if (item.value.equals(name)) {
                    return item;
                }
            }
            return null;
        }

        private final String value;

        private AncillaryElement(String value) {
            this.value = value;
        }
    }

    public static Contribution createContribution(Document contributionDocument) {
        return createContribution(contributionDocument.getDocumentElement());
    }

    public static Contribution createContribution(Node contributionNode) {
        Contribution contribution = new Contribution();
        for (int i = 0; i < contributionNode.getChildNodes().getLength(); i++) {
            Node child = contributionNode.getChildNodes().item(i);
            ContributionElement elem = ContributionElement.get(child.getNodeName());
            if (elem != null) {
                switch (elem) {
                case Name:
                	contribution.setName(getString(child));
                	break;
                case Dataset:
                	contribution.setDatasetQName(getString(child));
                	break;
                case ContributionID:
                    contribution.setId(getInteger(child));
                    break;
                case UniqueName:
                    contribution.setUniqueName(getString(child));
                    break;
                case OwnerID:
                    contribution.setOwnerID(getInteger(child));
                    break;
                case Size:
                    contribution.setSize(getInteger(child));
                    break;
                case Status:
                    contribution.setStatus(Status.valueOf(getString(child)));
                    break;
                case RowsRead:
                    contribution.setRowsRead(getInteger(child));
                    break;
                case RowsProcessed:
                    contribution.setRowsProcessed(getInteger(child));
                    break;
                case CreationDate:
                	contribution.setCreationDate(getDate(child));
                	break;
                case Ancillaries:
                	contribution.setAncillaries(createAncillaries(child));
                	break;
                case JobID:
                	contribution.setJobId(getString(child));
                	break;
            	default:
            		break;
                }
            }
        }
        return contribution;
    }

    public static Contribution createContribution(String contributionDocument) {
        return createContribution(getDocumentFromString(contributionDocument));
    }

    public static List<Contribution> createContributions(Document contributionsDocument) {
        List<Contribution> contributions = new ArrayList<Contribution>();
        for (int i = 0; i < contributionsDocument.getChildNodes().getLength(); i++) {
            Node child = contributionsDocument.getChildNodes().item(i);
            ContributionElement elem = ContributionElement.get(child.getNodeName());
            if (elem != null && elem.equals(ContributionElement.Contributions)) {
                contributions = createContributions(child);
            }
        }
        return contributions;
    }

    private static List<Contribution> createContributions(Node contributionsNode) {
        List<Contribution> contributions = new ArrayList<Contribution>();
        for (int i = 0; i < contributionsNode.getChildNodes().getLength(); i++) {
            Node child = contributionsNode.getChildNodes().item(i);
            ContributionElement elem = ContributionElement.get(child.getNodeName());
            if (elem != null && elem.equals(ContributionElement.Contribution)) {
                contributions.add(createContribution(child));
            }
        }
        return contributions;
    }
    
    private static Ancillary createAncillary(Node ancillaryNode){
        Ancillary ancillary = new Ancillary();
        for (int i = 0; i < ancillaryNode.getChildNodes().getLength(); i++) {
            Node child = ancillaryNode.getChildNodes().item(i);
            AncillaryElement elem = AncillaryElement.get(child.getNodeName());
            if (elem != null) {
                switch (elem) {
                case Name:
                    ancillary.setName(getString(child));
                    break;
                case StringValue:
                    ancillary.setStringValue(getString(child));
                    break;
                case DoubleValue:
                    ancillary.setDoubleValue(getDouble(child));
                    break;
                case DateValue:
                    ancillary.setDateValue(getString(child));
                    break;
                default:
                	break;
                }
            }
        }
        return ancillary;
    }
    
    private static List<Ancillary> createAncillaries(Node ancillaryNode) {
        List<Ancillary> ancillaries = new ArrayList<Ancillary>();
        for (int i = 0; i < ancillaryNode.getChildNodes().getLength(); i++) {
            Node child = ancillaryNode.getChildNodes().item(i);
            AncillaryElement elem = AncillaryElement.get(child.getNodeName());
            if (elem != null && elem.equals(AncillaryElement.Ancillary)) {
            	ancillaries.add(createAncillary(child));
            }
        }
        return ancillaries;
    }

}
