package com.indicee.api.parser;

import java.util.ArrayList;
import java.util.List;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import com.indicee.api.objects.dashboard.Dashboard;
import com.indicee.api.objects.dashboard.DashboardMetadata;
import com.indicee.api.objects.dashboard.CachingMetadata;
import com.indicee.api.util.XmlHelper;

public class DashboardParser extends IndiceeParser {
	
	public static Dashboard createDashboard(Document dashDoc){
		Element dashboard = XmlHelper.getElements(dashDoc.getElementsByTagName("dashboard"), "dashboard").get(0);
		return createDashboard(dashboard);
	}

	
	public static List<Dashboard> createDashboardList(Document doc){
		List<Dashboard> dashboardsList = new ArrayList<Dashboard>();
		List<Element> dashboards = XmlHelper.getElements(doc.getElementsByTagName("dashboard"), "dashboard");
		for(Element dashboard : dashboards){
			dashboardsList.add(createDashboard(dashboard));
		}
		return dashboardsList;
	}
	
	private static Dashboard createDashboard(Element dashboard) {
		Dashboard dash= new Dashboard();
		dash.setId(Long.parseLong(XmlHelper.childText(dashboard, "id")));
		dash.setQname(XmlHelper.childText(dashboard, "qname"));
		dash.setName(XmlHelper.childText(dashboard, "name"));
		dash.setOwnerId(Long.parseLong(XmlHelper.childText(dashboard, "owner_id")));
		dash.setDescription(XmlHelper.childText(dashboard, "description"));
		dash.setDefinition(XmlHelper.childText(dashboard, "definition"));
		return dash;
	}


	public static DashboardMetadata createDashboardMetadata(Document doc) {
		Element metadataElement = doc.getDocumentElement();
		String qname = XmlHelper.childText(metadataElement,"qname");
		
		Element cachingMetadataElement = XmlHelper.child(metadataElement, "caching_metadata");
		CachingMetadata cachingMetadata = createCachingMetadata(cachingMetadataElement);
		
		return new DashboardMetadata(qname, cachingMetadata);
	}


	public static CachingMetadata createCachingMetadata(
			Element cachingMetadataElement) {
		Integer ttl = Integer.parseInt(XmlHelper.childText(cachingMetadataElement, "ttl"));
		Integer validityTime = Integer.parseInt(XmlHelper.childText(cachingMetadataElement, "validity_time"));
		String scheduleCron = XmlHelper.childText(cachingMetadataElement, "schedule_cron");
		Boolean isEnabled = Boolean.parseBoolean(XmlHelper.childText(cachingMetadataElement, "is_enabled"));
		Boolean isPerUser = Boolean.parseBoolean(XmlHelper.childText(cachingMetadataElement, "is_per_user"));
		Boolean isRefreshOnDataChange = Boolean.parseBoolean(XmlHelper.childText(cachingMetadataElement, "is_refresh_on_data_change"));
		Boolean isRefreshPeriodically = Boolean.parseBoolean(XmlHelper.childText(cachingMetadataElement, "is_refresh_periodically"));
		
		CachingMetadata cachingMetadata = new CachingMetadata(ttl, validityTime, scheduleCron, isEnabled,
				isPerUser, isRefreshOnDataChange, isRefreshPeriodically);
		return cachingMetadata;
	}

}
