package com.indicee.api.parser;

import java.util.ArrayList;
import java.util.List;

import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import com.indicee.api.objects.group.Group;
import com.indicee.api.objects.group.Group.GroupColor;

public class GroupParser extends IndiceeParser {

	private enum GroupElement {
        Group("group"), Color("color"), Deleted("deleted"), Description("description"), ID("id"), Public("public"), Name("name"), OwnerID("owner_id");

        public static GroupElement get(String name) {
            for (GroupElement item : GroupElement.values()) {
                if (item.value.equals(name)) {
                    return item;
                }
            }
            return null;
        }

        private final String value;

        private GroupElement(String value) {
            this.value = value;
        }
    }
	
	public static Group createGroup(Document groupDocument) {
        return createGroup(groupDocument.getDocumentElement());
    }
	
	private static Group createGroup(Node node) {
        Group group = new Group();
        for (int i = 0; i < node.getChildNodes().getLength(); i++) {
            Node child = node.getChildNodes().item(i);
            GroupElement elem = GroupElement.get(child.getNodeName());
            if (elem != null) {
                switch (elem) {
                case ID:
                    group.setId(getLong(child));
                    break;
                case Name:
                    group.setName(getString(child));
                    break;
                case Deleted:
                	group.setDeleted(getBoolean(child));
                	break;
                case Public:
                	group.setIsPublic(getBoolean(child));
                	break;
                case OwnerID:
                	group.setOwnerId(getLong(child));
                	break;
                case Color:
                	group.setColor(GroupColor.values()[getInteger(child)]);
                	break;
                case Description:
                	group.setDescription(getString(child));
                	break;
                default:
                	break;
                }
            }
        }
        return group;
    }
	
	public static List<Group> createGroups(Document groupsDocument) {
        List<Group> groups = new ArrayList<Group>();
        NodeList groupNodes = groupsDocument.getElementsByTagName(GroupElement.Group.value);
        for (int i = 0; i < groupNodes.getLength(); i++) {
            groups.add(createGroup(groupNodes.item(i)));
        }
        return groups;
    }
}
