package com.indicee.api.parser;

import java.util.ArrayList;
import java.util.List;

import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import com.indicee.api.objects.identityprovider.IdentityProvider;

public class IdentityProviderParser extends IndiceeParser {

	private enum IdentityProviderElement {
        IdentityProvider("identity_provider"), Domain("domain"), ProviderURL("provider_url"), ProviderCertificate("provider_certificate");

        public static IdentityProviderElement get(String name) {
            for (IdentityProviderElement item : IdentityProviderElement.values()) {
                if (item.value.equals(name)) {
                    return item;
                }
            }
            return null;
        }

        private final String value;

        private IdentityProviderElement(String value) {
            this.value = value;
        }
    }
	
	public static IdentityProvider createIdentityProvider(Document idpDocument) {
		return createIdentityProvider(idpDocument.getDocumentElement());
	}

	private static IdentityProvider createIdentityProvider(Node node) {
		IdentityProvider idp = new IdentityProvider();
		
		for (int i = 0; i < node.getChildNodes().getLength(); i++) {
            Node child = node.getChildNodes().item(i);
            IdentityProviderElement elem = IdentityProviderElement.get(child.getNodeName());
            if (elem != null) {
                switch (elem) {
                case Domain:
                    idp.setDomain(getString(child));
                    break;
                case ProviderURL:
                    idp.setIdpUrl(getString(child));
                    break;
                case ProviderCertificate:
                    idp.setIdpCertificate(getString(child));
                    break;
                default:
                	break;
                }
            }
        }
        return idp;
	}
	
	public static IdentityProvider creaIdentityProvider(String documentString) {
		return createIdentityProvider(getDocumentFromString(documentString));
    }
	
	public static List<IdentityProvider> createIdentityProviders(Document idpDocument) {
        List<IdentityProvider> idps = new ArrayList<IdentityProvider>();
        NodeList idpNodes = idpDocument.getElementsByTagName(IdentityProviderElement.IdentityProvider.value);
        for (int i = 0; i < idpNodes.getLength(); i++) {
            idps.add(createIdentityProvider(idpNodes.item(i)));
        }
        return idps;
    }
}
