/**
 * 
 */
package com.indicee.api.parser;

import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import com.indicee.api.objects.Job;
import com.indicee.api.objects.JobStatus;

/**
 * @author thaggie
 * 
 */
public class JobParser extends IndiceeParser {

	private enum JobElement {
		Created("created"), Description("description"), Expires("expires"), Finished(
				"finished"), Id("id"), Message("message"), Owner("owner"), PercentComplete(
				"percent_complete"), Started("started"), Status("status"), Type("type");

		public static JobElement get(String name) {
			for (JobElement item : JobElement.values()) {
				if (item.value.equals(name)) {
					return item;
				}
			}
			return null;
		}

		private final String value;

		private JobElement(String value) {
			this.value = value;
		}
	}
	
    public static Job createJob(Document document) {
        return createJob(document.getDocumentElement());
    }

    public static Job createJob(Node node) {
    
    	Job job = new Job();
    	
    	NodeList childNodes = node.getChildNodes();
    	int nChildNodes = childNodes.getLength();
        for (int i = 0; i < nChildNodes; ++i) {
            Node child = childNodes.item(i);
            JobElement elem = JobElement.get(child.getNodeName());
            if (elem != null) {
                switch (elem) {
                case Created:
                	job.setCreated(getDate(child));
                	break;
                case Description:
                	job.setDescription(getString(child));
                	break;
                case Expires:
                	job.setExpires(getDate(child));
                	break;
                case Finished:
                	job.setFinished(getDate(child));
                	break;
                case Id:
                	job.setId(getString(child));
                	break;
                case Message:
                	job.setMessage(getString(child));
                	break;
                case Owner:
                	job.setOwner(getLong(child));
                	break;
                case PercentComplete:
                	job.setPercentComplete(getInteger(child));
                	break;
                case Started:
                	job.setStarted(getDate(child));
                	break;
                case Status:
                	job.setStatus(JobStatus.valueOf(getString(child)));
                	break;
                case Type:
                	job.setType(getString(child));
                	break;
                }
            }
        }
    	
    	return job;
    }
}
