package com.indicee.api.parser;

import java.util.ArrayList;
import java.util.List;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import com.indicee.api.objects.model.Attribute;
import com.indicee.api.objects.model.Attribute.Type;
import com.indicee.api.objects.model.DataType;
import com.indicee.api.objects.model.Dataset;
import com.indicee.api.objects.model.Entity;
import com.indicee.api.objects.model.Field;
import com.indicee.api.objects.model.Hierarchy;
import com.indicee.api.objects.model.Measure;
import com.indicee.api.objects.model.Model;
import com.indicee.api.objects.model.Relationship;
import com.indicee.api.objects.model.SourceField;
import com.indicee.api.objects.model.Table;
import com.indicee.api.objects.model.binding.Mapping;
import com.indicee.api.objects.model.binding.Projection;
import com.indicee.api.objects.model.binding.SourceTableBinding;

public final class ModelParser extends IndiceeParser {
	
    private enum BindingElement {
        Attributes("attributes"), Mappings("mappings"), Name("name"), Projections("projections"), QName("qname"), SourceTableBinding(
                "source_table_binding"), UUID("uuid");

        public static BindingElement get(String name) {
            for (BindingElement fieldChild : BindingElement.values()) {
                if (fieldChild.value.equals(name)) {
                    return fieldChild;
                }
            }
            return null;
        }

        private final String value;

        private BindingElement(String value) {
            this.value = value;
        }
    }

    private enum DatasetElement {
        Attributes("attributes"), Dataset("dataset"), Description("description"), Name("name"), QName("qname"), Tables(
                "tables"), UUID("uuid");

        public static DatasetElement get(String name) {
            for (DatasetElement item : DatasetElement.values()) {
                if (item.value.equals(name)) {
                    return item;
                }
            }
            return null;
        }

        private final String value;

        private DatasetElement(String value) {
            this.value = value;
        }
    }

    private enum DatasetTableElement {
        Attributes("attributes"), Name("name"), QName("qname"), RowDataFilter("row_data_filter"), RowEndFilter(
                "row_end_filter"), SourceFields("source_fields"), Table("table"), UUID("uuid");

        public static DatasetTableElement get(String name) {
            for (DatasetTableElement item : DatasetTableElement.values()) {
                if (item.value.equals(name)) {
                    return item;
                }
            }
            return null;
        }

        private final String value;

        private DatasetTableElement(String value) {
            this.value = value;
        }
    }

    private enum DatasetTableSourceFieldElement {
        Attributes("attributes"), DataFormat("data_format"), DataType("data_type"), Name("name"), QName("qname"), SourceField(
                "source_field"), UUID("uuid");

        public static DatasetTableSourceFieldElement get(String name) {
            for (DatasetTableSourceFieldElement item : DatasetTableSourceFieldElement.values()) {
                if (item.value.equals(name)) {
                    return item;
                }
            }
            return null;
        }

        private final String value;

        private DatasetTableSourceFieldElement(String value) {
            this.value = value;
        }
    }

    private enum EntityElement {
        Attributes("attributes"), caption("caption"), Entity("entity"), Field("field"), Keys("keys"), Name("name"), Properties(
                "properties"), QName("qname"), UUID("uuid");

        public static EntityElement get(String name) {
            for (EntityElement fieldChild : EntityElement.values()) {
                if (fieldChild.value.equals(name)) {
                    return fieldChild;
                }
            }
            return null;
        }

        private final String value;

        private EntityElement(String value) {
            this.value = value;
        }
    }

    private enum FieldElement {
        AggregateType("aggregate_type"), Attributes("attributes"), DataFormat("data_format"), DataType("data_type"), Field(
                "field"), Formula("formula"), Name("name"), QName("qname"), TrackingType("tracking_type"), UUID("uuid");

        public static FieldElement get(String name) {
            for (FieldElement fieldChild : FieldElement.values()) {
                if (fieldChild.value.equals(name)) {
                    return fieldChild;
                }
            }
            return null;
        }

        private final String value;

        private FieldElement(String value) {
            this.value = value;
        }
    }

    private enum MappingElement {
        Field("field"), Mapping("mapping"), Projection("projection");

        public static MappingElement get(String name) {
            for (MappingElement fieldChild : MappingElement.values()) {
                if (fieldChild.value.equals(name)) {
                    return fieldChild;
                }
            }
            return null;
        }

        private final String value;

        private MappingElement(String value) {
            this.value = value;
        }
    }

    private enum MeasureElement {
        Attributes("attributes"), Entities("entities"), Entity("entity"), Measure("measure"), Name("name"), QName(
                "qname"), UUID("uuid");

        public static MeasureElement get(String name) {
            for (MeasureElement fieldChild : MeasureElement.values()) {
                if (fieldChild.value.equals(name)) {
                    return fieldChild;
                }
            }
            return null;
        }

        private final String value;

        private MeasureElement(String value) {
            this.value = value;
        }
    }
    
    private enum HierarchyElement {
        Attributes("attributes"), Entities("entities"), Entity("entity"), Hierarchy("hierarchy"), Name("name"), QName(
                "qname"), UUID("uuid");

        public static HierarchyElement get(String name) {
            for (HierarchyElement fieldChild : HierarchyElement.values()) {
                if (fieldChild.value.equals(name)) {
                    return fieldChild;
                }
            }
            return null;
        }

        private final String value;

        private HierarchyElement(String value) {
            this.value = value;
        }
    }

    private enum ModelElement {
        Attributes("attributes"), Bindings("bindings"), Datasets("datasets"), Entities("entities"), Fields("fields"), Hierarchies(
                "hierarchies"), Measures("measures"), Model("model"), OwnerID("owner_id"), QName("qname"), SpaceID(
                "space_id"), UUID("uuid"), Version("version"), Relationships("relationships");

        public static ModelElement get(String name) {
            for (ModelElement item : ModelElement.values()) {
                if (item.value.equals(name)) {
                    return item;
                }
            }
            return null;
        }

        private final String value;

        private ModelElement(String value) {
            this.value = value;
        }
    }

    private enum ProjectionElement {
        Attributes("attributes"), DataType("data_type"), Formula("formula"), Name("name"), Projection("projection"), QName(
                "qname"), UUID("uuid");

        public static ProjectionElement get(String name) {
            for (ProjectionElement fieldChild : ProjectionElement.values()) {
                if (fieldChild.value.equals(name)) {
                    return fieldChild;
                }
            }
            return null;
        }

        private final String value;

        private ProjectionElement(String value) {
            this.value = value;
        }
    }

    public static Dataset createDataset(Document datasetDocument) {
        return getDataset(datasetDocument.getDocumentElement());
    }

    public static List<Dataset> createDatasetList(Document datasetListDocument) {
        List<Dataset> datasets = new ArrayList<Dataset>();
        for (int i = 0; i < datasetListDocument.getDocumentElement().getChildNodes().getLength(); i++) {
            Node child = datasetListDocument.getDocumentElement().getChildNodes().item(i);
            DatasetElement elem = DatasetElement.get(child.getNodeName());
            if (elem != null && elem.equals(DatasetElement.Dataset)) {
                datasets.add(getDataset(child));
            }
        }
        return datasets;
    }

    public static Table createDatasetTable(Document datasetTableDocument) {
        return getTable(datasetTableDocument.getDocumentElement());
    }

    public static List<Table> createDatasetTablesList(Document datasetTablesListDocument) {
        return getTables(datasetTablesListDocument.getDocumentElement().getChildNodes());
    }
    
    public static SourceField createDatasetTableField(Document datasetTableFieldDocument) {
        return getSourceField(datasetTableFieldDocument.getDocumentElement());
    }

    public static Hierarchy createHierarchy(Document hierarchyDocument) {
        return getHierarchy(hierarchyDocument.getDocumentElement());
    }
    
    public static List<Hierarchy> createHierarchyList(Document hierarchyListDocument) {
    	List<Hierarchy> hierarchies = new ArrayList<Hierarchy>();
        for (int i = 0; i < hierarchyListDocument.getDocumentElement().getChildNodes().getLength(); i++) {
            Node child = hierarchyListDocument.getDocumentElement().getChildNodes().item(i);
            HierarchyElement elem = HierarchyElement.get(child.getNodeName());
            if (elem != null && elem.equals(HierarchyElement.Hierarchy)) {
            	hierarchies.add(getHierarchy(child));
            }
        }
        return hierarchies;
    }
    
    public static Model createModel(Document modelDocument) {
        return getModel(modelDocument.getDocumentElement());
    }

    static List<Attribute> getAttributes(NodeList attributeNodes) {
    	int nNodes = attributeNodes.getLength();
    	List<Attribute> attributes = new ArrayList<Attribute>(nNodes);
    	for (int i=0; i<nNodes; ++i) {
    		Node node = attributeNodes.item(i);
    		if (node instanceof Element) {
    			Element element = (Element) node;
    			String name = element.getAttribute("name");
    			Type type = Attribute.Type.get(element.getAttribute("type"));
    			String value = element.getAttribute("value");
    			attributes.add(new Attribute(name, type, value));
    		}
    	}
    	return attributes;
    }

    public static SourceTableBinding getBinding(Node bindingNode) {
        SourceTableBinding binding = new SourceTableBinding();
        for (int i = 0; i < bindingNode.getChildNodes().getLength(); i++) {
            Node child = bindingNode.getChildNodes().item(i);
            BindingElement elem = BindingElement.get(child.getNodeName());
            if (elem != null) {
                switch (elem) {
                case UUID:
                    binding.setUUID(getString(child));
                    break;
                case Name:
                    binding.setName(getString(child));
                    break;
                case QName:
                    binding.setQName(getString(child));
                    break;
                case Attributes:
                    binding.setAttributes(getAttributes(child.getChildNodes()));
                    break;
                case Mappings:
                    binding.setMappings(getMappings(child.getChildNodes()));
                    break;
                case Projections:
                    binding.setProjections(getProjections(child.getChildNodes()));
                    break;
                default:
                	break;
                }
            }
        }
        return binding;
    }

    private static Dataset getDataset(Node datasetNode) {
        Dataset dataset = new Dataset();
        for (int i = 0; i < datasetNode.getChildNodes().getLength(); i++) {
            Node child = datasetNode.getChildNodes().item(i);
            DatasetElement elem = DatasetElement.get(child.getNodeName());
            if (elem != null) {
                switch (elem) {
                case UUID:
                    dataset.setUUID(getString(child));
                    break;
                case Name:
                    dataset.setName(getString(child));
                    break;
                case QName:
                    dataset.setQName(getString(child));
                    break;
                case Description:
                    dataset.setDescription(getString(child));
                    break;
                case Attributes:
                    dataset.setAttributes(getAttributes(child.getChildNodes()));
                    break;
                case Tables:
                    dataset.setTables(getTables(child.getChildNodes()));
                    break;
                default:
                	break;
                }
            }
        }
        return dataset;
    }

    public static NodeParser<Entity> entityParser = new NodeParser<Entity>(EntityElement.Entity.value) {
    	public Entity parse(Node entityNode) {
	        Entity entity = new Entity();
	        for (int i = 0; i < entityNode.getChildNodes().getLength(); i++) {
	            Node child = entityNode.getChildNodes().item(i);
	            EntityElement elem = EntityElement.get(child.getNodeName());
	            if (elem != null) {
	                switch (elem) {
	                case UUID:
	                    entity.setUUID(getString(child));
	                    break;
	                case Name:
	                    entity.setName(getString(child));
	                    break;
	                case QName:
	                    entity.setQName(getString(child));
	                    break;
	                case Attributes:
	                    entity.setAttributes(getAttributes(child.getChildNodes()));
	                    break;
	                case caption:
	                    entity.setCaption(getString(child));
	                    break;
	                case Keys:
	                    List<String> keys = new ArrayList<String>();
	                    for (int j = 0; j < child.getChildNodes().getLength(); j++) {
	                        Node keyNode = child.getChildNodes().item(j);
	                        EntityElement kElem = EntityElement.get(keyNode.getNodeName());
	                        if (kElem != null && kElem.equals(EntityElement.Field)) {
	                            keys.add(getString(keyNode));
	                        }
	                    }
	                    entity.setKeys(keys);
	                    break;
	                case Properties:
	                    List<String> properties = new ArrayList<String>();
	                    for (int j = 0; j < child.getChildNodes().getLength(); j++) {
	                        Node propertiesNode = child.getChildNodes().item(j);
	                        EntityElement pElem = EntityElement.get(propertiesNode.getNodeName());
	                        if (pElem != null && pElem.equals(EntityElement.Field)) {
	                            properties.add(getString(propertiesNode));
	                        }
	                    }
	                    entity.setProperties(properties);
	                    break;
	                default:
	                	break;
	                }
	            }
	        }
	        return entity;
	    }
	};

    public static NodeParser<Field> fieldParser = new NodeParser<Field>(FieldElement.Field.value){
    	public Field parse(Node fieldNode) {
	        Field field = new Field();
	        for (int i = 0; i < fieldNode.getChildNodes().getLength(); i++) {
	            Node child = fieldNode.getChildNodes().item(i);
	            FieldElement elem = FieldElement.get(child.getNodeName());
	            if (elem != null) {
	                switch (elem) {
	                case UUID:
	                    field.setUUID(getString(child));
	                    break;
	                case Name:
	                    field.setName(getString(child));
	                    break;
	                case QName:
	                    field.setQName(getString(child));
	                    break;
	                case Attributes:
	                    field.setAttributes(getAttributes(child.getChildNodes()));
	                    break;
	                case DataType:
	                    field.setDataType(DataType.valueOf(getString(child)));
	                    break;
	                case DataFormat:
	                    field.setDataFormat(getString(child));
	                    break;
	                case AggregateType:
	                    field.setAggregateType(Field.AggregateType.get(getString(child)));
	                    break;
	                case TrackingType:
	                    field.setTrackingType(Field.TrackingType.get(getString(child)));
	                    break;
	                case Formula:
	                    field.setFormula(getString(child));
	                    break;
	                default:
	                	break;
	                }
	            }
	        }
	        return field;
	    }
    };

    private static List<Mapping> getMappings(NodeList mappingNodes) {
        List<Mapping> mappings = new ArrayList<Mapping>();
        for (int j = 0; j < mappingNodes.getLength(); j++) {
            Node mappingNode = mappingNodes.item(j);
            MappingElement mElem = MappingElement.get(mappingNode.getNodeName());
            if (mElem != null && mElem.equals(MappingElement.Mapping)) {
                Mapping mapping = new Mapping();
                for (int i = 0; i < mappingNode.getChildNodes().getLength(); i++) {
                    Node child = mappingNode.getChildNodes().item(i);
                    MappingElement elem = MappingElement.get(child.getNodeName());
                    if (elem != null) {
                        switch (elem) {
                        case Field:
                            mapping.setField(getString(child));
                            break;
                        case Projection:
                            mapping.setProjection(getString(child));
                            break;
                        default:
                        	break;
                        }
                    }
                }
                mappings.add(mapping);
            }
        }
        return mappings;
    }

    private static Hierarchy getHierarchy(Node hierarchyNode) {
        Hierarchy hierarchy = new Hierarchy();
        for (int i = 0; i < hierarchyNode.getChildNodes().getLength(); i++) {
            Node child = hierarchyNode.getChildNodes().item(i);
            HierarchyElement elem = HierarchyElement.get(child.getNodeName());
            if (elem != null) {
                switch (elem) {
                case UUID:
                    hierarchy.setUUID(getString(child));
                    break;
                case Name:
                    hierarchy.setName(getString(child));
                    break;
                case QName:
                    hierarchy.setQName(getString(child));
                    break;
                case Attributes:
                    hierarchy.setAttributes(getAttributes(child.getChildNodes()));
                    break;
                case Entities:
                    List<String> entities = new ArrayList<String>();
                    for (int j = 0; j < child.getChildNodes().getLength(); j++) {
                        Node entityNode = child.getChildNodes().item(j);
                        HierarchyElement eElem = HierarchyElement.get(entityNode.getNodeName());
                        if (eElem != null && eElem.equals(HierarchyElement.Entity)) {
                            entities.add(getString(entityNode));
                        }
                    }
                    hierarchy.setEntities(entities);
                    break;
                default:
                	break;
                }
            }
        }
        return hierarchy;
    }
    
	public static final NodeParser<Measure> measureParser = new NodeParser<Measure>(MeasureElement.Measure.value) {
		@Override
		public Measure parse(Node measureNode) {
	        Measure measure = new Measure();
	        for (int i = 0; i < measureNode.getChildNodes().getLength(); i++) {
	            Node child = measureNode.getChildNodes().item(i);
	            MeasureElement elem = MeasureElement.get(child.getNodeName());
	            if (elem != null) {
	                switch (elem) {
	                case UUID:
	                    measure.setUUID(getString(child));
	                    break;
	                case Name:
	                    measure.setName(getString(child));
	                    break;
	                case QName:
	                    measure.setQName(getString(child));
	                    break;
	                case Attributes:
	                    measure.setAttributes(getAttributes(child.getChildNodes()));
	                    break;
	                case Entities:
	                    List<String> entities = new ArrayList<String>();
	                    for (int j = 0; j < child.getChildNodes().getLength(); j++) {
	                        Node entityNode = child.getChildNodes().item(j);
	                        MeasureElement eElem = MeasureElement.get(entityNode.getNodeName());
	                        if (eElem != null && eElem.equals(MeasureElement.Entity)) {
	                            entities.add(getString(entityNode));
	                        }
	                    }
	                    measure.setEntities(entities);
	                    break;
	                default:
	                	break;
	                }
	            }
	        }
	        return measure;
	    }
	};

    private static Model getModel(Node modelNode) {
        Model model = new Model();
        for (int i = 0; i < modelNode.getChildNodes().getLength(); i++) {
            Node child = modelNode.getChildNodes().item(i);
            ModelElement elem = ModelElement.get(child.getNodeName());
            if (elem != null) {
                switch (elem) {
                case UUID:
                    model.setUUID(getString(child));
                    break;
                case OwnerID:
                    model.setOwnerID(getInteger(child));
                    break;
                case SpaceID:
                    model.setSpaceID(getInteger(child));
                    break;
                case QName:
                    model.setQName(getString(child));
                    break;
                case Version:
                    model.setVersion(getInteger(child));
                    break;
                case Attributes:
                    model.setAttributes(getAttributes(child.getChildNodes()));
                    break;
                case Datasets:
                    List<Dataset> datasets = new ArrayList<Dataset>();
                    for (int j = 0; j < child.getChildNodes().getLength(); j++) {
                        Node datasetNode = child.getChildNodes().item(j);
                        DatasetElement dsElem = DatasetElement.get(datasetNode.getNodeName());
                        if (dsElem != null && dsElem.equals(DatasetElement.Dataset)) {
                            datasets.add(getDataset(datasetNode));
                        }
                    }
                    model.setDatasets(datasets);
                    break;
                case Fields:
                    List<Field> fields = fieldParser.parseList(child);
                    model.setFields(fields);
                    break;
                case Bindings:
                    List<SourceTableBinding> bindings = new ArrayList<SourceTableBinding>();
                    for (int j = 0; j < child.getChildNodes().getLength(); j++) {
                        Node bindingNode = child.getChildNodes().item(j);
                        BindingElement bElem = BindingElement.get(bindingNode.getNodeName());
                        if (bElem != null && bElem.equals(BindingElement.SourceTableBinding)) {
                            bindings.add(getBinding(bindingNode));
                        }
                    }
                    model.setBindings(bindings);
                    break;
                case Entities:
                	List<Entity> entities = entityParser.parseList(child);
                    model.setEntities(entities);
                    break;
                case Measures:
                	List<Measure> measures = measureParser.parseList(child);
                    model.setMeasures(measures);
                    break;
                case Hierarchies:
                	List<Hierarchy> hierarchies = new ArrayList<Hierarchy>();
                    for (int j = 0; j < child.getChildNodes().getLength(); j++) {
                        Node hierarchyNode = child.getChildNodes().item(j);
                        HierarchyElement mElem = HierarchyElement.get(hierarchyNode.getNodeName());
                        if (mElem != null && mElem.equals(HierarchyElement.Hierarchy)) {
                            hierarchies.add(getHierarchy(hierarchyNode));
                        }
                    }
                    model.setHierarchies(hierarchies);
                    break;
                case Relationships:
                	List<Relationship> relationships = RelationshipParser.parseRelationships((Element)child);
                	model.setRelationships(relationships);
                default:
                	break;
                }
            }
        }
        return model;
    }

    private static Projection getProjection(Node projectionNode) {
        Projection projection = new Projection();
        for (int i = 0; i < projectionNode.getChildNodes().getLength(); i++) {
            Node child = projectionNode.getChildNodes().item(i);
            ProjectionElement elem = ProjectionElement.get(child.getNodeName());
            if (elem != null) {
                switch (elem) {
                case UUID:
                    projection.setUUID(getString(child));
                    break;
                case Name:
                    projection.setName(getString(child));
                    break;
                case QName:
                    projection.setQName(getString(child));
                    break;
                case Attributes:
                    projection.setAttributes(getAttributes(child.getChildNodes()));
                    break;
                case Formula:
                    projection.setFormula(getString(child));
                    break;
                case DataType:
                    projection.setDataType(DataType.valueOf(getString(child)));
                    break;
                default:
                	break;
                }
            }
        }
        return projection;
    }

    private static List<Projection> getProjections(NodeList projectionNodes) {
        List<Projection> projections = new ArrayList<Projection>();
        for (int j = 0; j < projectionNodes.getLength(); j++) {
            Node projectionNode = projectionNodes.item(j);
            ProjectionElement pElem = ProjectionElement.get(projectionNode.getNodeName());
            if (pElem != null && pElem.equals(ProjectionElement.Projection)) {
                projections.add(getProjection(projectionNode));
            }
        }
        return projections;
    }

    private static List<SourceField> getSourceFields(NodeList sourceFieldNodes) {
        List<SourceField> sourceFields = new ArrayList<SourceField>();
        for (int i = 0; i < sourceFieldNodes.getLength(); i++) {
            Node sourceFieldNode = sourceFieldNodes.item(i);
            DatasetTableSourceFieldElement elem = DatasetTableSourceFieldElement.get(sourceFieldNode.getNodeName());
            if (elem != null && elem.equals(DatasetTableSourceFieldElement.SourceField)) {
                SourceField sourceField = new SourceField();
                for (int j = 0; j < sourceFieldNode.getChildNodes().getLength(); j++) {
                    Node child = sourceFieldNode.getChildNodes().item(j);
                    elem = DatasetTableSourceFieldElement.get(child.getNodeName());
                    if (elem != null) {
                        switch (elem) {
                        case UUID:
                            sourceField.setUUID(getString(child));
                            break;
                        case Name:
                            sourceField.setName(getString(child));
                            break;
                        case QName:
                            sourceField.setQName(getString(child));
                            break;
                        case Attributes:
                            sourceField.setAttributes(getAttributes(child.getChildNodes()));
                            break;
                        case DataFormat:
                            sourceField.setDataFormat(getString(child));
                            break;
                        case DataType:
                            sourceField.setDataType(DataType.valueOf(getString(child)));
                            break;
                        default:
                        	break;
                        }
                    }
                }
                sourceFields.add(sourceField);
            }
        }
        return sourceFields;
    }

    private static Table getTable(Node tableNode) {
        Table table = new Table();
        for (int j = 0; j < tableNode.getChildNodes().getLength(); j++) {
            Node child = tableNode.getChildNodes().item(j);
            DatasetTableElement elem = DatasetTableElement.get(child.getNodeName());
            if (elem != null) {
                switch (elem) {
                case UUID:
                    table.setUUID(getString(child));
                    break;
                case Name:
                    table.setName(getString(child));
                    break;
                case QName:
                    table.setQName(getString(child));
                    break;
                case SourceFields:
                    table.setSourceFields(getSourceFields(child.getChildNodes()));
                    break;
                case Attributes:
                    table.setAttributes(getAttributes(child.getChildNodes()));
                    break;
                case RowDataFilter:
                    table.setRowDataFilter(getBoolean(child));
                    break;
                case RowEndFilter:
                    table.setRowEndFilter(getBoolean(child));
                    break;
                default:
                	break;
                }
            }
        }
        return table;
    }

    private static List<Table> getTables(NodeList tableNodes) {
        List<Table> tables = new ArrayList<Table>();
        for (int i = 0; i < tableNodes.getLength(); i++) {
            Node tableNode = tableNodes.item(i);
            DatasetTableElement elem = DatasetTableElement.get(tableNode.getNodeName());
            if (elem != null && elem.equals(DatasetTableElement.Table)) {
                tables.add(getTable(tableNode));
            }
        }
        return tables;
    }
    
    public static SourceField getSourceField(Node fieldNode) {
        SourceField sourceField = new SourceField();
        for (int j = 0; j < fieldNode.getChildNodes().getLength(); j++) {
            Node child = fieldNode.getChildNodes().item(j);
            DatasetTableSourceFieldElement elem = DatasetTableSourceFieldElement.get(child.getNodeName());
            if (elem != null) {
                switch (elem) {
                case UUID:
                    sourceField.setUUID(getString(child));
                    break;
                case Name:
                    sourceField.setName(getString(child));
                    break;
                case QName:
                    sourceField.setQName(getString(child));
                    break;
                case DataFormat:
                    sourceField.setDataFormat(getString(child));
                    break;
                case DataType:
                    sourceField.setDataType(DataType.valueOf(getString(child)));
                    break;
                case Attributes:
                    sourceField.setAttributes(getAttributes(child.getChildNodes()));
                    break;
                default:
                	break;
                }
            }
        }
        return sourceField;
    }
}
