package com.indicee.api.parser;

import java.util.ArrayList;
import java.util.List;

import org.w3c.dom.Element;

import com.indicee.api.objects.model.DataType;
import com.indicee.api.objects.model.binding.Projection;
import com.indicee.api.util.XmlHelper;

public class ProjectionParser extends IndiceeParser {

	private enum Tag {
		uuid,
		name,
		qname,
		attributes,
		formula,
		data_type
	}
	
	public static Projection parse(Element element) {
		Projection projection = new Projection();
		
		List<Element> children = XmlHelper.childElements(element);
		for (Element child : children) {
			Tag tag = Tag.valueOf(child.getTagName());
			switch (tag) {
			case uuid:
				projection.setUUID(child.getTextContent());
				break;
			case name:
				projection.setName(child.getTextContent());
				break;
			case qname:
				projection.setQName(child.getTextContent());
				break;
			case attributes:
				projection.setAttributes(AttributeParser.parseList(child));
				break;
			case formula:
				projection.setFormula(child.getTextContent());
				break;
			case data_type:
				projection.setDataType(DataType.valueOf(child.getTextContent()));
				break;
			}
		}
		return projection;
	}
	
	public static List<Projection> parseList(Element element) {
		List<Element> children = XmlHelper.childElements(element);
		List<Projection> projections = new ArrayList<Projection>(children.size());
		for (Element child : children) {
			Projection projection = parse(child);
				projections.add(projection);
		}
		
		return projections;
	}
}
