package com.indicee.api.parser;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import com.indicee.api.objects.dashboard.CachingMetadata;
import com.indicee.api.objects.report.Collection;
import com.indicee.api.objects.report.Column;
import com.indicee.api.objects.report.Metadata;
import com.indicee.api.objects.report.Metadatas;
import com.indicee.api.objects.report.Property;
import com.indicee.api.objects.report.Report;
import com.indicee.api.objects.report.VisualizationOptions;
import com.indicee.api.objects.report.axes.Axes;
import com.indicee.api.objects.report.axes.Axis;
import com.indicee.api.objects.report.axes.AxisColumn;
import com.indicee.api.objects.report.data.Data;
import com.indicee.api.objects.report.data.DataRow;
import com.indicee.api.objects.report.view.View;
import com.indicee.api.objects.report.view.attributes.Attribute;
import com.indicee.api.objects.report.view.attributes.AttributesOrigin;
import com.indicee.api.objects.report.view.axes.AxisSort;
import com.indicee.api.objects.report.view.axes.GroupFilter;
import com.indicee.api.objects.report.view.axes.GroupingAxisLayer;
import com.indicee.api.objects.report.view.axes.Layer;
import com.indicee.api.objects.report.view.axes.LayerGroupSorting;
import com.indicee.api.objects.report.view.axes.LayerQueryGrouping;
import com.indicee.api.objects.report.view.axes.QueryValue;
import com.indicee.api.objects.report.view.axes.QueryValueModel;
import com.indicee.api.objects.report.view.axes.QueryValueReport;
import com.indicee.api.objects.report.view.axes.ValuesAxislayer;
import com.indicee.api.objects.report.view.axes.ViewAxis;
import com.indicee.api.objects.report.view.filter.EntityKeysFilter;
import com.indicee.api.objects.report.view.filter.FieldFilter;
import com.indicee.api.objects.report.view.filter.FieldFilterValue;
import com.indicee.api.objects.report.view.filter.FieldValue;
import com.indicee.api.objects.report.view.filter.Filter;
import com.indicee.api.objects.report.view.filter.RelativeTimePeriod;
import com.indicee.api.objects.report.view.filter.RelativeTimePeriodFilter;
import com.indicee.api.util.XmlHelper;

public final class ReportParser extends IndiceeParser {

    private enum AxisElement {
        Axis("axis"), Column("column"), Name("name"), SubColumns("sub"), Value("value"), Properties("properties");

        public static AxisElement get(String name) {
            for (AxisElement item : AxisElement.values()) {
                if (item.value.equals(name)) {
                    return item;
                }
            }
            return null;
        }

        private final String value;

        private AxisElement(String value) {
            this.value = value;
        }
    }
    private enum ColumnElement {
        Column("column"), Name("name"), SubColumns("sub"), Value("value");

        public static ColumnElement get(String name) {
            for (ColumnElement item : ColumnElement.values()) {
                if (item.value.equals(name)) {
                    return item;
                }
            }
            return null;
        }

        private final String value;

        private ColumnElement(String value) {
            this.value = value;
        }
    }

    private enum MetadataElement {
        Groups("groups"), Measures("measures");

        public static MetadataElement get(String name) {
            for (MetadataElement item : MetadataElement.values()) {
                if (item.value.equals(name)) {
                    return item;
                }
            }
            return null;
        }

        private final String value;

        private MetadataElement(String value) {
            this.value = value;
        }
    }

    private enum ReportAttributesElement {
        Attribute("attribute"), Origin("origin");

        public static ReportAttributesElement get(String name) {
            for (ReportAttributesElement item : ReportAttributesElement.values()) {
                if (item.value.equals(name)) {
                    return item;
                }
            }
            return null;
        }

        private final String value;

        private ReportAttributesElement(String value) {
            this.value = value;
        }
    }

    private enum ReportDataElement {
        Cell("cell"), Row("row");

        private final String value;

        private ReportDataElement(String value) {
            this.value = value;
        }
    }

    private enum ReportElement {
        Axes("axes"), CreatedDateTime("created_datetime"), Data("data"), Deleted("deleted"), Header("header"), ID("id"), Metadata(
                "metadata"), Name("name"), OwnerID("owner_id"), QName("qname"), Report("report"), CachingMetadata("caching_metadata"), View("view"), Visualization(
                "vis"), TimeToGenerate("ms_to_generate"), Generated("generated_on");

        public static ReportElement get(String name) {
            for (ReportElement item : ReportElement.values()) {
                if (item.value.equals(name)) {
                    return item;
                }
            }
            return null;
        }

        private final String value;

        private ReportElement(String value) {
            this.value = value;
        }
    }

    private enum ReportFieldFilterElement {
        FieldFilter("fieldfilter"), FieldFilterValue("value"), FieldFilterValues("values"), RelativeTimePeriod(
                "relativetimeperiod"), RelativeTimePeriodFilter("relativetimeperiodfilter"), RelativeTimePeriods(
                "relativetimeperiods"), Not("not"), EntityKeysFilter("entitykeysfilter");

        public static ReportFieldFilterElement get(String name) {
            for (ReportFieldFilterElement item : ReportFieldFilterElement.values()) {
                if (item.value.equals(name)) {
                    return item;
                }
            }
            return null;
        }

        private final String value;

        private ReportFieldFilterElement(String value) {
            this.value = value;
        }
    }

    private enum ReportViewAxisElement {
        Attributes("attributes"), Axis("axis"), AxisSort("axissort"), AxisSorting("axissorting"), ComparatorAttributes(
                "comparatorattributes"), GroupFilter("groupfilter"), GroupFilters("groupfilters"), GroupingAxisLayer(
                "groupingaxislayer"), GroupSorting("groupsorting"), Layers("layers"), Properties("properties"), QueryGrouping(
                "querygrouping"), QueryValue("queryvalue"), Values("values"), ValuesAxisLayer("valuesaxislayer");

        public static ReportViewAxisElement get(String name) {
            for (ReportViewAxisElement item : ReportViewAxisElement.values()) {
                if (item.value.equals(name)) {
                    return item;
                }
            }
            return null;
        }

        private final String value;

        private ReportViewAxisElement(String value) {
            this.value = value;
        }
    }

    private enum ReportViewElement {
        Attributes("attributes"), Axes("axes"), Collections("collections"), PreferredJoinEntities("preferredJoinEntities"), FieldFilters("fieldfilters");

        public static ReportViewElement get(String name) {
            for (ReportViewElement item : ReportViewElement.values()) {
                if (item.value.equals(name)) {
                    return item;
                }
            }
            return null;
        }

        private final String value;

        private ReportViewElement(String value) {
            this.value = value;
        }
    }

    private enum ReportVisualizationElement {
        ChartType("chart_type"), Options("options");

        public static ReportVisualizationElement get(String name) {
            for (ReportVisualizationElement item : ReportVisualizationElement.values()) {
                if (item.value.equals(name)) {
                    return item;
                }
            }
            return null;
        }

        private final String value;

        private ReportVisualizationElement(String value) {
            this.value = value;
        }
    }
    public static Report createReport(Document reportDocument) {
        return getReport(reportDocument.getDocumentElement());
    }

    public static Report createReport(String reportDocument) {
        return createReport(getDocumentFromString(reportDocument));
    }

    public static Axes createReportAxes(Document reportAxesDocument) {
        return getReportAxes(reportAxesDocument.getElementsByTagName(ReportElement.Axes.value).item(0));
    }
    public static Data createReportData(Document reportDataDocument) {
        return getData((Element)reportDataDocument.getElementsByTagName(ReportElement.Data.value).item(0));
    }
    public static Date createReportGeneratedOn(Document reportDataDocument) {
        return getDate((Element)reportDataDocument.getElementsByTagName(ReportElement.Generated.value).item(0));
    }
    public static long createReportTimeToGenerate(Document reportDataDocument) {
        return getLong((Element)reportDataDocument.getElementsByTagName(ReportElement.TimeToGenerate.value).item(0));
    }

    
    
    public static Data createReportData(String reportDataString) {
        return createReportData(getDocumentFromString(reportDataString));
    }

    public static List<Column> createReportHeader(Document headerDocument) {
        return getColumns(headerDocument.getElementsByTagName(ReportElement.Header.value).item(0));
    }

    public static Metadatas createReportMetadatas(Document reportMetadatasDocument) {
        return getMetadatas(reportMetadatasDocument.getElementsByTagName(ReportElement.Metadata.value).item(0));
    }

    public static List<Report> createReports(Document reportsDocument) {
        List<Report> reports = new ArrayList<Report>();
        for (int i = 0; i < reportsDocument.getDocumentElement().getChildNodes().getLength(); i++) {
            Node child = reportsDocument.getDocumentElement().getChildNodes().item(i);
            ReportElement elem = ReportElement.get(child.getNodeName());
            if (elem != null && elem.equals(ReportElement.Report)) {
                reports.add(getReport(child));
            }
        }
        return reports;
    }

    public static List<Report> createReports(String reportsDocument) {
        return createReports(getDocumentFromString(reportsDocument));
    }

    public static View createReportView(Document reportViewDocument) {
        return getView(reportViewDocument.getElementsByTagName(ReportElement.View.value).item(0));
    }

    public static View createReportView(String reportViewDocument) {
        return createReportView(getDocumentFromString(reportViewDocument));
    }

    private static Attribute getAttribute(Node attributeNode) {
        Attribute attribute = new Attribute();
        NamedNodeMap entryAttributes = attributeNode.getAttributes();
        attribute.setName(entryAttributes.getNamedItem("name").getNodeValue());
        String type = entryAttributes.getNamedItem("type").getNodeValue();
        attribute.setType(type);
        if ("stringList".equals(type)) {
        	ArrayList<String> valueList = new ArrayList<String>(); 
        	NodeList childNodes = attributeNode.getChildNodes();
        	for (int i=0;i<childNodes.getLength();++i) {
        		Node childNode = childNodes.item(i);
        		String nodeName = childNode.getNodeName();
        		if ("item".equals(nodeName)) {
        			String value = childNode.getAttributes().getNamedItem("value").getNodeValue();
        			valueList.add(value);
        		}
        	}
        	attribute.setValueList(valueList);
        } else {
        	attribute.setValue(entryAttributes.getNamedItem("value").getNodeValue());
        }
        return attribute;
    }

    private static List<ViewAxis> getAxes(Node axesNode) {
        List<ViewAxis> axes = new ArrayList<ViewAxis>();
        for (int i = 0; i < axesNode.getChildNodes().getLength(); i++) {
            Node axisNode = axesNode.getChildNodes().item(i);
            ReportViewAxisElement elem = ReportViewAxisElement.get(axisNode.getNodeName());
            if (elem != null && elem.equals(ReportViewAxisElement.Axis)) {
                ViewAxis axis = new ViewAxis();
                NamedNodeMap entryAttributes = axisNode.getAttributes();
                if (entryAttributes.getNamedItem("name") != null) {
                    axis.setName(entryAttributes.getNamedItem("name").getNodeValue());
                }

                for (int j = 0; j < axisNode.getChildNodes().getLength(); j++) {
                    Node child = axisNode.getChildNodes().item(j);
                    elem = ReportViewAxisElement.get(child.getNodeName());
                    if (elem != null) {
                        switch (elem) {
                        case Layers:
                            axis.setLayers(getLayers(child));
                            break;
                        case AxisSorting:
                            axis.setAxisSortings(getAxisSortings(child));
                            break;
                        default:
                        	break;
                        }
                    }
                }

                axes.add(axis);
            }
        }
        return axes;
    }

    static AxisColumn getAxisColumn(Node axisNode) {
        AxisColumn column = new AxisColumn();
        NamedNodeMap entryAttributes = axisNode.getAttributes();
        column.setType(entryAttributes.getNamedItem("type").getNodeValue());
        for (int i = 0; i < axisNode.getChildNodes().getLength(); i++) {
            Node child = axisNode.getChildNodes().item(i);
            AxisElement elem = AxisElement.get(child.getNodeName());
            if (elem != null) {
                switch (elem) {
                case Name:
                    column.setName(getString(child));
                    break;
                case Value:
                    column.setValue(getString(child));
                    break;
                case SubColumns:
                    column.setSubColumns(getAxisColumns(child));
                    break;
                case Properties:
                	column.setProperties(getProperties(child));
                	break;
                default:
                	break;
                }
            }
        }
        return column;
    }

    private static List<AxisColumn> getAxisColumns(Node child) {
        List<AxisColumn> columns = new ArrayList<AxisColumn>();
        for (int j = 0; j < child.getChildNodes().getLength(); j++) {
            Node columnNode = child.getChildNodes().item(j);
            AxisElement elem = AxisElement.get(columnNode.getNodeName());
            if (elem != null && elem.equals(AxisElement.Column)) {
                columns.add(getAxisColumn(columnNode));
            }
        }
        return columns;
    }

    private static List<AxisSort> getAxisSortings(Node axisSortingsNode) {
        List<AxisSort> axisSortings = new ArrayList<AxisSort>();
        for (int i = 0; i < axisSortingsNode.getChildNodes().getLength(); i++) {
            Node axisSortNode = axisSortingsNode.getChildNodes().item(i);
            ReportViewAxisElement elem = ReportViewAxisElement.get(axisSortNode.getNodeName());
            if (elem != null && elem.equals(ReportViewAxisElement.AxisSort)) {
                AxisSort axisSort = new AxisSort();
                NamedNodeMap entryAttributes = axisSortNode.getAttributes();
                axisSort.setAscending(Boolean.parseBoolean(entryAttributes.getNamedItem("ascending").getNodeValue()));
                axisSort.setBreakHierarchies(Boolean.parseBoolean(entryAttributes.getNamedItem("breakhierarchies")
                        .getNodeValue()));

                for (int j = 0; j < axisSortNode.getChildNodes().getLength(); j++) {
                    Node child = axisSortNode.getChildNodes().item(j);
                    elem = ReportViewAxisElement.get(child.getNodeName());
                    if (elem != null) {
                        switch (elem) {
                        case QueryValue:
                            axisSort.setQueryValue(getQueryValue(child));
                            break;
                        case Attributes:
                            axisSort.setAttributes(getOriginAttributes(child));
                            break;
                        default:
                        	break;
                        }
                    }
                }

                axisSortings.add(axisSort);
            }
        }
        return axisSortings;
    }

    private static List<Collection> getCollections(Node collectionsNode) {
        List<Collection> collections = new ArrayList<Collection>();
        for (int i = 0; i < collectionsNode.getChildNodes().getLength(); i++) {
            NamedNodeMap entryAttributes = collectionsNode.getChildNodes().item(i).getAttributes();
            if (entryAttributes != null) {
                Collection collection = new Collection(entryAttributes.getNamedItem("name").getNodeValue());
                collections.add(collection);
            }
        }
        return collections;
    }

    private static Column getColumn(Node columnNode) {
        Column column = new Column();
        for (int i = 0; i < columnNode.getChildNodes().getLength(); i++) {
            Node child = columnNode.getChildNodes().item(i);
            ColumnElement elem = ColumnElement.get(child.getNodeName());
            if (elem != null) {
                switch (elem) {
                case Name:
                    column.setName(getString(child));
                    break;
                case Value:
                    column.setValue(getString(child));
                    break;
                case SubColumns:
                    column.setSubColumns(getColumns(child));
                    break;
                default:
                	break;
                }
            }
        }
        return column;
    }
    
    private static List<Property> getProperties(Node child) {
    	
    	List<Element> elements = XmlHelper.getElements(child.getChildNodes(), "property");
    	List<Property> properties = new ArrayList<Property>(elements.size());
    	
    	for (Element element : elements) {
    		String name = element.getAttribute("name");
    		String value = XmlHelper.childText(element, "value");
    		properties.add(new Property(name, value));
    	}
    	
    	return properties;
    }

    private static List<Column> getColumns(Node child) {
        List<Column> columns = new ArrayList<Column>();
        for (int j = 0; j < child.getChildNodes().getLength(); j++) {
            Node columnNode = child.getChildNodes().item(j);
            ColumnElement columnElem = ColumnElement.get(columnNode.getNodeName());
            if (columnElem != null && columnElem.equals(ColumnElement.Column)) {
                columns.add(getColumn(columnNode));
            }
        }
        return columns;
    }

    private static List<String> getCells(Element parent) {
		List<Element> cellElements = XmlHelper.childElements(parent, ReportDataElement.Cell.value);
		List<String> cellValues = new ArrayList<String>(cellElements.size());
		for (Element cellElement : cellElements) {
			String textValue = cellElement.getTextContent();
			cellValues.add(textValue);
		}
		return cellValues;
    }
    
    private static Data getData(Element dataElement) {
    	Data reportData = new Data();
    	String cols = dataElement.getAttribute("cols");
    	if (cols != null && cols.length() > 0) {
    		reportData.setTotalColumns(Integer.parseInt(cols));
    	}
    	String rows = dataElement.getAttribute("rows");
    	if (rows != null && rows.length() > 0) {
    		reportData.setTotalRows(Integer.parseInt(rows));
    	}
    	List<Element> rowElements = XmlHelper.childElements(dataElement, ReportDataElement.Row.value);
    	List<DataRow> dataRows = new ArrayList<DataRow>(rowElements.size());
    	for (Element rowElement : rowElements) {
    		List<String> cellValues = getCells(rowElement);
    		DataRow reportDataRow = new DataRow();
    		reportDataRow.setCells(cellValues);
    		dataRows.add(reportDataRow);
    	}
    	
    	if (!dataRows.isEmpty()) {
    		reportData.setReportDataRows(dataRows);
    	} else {
    		List<String> cellValues = getCells(dataElement);
    		reportData.setReportDataCells(cellValues);
    	}
    	
    	return reportData;
    }

    private static FieldFilter getFieldFilter(Node fieldFilterNode) {
        FieldFilter fieldFilter = new FieldFilter();
        NamedNodeMap entryAttributes = fieldFilterNode.getAttributes();
        fieldFilter.setField(entryAttributes.getNamedItem("field").getNodeValue());

        fieldFilter.setInclusive(testInclusive(entryAttributes));
        
        if (entryAttributes.getNamedItem("entity") != null) {
            fieldFilter.setEntity(entryAttributes.getNamedItem("entity").getNodeValue());
        }
        if (entryAttributes.getNamedItem("value") != null) {
            List<FieldFilterValue> values = new ArrayList<FieldFilterValue>();
            FieldFilterValue value = new FieldFilterValue();
            value.setValue(entryAttributes.getNamedItem("value").getNodeValue());
            values.add(value);
            fieldFilter.setValues(values);
        } else {
            for (int i = 0; i < fieldFilterNode.getChildNodes().getLength(); i++) {
                Node fieldFilterValuesNode = fieldFilterNode.getChildNodes().item(i);
                ReportFieldFilterElement elem = ReportFieldFilterElement.get(fieldFilterValuesNode.getNodeName());
                if (elem != null && elem.equals(ReportFieldFilterElement.FieldFilterValues)) {
                    List<FieldFilterValue> values = new ArrayList<FieldFilterValue>();
                    for (int j = 0; j < fieldFilterValuesNode.getChildNodes().getLength(); j++) {
                        Node fieldFilterValueNode = fieldFilterValuesNode.getChildNodes().item(j);
                        elem = ReportFieldFilterElement.get(fieldFilterValueNode.getNodeName());
                        if (elem != null && elem.equals(ReportFieldFilterElement.FieldFilterValue)) {
                            FieldFilterValue value = new FieldFilterValue();
                            entryAttributes = fieldFilterValueNode.getAttributes();
                            value.setValue(entryAttributes.getNamedItem("value").getNodeValue());
                            values.add(value);
                        }
                    }
                    fieldFilter.setValues(values);
                    //break;
                } else if ("attributes".equals(fieldFilterValuesNode.getNodeName())) {
                	ArrayList<AttributesOrigin> attributes = getOriginAttributes(fieldFilterValuesNode);
                	fieldFilter.setAttributeOrigins(attributes);
                }
            }
        }
        return fieldFilter;
    }

	private static boolean testInclusive(NamedNodeMap entryAttributes) {
		Node attributeNode = entryAttributes.getNamedItem("inclusive");
		if (attributeNode != null) {
			String nodeValue = attributeNode.getNodeValue();
			if (nodeValue != null && nodeValue.length() > 0) {
				return Boolean.parseBoolean(nodeValue);
			}
		} 
		
		return true;
	}

    static List<Filter> getFieldFilters(Node fieldFiltersNode) {
        List<Filter> fieldFilters = new ArrayList<Filter>();
        for (int i = 0; i < fieldFiltersNode.getChildNodes().getLength(); i++) {
            Node child = fieldFiltersNode.getChildNodes().item(i);
            ReportFieldFilterElement elem = ReportFieldFilterElement.get(child.getNodeName());
            if (elem != null) {
                switch (elem) {
                case FieldFilter:
                    fieldFilters.add(getFieldFilter(child));
                    break;
                case RelativeTimePeriodFilter:
                    fieldFilters.add(getRelativeTimePeriodFilter(child));
                    break;
                case Not:
                	for(Filter filter: getFieldFilters(child)) {
                		filter.setNot(true);
                		fieldFilters.add(filter);
                	}
                	break;
                case EntityKeysFilter:
                	fieldFilters.add(getEntityKeysFilter(child));
                	break;
                default:
                	break;
                }
            }
        }
        return fieldFilters;
    }

    private static EntityKeysFilter getEntityKeysFilter(Node child) {
    	
    	NamedNodeMap attributes = child.getAttributes();
    	String entity = attributes.getNamedItem("entity").getNodeValue();
    	ArrayList<ArrayList<FieldValue>> keyValueSets = new ArrayList<ArrayList<FieldValue>>();
    	ArrayList<AttributesOrigin> originAttributes = null;
    	NodeList childNodes = child.getChildNodes();
    	for (int i = 0; i < childNodes.getLength(); ++i) {
    		Node childNode = childNodes.item(i);
    		String nodeName = childNode.getNodeName();
    		if ("keyvalueset".equals(nodeName)) {
    			ArrayList<FieldValue> keyValueSet = getKeyValueSet(childNode);
    			
    			keyValueSets.add(keyValueSet);
    		} else if ("attributes".equals(nodeName)) {
    			originAttributes = getOriginAttributes(childNode);
    		}
    	}

		return new EntityKeysFilter(entity, keyValueSets, originAttributes);
	}

	private static ArrayList<FieldValue> getKeyValueSet(Node child) {
		
		ArrayList<FieldValue> keyValueSet = new ArrayList<FieldValue>();

    	NodeList childNodes = child.getChildNodes();
    	for (int i = 0; i < childNodes.getLength(); ++i) {
    		Node childNode = childNodes.item(i);
    		String nodeName = childNode.getNodeName();
    		if ("fieldvalue".equals(nodeName)) {
    			FieldValue fieldValue = getFieldValue(childNode);
    			keyValueSet.add(fieldValue);
    		}
    	}
		return keyValueSet;
	}

	private static FieldValue getFieldValue(Node childNode) {
    	NamedNodeMap attributes = childNode.getAttributes();
    	String field = attributes.getNamedItem("field").getNodeValue();
    	String value = attributes.getNamedItem("value").getNodeValue();
		return new FieldValue(field, value);
	}

	private static List<GroupFilter> getGroupFilters(Node groupFiltersNode) {
        List<GroupFilter> groupFilters = new ArrayList<GroupFilter>();
        for (int i = 0; i < groupFiltersNode.getChildNodes().getLength(); i++) {
            Node groupFilterNode = groupFiltersNode.getChildNodes().item(i);
            ReportViewAxisElement elem = ReportViewAxisElement.get(groupFilterNode.getNodeName());
            if (elem != null && elem.equals(ReportViewAxisElement.GroupFilter)) {
                GroupFilter groupFilter = new GroupFilter();
                NamedNodeMap entryAttributes = groupFilterNode.getAttributes();
                groupFilter.setComparator(entryAttributes.getNamedItem("comparator").getNodeValue());

                for (int j = 0; j < groupFilterNode.getChildNodes().getLength(); j++) {
                    Node child = groupFilterNode.getChildNodes().item(j);
                    elem = ReportViewAxisElement.get(child.getNodeName());
                    if (elem != null) {
                        switch (elem) {
                        case QueryValue:
                            groupFilter.setQueryValue(getQueryValue(child));
                            break;
                        case ComparatorAttributes:
                            List<Attribute> attributes = new ArrayList<Attribute>();
                            for (int k = 0; k < child.getChildNodes().getLength(); k++) {
                                Node attributeNode = child.getChildNodes().item(k);
                                ReportAttributesElement attributeElem = ReportAttributesElement.get(attributeNode
                                        .getNodeName());
                                if (attributeElem != null && attributeElem.equals(ReportAttributesElement.Attribute)) {
                                    attributes.add(getAttribute(attributeNode));
                                }
                            }
                            groupFilter.setComparatorAttributes(attributes);
                            break;
                        default:
                        	break;
                        }
                    }
                }

                groupFilters.add(groupFilter);
            }
        }
        return groupFilters;
    }

    private static LayerGroupSorting getGroupSorting(Node groupSortingNode) {
        LayerGroupSorting groupSorting = new LayerGroupSorting();
        NamedNodeMap entryAttributes = groupSortingNode.getAttributes();
        if (entryAttributes.getNamedItem("ascending") != null) {
            groupSorting.setAscending(Boolean.parseBoolean(entryAttributes.getNamedItem("ascending").getNodeValue()));
        }
        if (entryAttributes.getNamedItem("sortbycaption") != null) {
            groupSorting.setSortByCaption(Boolean.parseBoolean(entryAttributes.getNamedItem("sortbycaption")
                    .getNodeValue()));
        }
        return groupSorting;
    }

    private static List<Layer> getLayers(Node layersNode) {
        List<Layer> layers = new ArrayList<Layer>();
        for (int i = 0; i < layersNode.getChildNodes().getLength(); i++) {
            Node layerNode = layersNode.getChildNodes().item(i);
            ReportViewAxisElement elem = ReportViewAxisElement.get(layerNode.getNodeName());
            if (elem != null) {
                switch (elem) {
                case GroupingAxisLayer:
                    GroupingAxisLayer groupingAxisLayer = new GroupingAxisLayer();
                    NamedNodeMap entryAttributes = layerNode.getAttributes();
                    if (entryAttributes.getNamedItem("leadingtotals") != null) {
                        groupingAxisLayer.setLeadingTotals(Boolean.parseBoolean(entryAttributes.getNamedItem(
                                "leadingtotals").getNodeValue()));
                    }
                    if (entryAttributes.getNamedItem("trailingtotals") != null) {
                        groupingAxisLayer.setTrailingTotals(Boolean.parseBoolean(entryAttributes.getNamedItem(
                                "trailingtotals").getNodeValue()));
                    }
                    if (entryAttributes.getNamedItem("hierarchy") != null) {
                        groupingAxisLayer.setHierarchy(entryAttributes.getNamedItem("hierarchy").getNodeValue());
                    }
                    if (entryAttributes.getNamedItem("hierarchydisplay") != null) {
                        groupingAxisLayer.setHierarchyDisplay(entryAttributes.getNamedItem("hierarchydisplay")
                                .getNodeValue());
                    }
                    if (entryAttributes.getNamedItem("drillable") != null) {
                        groupingAxisLayer.setDrillable(Boolean.parseBoolean(entryAttributes.getNamedItem("drillable")
                                .getNodeValue()));
                    }

                    for (int j = 0; j < layerNode.getChildNodes().getLength(); j++) {
                        Node child = layerNode.getChildNodes().item(j);
                        elem = ReportViewAxisElement.get(child.getNodeName());
                        if (elem != null) {
                            switch (elem) {
                            case QueryGrouping:
                                groupingAxisLayer.setQueryGrouping(getQueryGrouping(child));
                                break;
                            case GroupSorting:
                                groupingAxisLayer.setGroupSorting(getGroupSorting(child));
                                break;
                            case GroupFilters:
                                groupingAxisLayer.setGroupFilters(getGroupFilters(child));
                                break;
                            case Attributes:
                                groupingAxisLayer.setAttributes(getOriginAttributes(child));
                                break;
                            default:
                            	break;
                            }
                        }
                    }

                    layers.add(groupingAxisLayer);
                    break;
                case ValuesAxisLayer:
                    ValuesAxislayer valuesaxislayer = new ValuesAxislayer();
                    for (int j = 0; j < layerNode.getChildNodes().getLength(); j++) {
                        Node child = layerNode.getChildNodes().item(j);
                        elem = ReportViewAxisElement.get(child.getNodeName());
                        if (elem != null && elem.equals(ReportViewAxisElement.Values)) {
                            valuesaxislayer.setValues(getQueryValues(child));
                        }
                    }
                    layers.add(valuesaxislayer);
                    break;
                default:
                	break;
                }
            }
        }
        return layers;
    }

    private static List<Metadata> getMetadata(Node metadataNode) {
        List<Metadata> metadatas = new ArrayList<Metadata>();
        for (int i = 0; i < metadataNode.getChildNodes().getLength(); i++) {
            NamedNodeMap entryAttributes = metadataNode.getChildNodes().item(i).getAttributes();
            if (entryAttributes != null) {
                Metadata reportMetadata = new Metadata(entryAttributes.getNamedItem("key").getNodeValue(),
                        entryAttributes.getNamedItem("value").getNodeValue());
                metadatas.add(reportMetadata);
            }
        }
        return metadatas;
    }

    private static Metadatas getMetadatas(Node child) {
        Metadatas metadatas = new Metadatas();
        for (int j = 0; j < child.getChildNodes().getLength(); j++) {
            Node metadataChild = child.getChildNodes().item(j);
            MetadataElement metadataElem = MetadataElement.get(metadataChild.getNodeName());
            if (metadataElem != null) {
                switch (metadataElem) {
                case Measures:
                    metadatas.setMeasures(getMetadata(metadataChild));
                    break;
                case Groups:
                    metadatas.setGroups(getMetadata(metadataChild));
                    break;
                }
            }
        }
        return metadatas;
    }

    private static AttributesOrigin getOriginAttribute(Node attribtuteOriginNode) {
        AttributesOrigin originAttribute = new AttributesOrigin();
        NamedNodeMap entryAttributes = attribtuteOriginNode.getAttributes();
        originAttribute.setName(entryAttributes.getNamedItem("name").getNodeValue());

        List<Attribute> reportsAttributes = new ArrayList<Attribute>();
        for (int i = 0; i < attribtuteOriginNode.getChildNodes().getLength(); i++) {
            Node child = attribtuteOriginNode.getChildNodes().item(i);
            ReportAttributesElement elem = ReportAttributesElement.get(child.getNodeName());
            if (elem != null && elem.equals(ReportAttributesElement.Attribute)) {
                reportsAttributes.add(getAttribute(child));
            }
        }
        originAttribute.setReportAttributes(reportsAttributes);
        return originAttribute;
    }

    private static ArrayList<AttributesOrigin> getOriginAttributes(Node attribtutesNode) {
    	ArrayList<AttributesOrigin> originAttributes = new ArrayList<AttributesOrigin>();
        for (int i = 0; i < attribtutesNode.getChildNodes().getLength(); i++) {
            Node child = attribtutesNode.getChildNodes().item(i);
            ReportAttributesElement elem = ReportAttributesElement.get(child.getNodeName());
            if (elem != null && elem.equals(ReportAttributesElement.Origin)) {
                originAttributes.add(getOriginAttribute(child));
            }
        }
        return originAttributes;
    }

    private static LayerQueryGrouping getQueryGrouping(Node queryGroupingNode) {
        LayerQueryGrouping queryGrouping = new LayerQueryGrouping();
        NamedNodeMap entryAttributes = queryGroupingNode.getAttributes();
        queryGrouping.setEntity(entryAttributes.getNamedItem("entity").getNodeValue());

        for (int i = 0; i < queryGroupingNode.getChildNodes().getLength(); i++) {
            Node child = queryGroupingNode.getChildNodes().item(i);
            ReportViewAxisElement elem = ReportViewAxisElement.get(child.getNodeName());
            if (elem != null && elem.equals(ReportViewAxisElement.Properties)) {
                queryGrouping.setProperties(getQueryValues(child));
            }
        }

        return queryGrouping;
    }

    private static QueryValue getQueryValue(Node queryValueNode) {
        NamedNodeMap entryAttributes = queryValueNode.getAttributes();
        
        Node formulaNode = entryAttributes.getNamedItem("formula");
        List<Element> formulaElements = XmlHelper.getElements(queryValueNode.getChildNodes(), "formula");
        QueryValue queryValue;
        if (formulaNode == null && (formulaElements == null || formulaElements.size() == 0)) {
        	QueryValueModel reportValue = new QueryValueModel(); 
       		reportValue.setEntity(entryAttributes.getNamedItem("entity").getNodeValue());
            if (entryAttributes.getNamedItem("field") != null) {
            	reportValue.setField(entryAttributes.getNamedItem("field").getNodeValue());
            }
            queryValue = reportValue;
        } else {
        	QueryValueReport reportValue = new QueryValueReport(); 
        	if (formulaNode != null) {
        		reportValue.setFormula(formulaNode.getNodeValue());
        	} else {
        		Element formulaElement = formulaElements.get(0);
        		reportValue.setFormula(formulaElement.getTextContent());
        	}
        	reportValue.setName(entryAttributes.getNamedItem("name").getNodeValue());
        	queryValue = reportValue;
        }
        
        if (entryAttributes.getNamedItem("aggregator") != null) {
            queryValue.setAggregator(entryAttributes.getNamedItem("aggregator").getNodeValue());
        }
        for (int j = 0; j < queryValueNode.getChildNodes().getLength(); j++) {
            Node attributesNode = queryValueNode.getChildNodes().item(j);
            ReportViewAxisElement elem = ReportViewAxisElement.get(attributesNode.getNodeName());
            if (elem != null && elem.equals(ReportViewAxisElement.Attributes)) {
                queryValue.setAttributes(getOriginAttributes(attributesNode));
                break;
            }
        }
        return queryValue;
    }

    private static List<QueryValue> getQueryValues(Node queryValuesNode) {
        List<QueryValue> queryValues = new ArrayList<QueryValue>();
        for (int i = 0; i < queryValuesNode.getChildNodes().getLength(); i++) {
            Node queryValueNode = queryValuesNode.getChildNodes().item(i);
            ReportViewAxisElement elem = ReportViewAxisElement.get(queryValueNode.getNodeName());
            if (elem != null && elem.equals(ReportViewAxisElement.QueryValue)) {
                queryValues.add(getQueryValue(queryValueNode));
            }
        }
        return queryValues;
    }

    static RelativeTimePeriodFilter getRelativeTimePeriodFilter(Node relativeTimePeriodFilterNode) {
        RelativeTimePeriodFilter relativeTimePeriodFilter = new RelativeTimePeriodFilter();
        NamedNodeMap entryAttributes = relativeTimePeriodFilterNode.getAttributes();
        relativeTimePeriodFilter.setField(entryAttributes.getNamedItem("field").getNodeValue());
        if (entryAttributes.getNamedItem("entity") != null) {
            relativeTimePeriodFilter.setEntity(entryAttributes.getNamedItem("entity").getNodeValue());
        }
        
        relativeTimePeriodFilter.setInclusive(testInclusive(entryAttributes));
        
        Node calendarNode = entryAttributes.getNamedItem("calendar");
        if (calendarNode != null) {
        	relativeTimePeriodFilter.setCalendar(calendarNode.getNodeValue());
        }

        for (int i = 0; i < relativeTimePeriodFilterNode.getChildNodes().getLength(); i++) {
            Node relativeTimePeriodsNode = relativeTimePeriodFilterNode.getChildNodes().item(i);
            String nodeName = relativeTimePeriodsNode.getNodeName();
            ReportFieldFilterElement elem = ReportFieldFilterElement.get(nodeName);
            if (elem != null && elem.equals(ReportFieldFilterElement.RelativeTimePeriods)) {
                List<RelativeTimePeriod> relativeTimePeriods = new ArrayList<RelativeTimePeriod>();
                for (int j = 0; j < relativeTimePeriodsNode.getChildNodes().getLength(); j++) {
                    Node relativeTimePeriodNode = relativeTimePeriodsNode.getChildNodes().item(j);
                    elem = ReportFieldFilterElement.get(relativeTimePeriodNode.getNodeName());
                    if (elem != null && elem.equals(ReportFieldFilterElement.RelativeTimePeriod)) {
                        RelativeTimePeriod relativeTimePeriod = new RelativeTimePeriod();
                        entryAttributes = relativeTimePeriodNode.getAttributes();
                        relativeTimePeriod.setRangeEnd(Integer.parseInt(entryAttributes.getNamedItem("rangeend")
                                .getNodeValue()));
                        relativeTimePeriod.setRangeStart(Integer.parseInt(entryAttributes.getNamedItem("rangestart")
                                .getNodeValue()));
                        relativeTimePeriod.setTimePeriod(entryAttributes.getNamedItem("timeperiod").getNodeValue());
                        if (entryAttributes.getNamedItem("referencetime") != null) {
                            relativeTimePeriod.setReferenceTime(Long.parseLong(entryAttributes.getNamedItem(
                                    "referencetime").getNodeValue()));
                        }
                        relativeTimePeriods.add(relativeTimePeriod);
                    }
                }
                relativeTimePeriodFilter.setRelativeTimePeriods(relativeTimePeriods);
            } else if ("attributes".equals(nodeName)) {
            	ArrayList<AttributesOrigin> attributes = getOriginAttributes(relativeTimePeriodsNode);
            	relativeTimePeriodFilter.setAttributeOrigins(attributes);
            }

        }
        return relativeTimePeriodFilter;
    }

    private static Report getReport(Node reportNode) {
        Report report = new Report();
        for (int i = 0; i < reportNode.getChildNodes().getLength(); i++) {
            Node child = reportNode.getChildNodes().item(i);
            ReportElement elem = ReportElement.get(child.getNodeName());
            if (elem != null) {
                switch (elem) {
                case ID:
                    report.setId(getLong(child));
                    break;
                case Name:
                    report.setName(getString(child));
                    break;
                case QName:
                    report.setQname(getString(child));
                    break;
                case OwnerID:
                    report.setOwnerId(getLong(child));
                    break;
                case Deleted:
                    report.setDeleted(getBoolean(child));
                    break;
                case CreatedDateTime:
                    report.setCreatedDate(getDate(child));
                    report.setCreatedDateString(getString(child));
                    break;
                case CachingMetadata:
                	report.setCachingMetadata(getCachingMetadata(child));
                	break;
                case Metadata:
                    report.setMetadatas(getMetadatas(child));
                    break;
                case Visualization:
                    report.setVisualizationOptions(getVisualization(child));
                    break;
                case View:
                    report.setReportView(getView(child));
                    break;
                case Data:
                    report.setReportData(getData((Element)child));
                    break;
                case Axes:
                    report.setAxes(getReportAxes(child));
                    break;
                case Header:
                    report.setColumns(getColumns(child));
                    break;
                    
                case TimeToGenerate:
                	report.setTimeToGenerate(getLong(child));
                	break;
                	
                case Generated:
                	report.setGeneratedOn(getDate(child));
                	break;
                	
                default:
                	break;
                }
            }
        }
        return report;
    }

    private static CachingMetadata getCachingMetadata(Node child) {
    	return DashboardParser.createCachingMetadata((Element)child);
	}

	private static Axes getReportAxes(Node child) {
        List<Axis> axes = new ArrayList<Axis>();
        for (int j = 0; j < child.getChildNodes().getLength(); j++) {
            Node axisNode = child.getChildNodes().item(j);
            AxisElement axisElem = AxisElement.get(axisNode.getNodeName());
            if (axisElem != null && axisElem.equals(AxisElement.Axis)) {
            	NamedNodeMap attributes = axisNode.getAttributes();
            	long extent = Long.parseLong(attributes.getNamedItem("extent").getNodeValue());
            	
                axes.add(new Axis(extent, getAxisColumns(axisNode)));
            }
        }
        return new Axes(axes);
    }

    private static View getView(Node viewNode) {
        View reportView = new View();
        for (int i = 0; i < viewNode.getChildNodes().getLength(); i++) {
            Node child = viewNode.getChildNodes().item(i);
            ReportViewElement elem = ReportViewElement.get(child.getNodeName());
            if (elem != null) {
                switch (elem) {
                case Axes:
                    reportView.setReportViewAxes(getAxes(child));
                    break;
                case FieldFilters:
                    reportView.setFieldFilters(getFieldFilters(child));
                    break;
                case Collections:
                    reportView.setCollections(getCollections(child));
                    break;
                case PreferredJoinEntities:
                	reportView.setPreferredJoinEntities(getPreferredJoinEntities(child));
                	break;
                case Attributes:
                    reportView.setAttributeOrigins(getOriginAttributes(child));
                    break;
                }
            }
        }
        return reportView;
    }

    static List<String> getPreferredJoinEntities(Node node) {
    	NodeList childNodes = node.getChildNodes();
    	int nChildren = childNodes.getLength();
    	List<String> preferredJoinEntities = new ArrayList<String>(nChildren);
    	for (int i = 0; i < nChildren; ++i) {
    		Node child = childNodes.item(i);
    		if (child instanceof Element) {
    			Element childElement = (Element)child;
    			if ("entity".equals(childElement.getTagName())) {
    				String qname = childElement.getAttribute("name");
    				preferredJoinEntities.add(qname);
    			}
    		}
    		
    	}
		return preferredJoinEntities;
	}

	private static VisualizationOptions getVisualization(Node visualizationNode) {
        VisualizationOptions visualizationOptions = new VisualizationOptions();
        for (int i = 0; i < visualizationNode.getChildNodes().getLength(); i++) {
            Node child = visualizationNode.getChildNodes().item(i);
            ReportVisualizationElement elem = ReportVisualizationElement.get(child.getNodeName());
            if (elem != null) {
                switch (elem) {
                case ChartType:
                    visualizationOptions.setChartType(getString(child));
                    break;
                case Options:
                    visualizationOptions.setOptions(getString(child));
                    break;
                }
            }
        }
        return visualizationOptions;
    }
}
