package com.indicee.api.services.impl;

import java.util.HashMap;
import java.util.Map;

import org.w3c.dom.Document;

import com.indicee.api.connector.IndiceeAPIConnector;
import com.indicee.api.connector.exception.IndiceeAPIConnectorException;
import com.indicee.api.parser.ApplicationDataParser;
import com.indicee.api.parser.IndiceeParser.NodeParser;
import com.indicee.api.services.interfaces.ApplicationService;

public class ApplicationServiceImpl extends IndiceeAPIService implements ApplicationService {

    private static final String APP_URL = API_URL + "/apps";
    private static final String DATA_URL = APP_URL + "/data";
    private static final String SECURE_DATA_URL = APP_URL + "/securedata";
    private static final String QUOTA_URL = "/quota";
    

    protected ApplicationServiceImpl() {
        super();
    }

    public ApplicationServiceImpl(IndiceeAPIConnector connector) {
        super(connector);
    }

    @Override
    public void deleteData(String key) throws IndiceeAPIConnectorException {
        connector.makePostCallRtnString(DATA_URL + URL_SLASH + urlEncode(key) + DELETE_URL);
    }

    @Override
    public String getData(String key) throws IndiceeAPIConnectorException {
        Document dataItemDocument = connector.makeGetCallRtnDocument(DATA_URL + URL_SLASH + urlEncode(key));
        return ApplicationDataParser.valueParser.parse (dataItemDocument);
    }

	@Override
	public String getSecureData(String key) throws IndiceeAPIConnectorException {
		Document dataItemDocument = connector.makeGetCallRtnDocument(SECURE_DATA_URL + URL_SLASH + urlEncode(key));
        return ApplicationDataParser.valueParser.parse (dataItemDocument);
	}
	
	@Override
    protected IndiceeAPIService getInstance(IndiceeAPIConnector connector) {
        return new ApplicationServiceImpl(connector);
    }

    @Override
    public Map<String, String> getData() throws IndiceeAPIConnectorException {
        Document dataItemsDocument = connector.makeGetCallRtnDocument(DATA_URL);
        Map<String,String> data = new HashMap<String, String>();
        NodeParser<Void> parser = ApplicationDataParser.createMapParser(data);
        parser.parseList(dataItemsDocument);
    	return data;
    }

    @Override
    public void setData(String key, String value) throws IndiceeAPIConnectorException {
        Map<String, Object> params = new HashMap<String, Object>();
        params.put("value", value);
        connector.makePostCallRtnString(DATA_URL + URL_SLASH + urlEncode(key) + SET_URL, params);
    }


    @Override
    public void setData(Map<String, String> attributes)
    		throws IndiceeAPIConnectorException {
         connector.makePostCallRtnString(DATA_URL, attributes);
    }


	@Override
	public void setSecureData(String key, String value)
			throws IndiceeAPIConnectorException {
		Map<String, Object> params = new HashMap<String, Object>();
        params.put("value", value);
		connector.makePostCallRtnString(SECURE_DATA_URL + URL_SLASH + urlEncode(key) + SET_URL, params);
	}

	@Override
	public int getQuota() throws IndiceeAPIConnectorException {
		try{
			String response = connector.makeGetCallRtnString(APP_URL+QUOTA_URL);
			return Integer.parseInt(response);
		}catch(NumberFormatException e){
			return -1; //unknown
		}
	}
}
