package com.indicee.api.services.impl;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.w3c.dom.Document;

import com.indicee.api.connector.IndiceeAPIConnector;
import com.indicee.api.connector.exception.IndiceeAPIConnectorException;
import com.indicee.api.objects.auth.Consumer;
import com.indicee.api.objects.auth.Token;
import com.indicee.api.parser.AuthParser;
import com.indicee.api.services.interfaces.AuthService;

public class AuthServiceImpl extends IndiceeAPIService implements AuthService {

    private static final String OAUTH_URL = API_URL + "/oauth";
    private static final String CONSUMER_URL = OAUTH_URL + "/consumers";
    private static final String TOKEN_URL = OAUTH_URL + "/tokens";

    protected AuthServiceImpl() {
        super();
    }

    /**
     * Creates a new auth service.
     * 
     * @param connector
     *            The {@link IndiceeAPIConnector} to use.
     */
    public AuthServiceImpl(IndiceeAPIConnector connector) {
        super(connector);
    }

    @Override
    public Consumer createConsumer(String name, String website, String description) throws IndiceeAPIConnectorException {
        Map<String, Object> params = new HashMap<String, Object>();
        params.put("name", name);
        params.put("website", website);
        params.put("description", description);
        Document consumerDocument = connector.makePostCallRtnDocument(CONSUMER_URL + CREATE_URL, params);
        return AuthParser.createConsumer(consumerDocument);
    }
    
    @Override
    public Consumer createConsumer(String name, String website, String description, Long ownerId) throws IndiceeAPIConnectorException {
        Map<String, Object> params = new HashMap<String, Object>();
        params.put("name", name);
        params.put("website", website);
        params.put("description", description);
        params.put("ownerId", ownerId);
        Document consumerDocument = connector.makePostCallRtnDocument(CONSUMER_URL + CREATE_URL, params);
        return AuthParser.createConsumer(consumerDocument);
    }

    @Override
    public Token createToken(String userEmail) throws IndiceeAPIConnectorException {
        return createToken(userEmail, null);
    }

    @Override
    public Token createToken(String userEmail, String consumerKey) throws IndiceeAPIConnectorException {
        Map<String, Object> params = new HashMap<String, Object>();
        params.put("user_login", userEmail);
        if (consumerKey != null) {
            params.put("consumerKey", consumerKey);
        }
        Document tokenDocument = connector.makePostCallRtnDocument(TOKEN_URL + CREATE_URL, params);
        return AuthParser.createToken(tokenDocument);
    }

    @Override
    public Consumer getConsumer(String consumerID) throws IndiceeAPIConnectorException {
        Document entityDocument;
        entityDocument = connector.makeGetCallRtnDocument(CONSUMER_URL + URL_SLASH + urlEncode(consumerID));
        return AuthParser.createConsumer(entityDocument);
    }

    @Override
    protected IndiceeAPIService getInstance(IndiceeAPIConnector connector) {
        return new AuthServiceImpl(connector);
    }

    @Override
    public Token getToken(String tokenID) throws IndiceeAPIConnectorException {
        Document entityDocument;
        entityDocument = connector.makeGetCallRtnDocument(TOKEN_URL + URL_SLASH + urlEncode(tokenID));
        return AuthParser.createToken(entityDocument);
    }

    @Override
    public List<Consumer> listConsumers() throws IndiceeAPIConnectorException {
        Document entityDocument;
        entityDocument = connector.makeGetCallRtnDocument(CONSUMER_URL);
        return AuthParser.createConsumerList(entityDocument);
    }

    @Override
    public List<Token> listTokens() throws IndiceeAPIConnectorException {
        Document entityDocument;
        entityDocument = connector.makeGetCallRtnDocument(TOKEN_URL);
        return AuthParser.createTokenList(entityDocument);
    }

	@Override
	public Token updateToken(String tokenID, long scope)
			throws IndiceeAPIConnectorException {
		Map<String, Object> params = new HashMap<String, Object>();
        params.put("tokenID", tokenID);
        params.put("scope", scope);
        Document tokenDocument = connector.makePostCallRtnDocument(TOKEN_URL + UPDATE_URL, params);
        return AuthParser.createToken(tokenDocument);
	}

	@Override
	public void deleteToken(String tokenID, String tokenSecret)
			throws IndiceeAPIConnectorException {
		Map<String, Object> params = new HashMap<String, Object>();
        params.put("tokenSecret", tokenSecret);
        connector.makePostCallRtnString(TOKEN_URL + URL_SLASH + tokenID + DELETE_URL, params);		
	}

	@Override
	public void deleteConsumer(String consumerID, String consumerSecret)
			throws IndiceeAPIConnectorException {
		Map<String, Object> params = new HashMap<String, Object>();
        params.put("consumerSecret", consumerSecret);
        connector.makePostCallRtnString(CONSUMER_URL + URL_SLASH + consumerID + DELETE_URL, params);
	}
}
