package com.indicee.api.services.impl;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.w3c.dom.Document;

import com.indicee.api.connector.IndiceeAPIConnector;
import com.indicee.api.connector.exception.IndiceeAPIConnectorException;
import com.indicee.api.objects.collection.Collection;
import com.indicee.api.objects.collection.CollectionShareScope;
import com.indicee.api.parser.CollectionParser;
import com.indicee.api.services.interfaces.CollectionService;

public class CollectionServiceImpl extends IndiceeAPIService implements
		CollectionService {

	private static final String COLLECTIONS_URL = API_URL + "/collections";
	private static final String ADD_URL = "/add.xml";
	private static final String CREATE_URL = "/create.xml";
	private static final String UPDATE_URL = "/update.xml";
	private static final String DELETE_URL = "/delete.xml";
	private static final String SHOW_URL = "/show.xml";
	private static final String LIST_URL = "/list.xml";
	private static final String GROUP_URL = "/groups";
	private static final String DATASET_URL = "/datasets";
	private static final String SHARE_URL = "/share.xml";
	private static final String UNSHARE_URL = "/unshare.xml";
	private static final String SHARE_ALL_URL = "/share/all";
	private static final String UNSHARE_ALL_URL = "/unshare/all";
	
	protected CollectionServiceImpl(){
		super();
	}
	
	public CollectionServiceImpl(IndiceeAPIConnector connector) {
        super(connector);
    }
	

	/**
     * {@inheritDoc}
     */
	@Override
	public Collection createCollection(Collection collection)
			throws IndiceeAPIConnectorException {
		return CollectionParser.createCollection(connector.makePostCallRtnDocument(COLLECTIONS_URL+CREATE_URL, collection.toString()));
	}


	/**
     * {@inheritDoc}
     */
	@Override
	public void deleteCollection(long collectionId)
			throws IndiceeAPIConnectorException {
        connector.makePostCallRtnString(COLLECTIONS_URL+ URL_SLASH + collectionId +DELETE_URL);
	}


	/**
     * {@inheritDoc}
     */
	@Override
	public void updateCollection(Collection collection)
			throws IndiceeAPIConnectorException {
		connector.makePostCallRtnDocument(COLLECTIONS_URL+UPDATE_URL, collection.toString());
	}


	/**
     * {@inheritDoc}
     */
	@Override
	public Collection showCollection(String qname)
			throws IndiceeAPIConnectorException {
		HashMap<String, Object> map = new HashMap<String, Object>();
        map.put("qname", qname);
		return CollectionParser.createCollection(connector.makeGetCallRtnDocument(COLLECTIONS_URL+SHOW_URL, map));
		
	}


	/**
     * {@inheritDoc}
     */
	@Override
	public List<Collection> listCollection()
			throws IndiceeAPIConnectorException {
		return listCollectionByInfospace(null);
	}
	
	//TODO: collectionResource.list is not paged yet
//	/**
//	 * {@inheritDoc}
//	 */
//	@Override
//	public List<Collection> listCollection(Page page)
//			throws IndiceeAPIConnectorException {
//		
//		return listCollectionByInfospace(null, page);
//	}
	

	/**
     * {@inheritDoc}
     */
	@Override
	public List<Collection> listCollectionByInfospace(Long infospaceId)
			throws IndiceeAPIConnectorException {
		//TODO: collectionResource.list is not paged yet
//		Page page = new Page();
//    	List<Collection> allList = new ArrayList<Collection>();
//        do {
//        	allList.addAll(listCollectionByInfospace(infospaceId, page));
//        }while(!page.isDone());
//        
//        return allList;
		
		Map<String,Object> map = new HashMap<String, Object>();
    	if(infospaceId!=null){
    		map.put("infospaceId", infospaceId);
    	}
    	
    	Document collectionDocument = connector.makeGetCallRtnDocument(COLLECTIONS_URL+LIST_URL, map);
    	return CollectionParser.createCollectionList(collectionDocument);
	}
	
	//TODO: collectionResource.list is not paged yet
//	/**
//	 * {@inheritDoc}
//	 */
//	@Override
//	public List<Collection> listCollectionByInfospace(Long infospaceId, Page page)
//			throws IndiceeAPIConnectorException {
//		Map<String,Object> map = new HashMap<String, Object>();
//    	if(page != null) {
//    		if(!page.isDefault()) {
//    			map.put(PARAM_PAGE, page.getRpp());
//    			map.put(PARAM_CURSOR, page.getCursor());
//    		}
//    	}else{
//    		throw new IllegalArgumentException("You have to pass a valid Page to list(String,Page) method.");
//    	}
//    	
//    	if(infospaceId!=null){
//    		map.put("infospaceId", infospaceId);
//    	}
//    	
//    	Document collectionDocument = connector.makeGetCallRtnDocument(COLLECTIONS_URL+LIST_URL, map);
//        page.setPage(XmlHelper.childText(collectionDocument.getDocumentElement(), "next_page")); 
//        return CollectionParser.createCollectionList(collectionDocument);
//	}


	/**
     * {@inheritDoc}
     */
	@Override
	public void addDataset(String collectionQname, String datasetQname)
			throws IndiceeAPIConnectorException {
		HashMap<String, Object> map = new HashMap<String, Object>();
        map.put("qname", collectionQname);
        map.put("dataset_qname", datasetQname);
		connector.makePostCallRtnString(COLLECTIONS_URL+DATASET_URL+ADD_URL, map);
	}


	/**
     * {@inheritDoc}
     */
	@Override
	public void deleteDataset(String collectionQname, String datasetQname)
			throws IndiceeAPIConnectorException {
		HashMap<String, Object> map = new HashMap<String, Object>();
        map.put("qname", collectionQname);
        map.put("dataset_qname", datasetQname);
		connector.makePostCallRtnString(COLLECTIONS_URL+DATASET_URL+DELETE_URL, map);
		return;
	}

	/**
     * {@inheritDoc}
     */
	@Override
	public List<Collection> listByGroup(long groupId)
			throws IndiceeAPIConnectorException {
		return CollectionParser.createCollectionList(connector.makeGetCallRtnDocument(COLLECTIONS_URL+GROUP_URL + URL_SLASH + groupId + LIST_URL));
	}
	
	/**
     * {@inheritDoc}
     */
	@Override
	public void shareCollection(String qname, long groupId, CollectionShareScope scope)
			throws IndiceeAPIConnectorException {
		HashMap<String, Object> map = new HashMap<String, Object>();
        map.put("qname", qname);
        map.put("group_id", groupId);
        map.put("scope", scope.getScope());
		connector.makePostCallRtnString(COLLECTIONS_URL+SHARE_URL, map);
	}

	
	@Override
	public void shareAllCollections(long groupId, CollectionShareScope scope)
			throws IndiceeAPIConnectorException {
		shareAllCollections(groupId, scope, false);
	}

	@Override
	public void shareAllCollections(long groupId, CollectionShareScope scope, boolean force)
			throws IndiceeAPIConnectorException {
		HashMap<String, Object> map = new HashMap<String, Object>();
        map.put("group_id", groupId);
        map.put("scope", scope.getScope());
       	map.put("force", force);

		connector.makePostCallRtnString(COLLECTIONS_URL+SHARE_ALL_URL, map);
	}
	
	/**
     * {@inheritDoc}
     */
	@Override
	public void unshareCollection(String qname, long groupId, CollectionShareScope scope)
			throws IndiceeAPIConnectorException {
		HashMap<String, Object> map = new HashMap<String, Object>();
        map.put("qname", qname);
        map.put("group_id", groupId);
        map.put("scope", scope.getScope());
		connector.makePostCallRtnString(COLLECTIONS_URL+UNSHARE_URL, map);
	}

	@Override
	public void unshareAllCollections(long groupId, CollectionShareScope scope)
			throws IndiceeAPIConnectorException {
		HashMap<String, Object> map = new HashMap<String, Object>();
        map.put("group_id", groupId);
        map.put("scope", scope.getScope());
		connector.makePostCallRtnString(COLLECTIONS_URL + UNSHARE_ALL_URL, map);
	}

	/**
     * {@inheritDoc}
     */
	@Override
	protected IndiceeAPIService getInstance(IndiceeAPIConnector connector) {
		return new InfospaceServiceImpl(connector);
	}

}
