package com.indicee.api.services.impl;

import java.io.File;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.UUID;

import org.w3c.dom.Document;

import com.indicee.api.connector.IndiceeAPIConnector;
import com.indicee.api.connector.exception.IndiceeAPIConnectorException;
import com.indicee.api.connector.util.InputStreamWrapper;
import com.indicee.api.objects.Page;
import com.indicee.api.objects.contribution.Contribution;
import com.indicee.api.parser.ContributionParser;
import com.indicee.api.services.interfaces.ContributionService;
import com.indicee.api.services.interfaces.JobService;
import com.indicee.api.util.XmlHelper;


/**
 * This is the implementation of the {@link ContributionService}
 * 
 * @author aminy , tomh
 * 
 */
public class ContributionServiceImpl extends IndiceeAPIService implements ContributionService {

    private static final String CONTRIBUTION_URL = "/api/contributions/v3";
    private static final String DELETE_URL = "/delete";
    private static final String UPLOAD_URL = "/upload";
    private static final String DOWNLOAD_URL = "/download";
    private static final String SHOW_URL = "/show";
    private static final String ANCILLARY_URL = "/ancillary";
    private static final String SET_URL = "/set";
    private static final String LOAD_URL = "/load";
    
    private static final String PARAM_DATASET_QNAME = "dataSetQname";
	private static final String PARAM_ANCILLARY_VALUE = "ancillaryValue";
	private static final String PARAM_PAGE = "rpp";
	private static final String PARAM_CURSOR = "cursor";
	private static final String PARAM_COPY_REQUIRED = "copyRequired";
	private static final String PARAM_FILE = "file";
	private static final String PARAM_DATASET_NAME = "datasetName";
	private static final String PARAM_FORCE_CREATE = "forceCreate";
	private static final String PARAM_FORCE_SAME_NAME = "forceSameName";
	
	private static final String PARAM_INFOSPACE_ID = "infospaceId";
    
    private static final String UPLOAD_URL_V1= "/api/contributions/upload.xml";
    
    protected ContributionServiceImpl() {
        super();
    }

    /**
     * Creates a new contribution service.
     * 
     * @param connector
     *            The {@link IndiceeAPIConnector} to use.
     */
    public ContributionServiceImpl(IndiceeAPIConnector connector) {
        super(connector);
    }

    /**
     * {@inheritDoc}
     */
    public String delete(String dataSetQname, long contributionID) throws IndiceeAPIConnectorException {
    	Map<String, Object> map = new HashMap<String, Object>();
    	map.put(PARAM_DATASET_QNAME, dataSetQname);
        String contributionIDRTN = connector.makePostCallRtnString(CONTRIBUTION_URL + URL_SLASH
                + contributionID + DELETE_URL, map);
        return contributionIDRTN;
    }

    @Override
    protected IndiceeAPIService getInstance(IndiceeAPIConnector connector) {
        return new ContributionServiceImpl(connector);
    }

    
    @Override
    public List<Contribution> list(String dataSetQname, Page page) throws IndiceeAPIConnectorException{
    	String queryString = "?"+PARAM_DATASET_QNAME+"=" + urlEncode(dataSetQname);
    	if(page != null) {
    		if(!page.isDefault()) {
    			queryString+="&"+PARAM_PAGE+"="+page.getRpp()+"&"+PARAM_CURSOR+"="+page.getCursor();
        	}
    	}else{
    		throw new IllegalArgumentException("You have to pass a valid Page to list(String,Page) method.");
    	}
    	Document contributionDocument = connector.makeGetCallRtnDocument(CONTRIBUTION_URL + URL_SLASH + queryString );
        page.setPage(XmlHelper.childText(contributionDocument.getDocumentElement(), "next_page"));
        return ContributionParser.createContributions(contributionDocument);
    }
    
    /**
     * @see com.indicee.api.services.interfaces.ContributionService#latest(java.lang.String)
     */
    @Override
    public Contribution latest(String datasetQualifiedName)
    		throws IndiceeAPIConnectorException {
    	Page page = new Page(1, 0);
		List<Contribution> contributions = list(datasetQualifiedName, page);
    	return contributions.size() == 0 ? null : contributions.get(0);
    }
    
    /**
     * {@inheritDoc}
     */
    @Override
    public List<Contribution> list(String dataSetQname) throws IndiceeAPIConnectorException {
    	Page page = new Page();
    	List<Contribution> allList = new ArrayList<Contribution>();
        do {
        	allList.addAll(list(dataSetQname, page));
        }while(!page.isDone());
        
        return allList;
    }
    
    /**
     * {@inheritDoc}
     */
    @Override
	public byte[] downloadContribution(long contributionId)
			throws IndiceeAPIConnectorException {
		return connector.makeGetCallRtnByteArray(CONTRIBUTION_URL + URL_SLASH + contributionId + DOWNLOAD_URL);
	}

    
    /**
     * {@inheritDoc}
     */
    @Override
	public void downloadContribution(long contributionId, OutputStream os)
			throws IndiceeAPIConnectorException {
		connector.makeGetCallToStream(CONTRIBUTION_URL + URL_SLASH + contributionId + DOWNLOAD_URL, os);
	}

    /**
     * {@inheritDoc}
     */
    @Override
    public Contribution upload(File file) throws IndiceeAPIConnectorException {
        return upload(file, "");
    }
    
    @Override
    public Contribution upload(File file, Long infospaceId)
    		throws IndiceeAPIConnectorException {
    	Map<String, Object> map = new HashMap<String, Object>();
        map.put(PARAM_FILE, file);
        if (infospaceId != null) {
            map.put(PARAM_INFOSPACE_ID, infospaceId);
        }
        Document contributionDocument = connector.makePostCallRtnDocument(CONTRIBUTION_URL+UPLOAD_URL , map);
        return ContributionParser.createContribution(contributionDocument);
    }
    /**
     * {@inheritDoc}
     */
    @Override
    public Contribution upload(File file, String qname) throws IndiceeAPIConnectorException{
    	Map<String, Object> map = new HashMap<String, Object>();
        map.put(PARAM_FILE, file);
        if (qname != null && qname.length() > 0) {
            map.put(PARAM_DATASET_QNAME, qname);
        }
        Document contributionDocument = connector.makePostCallRtnDocument(CONTRIBUTION_URL+UPLOAD_URL , map);
        return ContributionParser.createContribution(contributionDocument);
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public Contribution uploadToNewDataset(File file, String datasetName, Boolean forceSameName) throws IndiceeAPIConnectorException{
    	return uploadToNewDataset(file, datasetName, forceSameName, null);
    }
    
    /** 
     * {@inheritDoc}
     */
    @Override
    public Contribution uploadToNewDataset(File file, String datasetName,
    		Boolean forceSameName, Long infospaceId)
    		throws IndiceeAPIConnectorException {
    	return createDataset(file, datasetName, forceSameName, infospaceId);
    }
    
    /**
     * {@inheritDoc}
     */
    @Override
    public Contribution uploadToNewDataset(InputStreamWrapper inputStreamWrapper, String datasetName, Boolean forceSameName) throws IndiceeAPIConnectorException{
    	return uploadToNewDataset(inputStreamWrapper, datasetName, forceSameName, null);
    }
    
    /**
     * {@inheritDoc}
     */
    @Override
    public Contribution uploadToNewDataset(
    		InputStreamWrapper inputStreamWrapper, String datasetName,
    		Boolean forceSameName, Long infospaceId)
    		throws IndiceeAPIConnectorException {
    	return createDataset(inputStreamWrapper, datasetName, forceSameName, infospaceId);
    }

	private Contribution createDataset(Object fileParam,
			String datasetName, Boolean forceSameName, Long infospaceId)
			throws IndiceeAPIConnectorException {
		Map<String, Object> map = new HashMap<String, Object>();
        map.put(PARAM_FILE, fileParam);
        if (datasetName != null && datasetName.length() > 0) {
            map.put(PARAM_DATASET_NAME, datasetName);
        }
        if(forceSameName!=null){
        	map.put(PARAM_FORCE_SAME_NAME, forceSameName);
        }
        if(infospaceId!=null){
        	map.put(PARAM_INFOSPACE_ID, infospaceId);
        }
    	map.put(PARAM_FORCE_CREATE, true);
        Document contributionDocument = connector.makePostCallRtnDocument(CONTRIBUTION_URL+UPLOAD_URL , map);
        return ContributionParser.createContribution(contributionDocument);
	}
    
    /**
     * {@inheritDoc}
     */
    @Override
    public Contribution upload(File file, UUID datasetID) throws IndiceeAPIConnectorException {
        Map<String, Object> parameters = new HashMap<String, Object>();
        parameters.put("file", file);
        if (datasetID != null) {
            parameters.put("dataset_uuid", datasetID);
        }
        Document contributionDocument = connector.makePostCallRtnDocument(UPLOAD_URL_V1, parameters);
        return ContributionParser.createContribution(contributionDocument);
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public Contribution upload(InputStreamWrapper inputStreamWrapper) throws IndiceeAPIConnectorException {
        return upload(inputStreamWrapper, null);
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public Contribution upload(InputStreamWrapper inputStreamWrapper, String datasetID)
            throws IndiceeAPIConnectorException {
        Map<String, Object> parameters = new HashMap<String, Object>();
        parameters.put(PARAM_FILE, inputStreamWrapper);
        if (datasetID != null && !datasetID.isEmpty()) {
        	parameters.put(PARAM_DATASET_QNAME, datasetID);
        }
        Document contributionDocument = connector.makePostCallRtnDocument(CONTRIBUTION_URL + UPLOAD_URL, parameters);
        return ContributionParser.createContribution(contributionDocument);
    }
    
    /**
     * {@inheritDoc}
     * @throws IndiceeAPIConnectorException 
     */
    @Override
    public Contribution get(long contributionID) throws IndiceeAPIConnectorException {
        Document contributionDocument = connector.makeGetCallRtnDocument(CONTRIBUTION_URL + URL_SLASH + contributionID + SHOW_URL);
        return ContributionParser.createContribution(contributionDocument);
    }
    
    /**
     * {@inheritDoc}
     * @throws IndiceeAPIConnectorException 
     */
    @Override
    public  void setAncillary(long contribitionId, String datasetQname, String ancillaryName, String ancillaryValue)
			throws IndiceeAPIConnectorException{
    	Map<String, Object> map = new HashMap<String, Object>();
    	map.put(PARAM_DATASET_QNAME, datasetQname);
    	map.put(PARAM_ANCILLARY_VALUE , ancillaryValue);
    	
    	connector.makePostCallRtnString(CONTRIBUTION_URL + URL_SLASH + contribitionId + ANCILLARY_URL + URL_SLASH 
    			+ urlEncode(ancillaryName) + SET_URL, map);
    }
    
    /**
     * {@inheritDoc}
     * @throws IndiceeAPIConnectorException 
     */
    @Override
    public  void deleteAncillary(long contribitionId, String datasetQname, String ancillaryName)
			throws IndiceeAPIConnectorException{
    	Map<String, Object> map = new HashMap<String, Object>();
    	map.put(PARAM_DATASET_QNAME, datasetQname);
    	
    	connector.makePostCallRtnString(CONTRIBUTION_URL + URL_SLASH + contribitionId + ANCILLARY_URL + URL_SLASH 
    			+ ancillaryName + DELETE_URL, map);
    }
    
    @Override
    public Contribution load(long contribitionId, String datasetQname, boolean copyRequired) throws IndiceeAPIConnectorException {
    	Map<String, Object> map = new HashMap<String, Object>();
    	map.put(PARAM_DATASET_QNAME, datasetQname);
    	map.put(PARAM_COPY_REQUIRED , copyRequired);
    	Document document = connector.makePostCallRtnDocument(CONTRIBUTION_URL + URL_SLASH + contribitionId + LOAD_URL, map);
    	return ContributionParser.createContribution(document);
    }

	@Override
	public JobService getJobService() {
		return new JobServiceImpl(this.connector);
	}
    
}
