package com.indicee.api.services.impl;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import com.indicee.api.connector.IndiceeAPIConnector;
import com.indicee.api.connector.exception.IndiceeAPIConnectorException;
import com.indicee.api.objects.dashboard.Dashboard;
import com.indicee.api.objects.dashboard.DashboardMetadata;
import com.indicee.api.objects.dashboard.DashboardShareScope;
import com.indicee.api.parser.DashboardParser;
import com.indicee.api.services.interfaces.DashboardService;

public class DashboardServiceImpl extends IndiceeAPIService implements
		DashboardService {
	

	private static final String DASHBOARDS_URL = API_URL + "/dashboards/v2";
	private static final String GROUP_URL = "/groups";
	private static final String SHARE_URL = "/share";
	private static final String ALL_URL ="/all";
	private static final String UNSHARE_URL = "/unshare";
	private static final String METADATA_URL ="/metadata";
	
	protected DashboardServiceImpl() {
        super();
    }
	
	public DashboardServiceImpl(IndiceeAPIConnector connector){
        super(connector);
    }
	
	/**
     * {@inheritDoc}
     */
	@Override
	public List<Dashboard> list()  throws IndiceeAPIConnectorException {
		return list(null, null);
	}

	/**
     * {@inheritDoc}
     */
	@Override
	public List<Dashboard> list(Long rpp, Long cursor)  throws IndiceeAPIConnectorException {
		HashMap<String, Object> map = new HashMap<String, Object>();
		if(rpp !=null) {
			map.put("rpp",rpp);
		}
		if(cursor !=null) {
			map.put("cursor", cursor);
		}
		return DashboardParser.createDashboardList( connector.makeGetCallRtnDocument(DASHBOARDS_URL + LIST_URL + DOT_XML, map));
	}

	/**
     * {@inheritDoc}
     */
	@Override
	public Dashboard show(String qname) throws IndiceeAPIConnectorException {
		HashMap<String, Object> map = new HashMap<String, Object>();
		map.put("qname", qname);
		return DashboardParser.createDashboard(connector.makeGetCallRtnDocument(DASHBOARDS_URL + URL_SLASH, map));
	}


	/**
     * {@inheritDoc}
     */
	@Override
	public Dashboard create(Dashboard dashboard)  throws IndiceeAPIConnectorException {
		return DashboardParser.createDashboard(connector.makePostCallRtnDocument(DASHBOARDS_URL + CREATE_URL + DOT_XML, dashboard.toString()));
	}


	/**
     * {@inheritDoc}
     */
	@Override
	public void delete(String qname)  throws IndiceeAPIConnectorException {
		HashMap<String, Object> map = new HashMap<String, Object>();
		map.put("qname", qname);
		connector.makePostCallRtnString(DASHBOARDS_URL + DELETE_URL + DOT_XML, map);
	}


	/**
     * {@inheritDoc}
     */
	@Override
	public Dashboard update(Long dashboardId, Dashboard dashboard)  throws IndiceeAPIConnectorException {
		HashMap<String, Object> map = new HashMap<String, Object>();
		map.put("dashboardId", dashboardId);
		map.put("dashboard", dashboard);
		return DashboardParser.createDashboard(connector.makePostCallRtnDocument(DASHBOARDS_URL + UPDATE_URL + DOT_XML, map));
	}


	/**
     * {@inheritDoc}
     */
	@Override
	protected IndiceeAPIService getInstance(IndiceeAPIConnector connector){
		return new DashboardServiceImpl(connector);
	}

	@Override
	public void share(String qname, long groupId, DashboardShareScope scope)
			throws IndiceeAPIConnectorException {
		HashMap<String, Object> map = new HashMap<String, Object>();
		map.put("qname", qname);
		map.put("group_id", groupId);
		map.put("scope", scope.getScope());
		connector.makePostCallRtnString(DASHBOARDS_URL + SHARE_URL , map );
		
	}

	@Override
	public void shareAll(long groupId, DashboardShareScope scope)
			throws IndiceeAPIConnectorException {
		shareAll(groupId, scope, false);
	}
	
	@Override
	public void shareAll(long groupId, DashboardShareScope scope, boolean force)
			throws IndiceeAPIConnectorException {
		HashMap<String, Object> map = new HashMap<String, Object>();
		map.put("group_id", groupId);
		map.put("scope", scope.getScope());
		map.put("force",force);

		connector.makePostCallRtnString(DASHBOARDS_URL + SHARE_URL + ALL_URL, map );	
	}
	
	@Override
	public void unshare(String qname, long groupId, DashboardShareScope scope)
			throws IndiceeAPIConnectorException {
		HashMap<String, Object> map = new HashMap<String, Object>();
		map.put("qname", qname);
		map.put("group_id", groupId);
		map.put("scope", scope.getScope());
		connector.makePostCallRtnString(DASHBOARDS_URL + UNSHARE_URL , map );
		
	}
	
	@Override
	public void unshareAll(long groupId, DashboardShareScope scope)
			throws IndiceeAPIConnectorException {
		HashMap<String, Object> map = new HashMap<String, Object>();
		map.put("group_id", groupId);
		map.put("scope", scope.getScope());
		connector.makePostCallRtnString(DASHBOARDS_URL + UNSHARE_URL + ALL_URL, map );
		
	}

	@Override
	public List<Dashboard> listByInfospace(Long infospaceId, Long rpp,
			Long cursor) throws IndiceeAPIConnectorException {
		HashMap<String, Object> map = new HashMap<String, Object>();
		if(rpp !=null) {
			map.put("rpp",rpp);
		}
		if(cursor !=null) {
			map.put("cursor", cursor);
		}
		return DashboardParser.createDashboardList( connector.makeGetCallRtnDocument(DASHBOARDS_URL + URL_SLASH + infospaceId + LIST_URL + DOT_XML, map));
	}

	@Override
	public List<Dashboard> listByInfospace(Long infospaceId)
			throws IndiceeAPIConnectorException {
		return listByInfospace(infospaceId, null, null);
	}
	
	@Override
	public List<Dashboard> listByGroup(long groupId, Long rpp, Long cursor)
			throws IndiceeAPIConnectorException {
		HashMap<String, Object> map = new HashMap<String, Object>();
		if(rpp != null) {
			map.put("rpp",rpp);
		}
		if(cursor != null) {
			map.put("cursor", cursor);
		}
		return DashboardParser.createDashboardList( connector.makeGetCallRtnDocument(DASHBOARDS_URL + GROUP_URL + URL_SLASH + groupId + LIST_URL + DOT_XML, map));
	}
	
	@Override
	public List<Dashboard> listByGroup(long groupId)
			throws IndiceeAPIConnectorException {
		return listByGroup(groupId, null, null);
	}

	@Override
	public DashboardMetadata getMetadata(String qname)
			throws IndiceeAPIConnectorException {
		Map<String,Object> map = new HashMap<String, Object>();
		map.put("qname", qname);
		
		return DashboardParser.createDashboardMetadata(connector.makePostCallRtnDocument(DASHBOARDS_URL + METADATA_URL + DOT_XML, map));
	}

	@Override
	public void setMetadata(DashboardMetadata metadata)
			throws IndiceeAPIConnectorException {
		connector.makePostCallRtnString(DASHBOARDS_URL + METADATA_URL, metadata.toString()); 
	}

}
