package com.indicee.api.services.impl;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import com.indicee.api.connector.IndiceeAPIConnector;
import com.indicee.api.connector.exception.IndiceeAPIConnectorException;
import com.indicee.api.objects.group.Group;
import com.indicee.api.parser.GroupParser;
import com.indicee.api.services.interfaces.GroupService;

public class GroupServiceImpl extends IndiceeAPIService implements GroupService {

	private static final String GROUPS_URL = API_URL + "/groups";
	private static final String OWNED_URL = "/owned";
	private static final String JOINED_URL = "/joined";
	private static final String MEMBERS_URL = "/members";
	
	protected GroupServiceImpl() {
        super();
    }
	
	public GroupServiceImpl(IndiceeAPIConnector connector){
        super(connector);
    }
	
	@Override
	public Group createGroup(Group group) throws IndiceeAPIConnectorException {
		return GroupParser.createGroup(connector.makePostCallRtnDocument(GROUPS_URL + CREATE_URL + DOT_XML, group.toString()));
	}

	@Override
	public void deleteGroup(Long groupId) throws IndiceeAPIConnectorException {
		connector.makePostCallRtnString(GROUPS_URL + URL_SLASH + groupId + DELETE_URL + DOT_XML);
	}

	@Override
	public Group updateGroup(Group group) throws IndiceeAPIConnectorException {
		return GroupParser.createGroup(connector.makePostCallRtnDocument(GROUPS_URL + URL_SLASH + group.getId()+ UPDATE_URL + DOT_XML, group.toString()));
	}

	@Override
	public Group getGroup(Long groupId) throws IndiceeAPIConnectorException {
		return GroupParser.createGroup(connector.makeGetCallRtnDocument(GROUPS_URL + URL_SLASH + groupId+ SHOW_URL + DOT_XML));
	}

	@Override
	public List<Group> getGroups(List<Long> groupIds) throws IndiceeAPIConnectorException {
		
		StringBuilder sb = new StringBuilder();
		for(Long id:groupIds){
			if(sb.length()==0){
				sb.append(id);
			}else{
				sb.append("," + id);
			}
		}
		
		Map<String,Object> map = new HashMap<String, Object>();
		map.put("group_ids", sb.toString());
		
		return GroupParser.createGroups(connector.makeGetCallRtnDocument(GROUPS_URL + LIST_URL + DOT_XML));
	}

	@Override
	public List<Group> listOwnedGroups() throws IndiceeAPIConnectorException {
		return GroupParser.createGroups(connector.makeGetCallRtnDocument(GROUPS_URL + OWNED_URL + LIST_URL + DOT_XML));
	}

	@Override
	public List<Group> listJoinedGroups() throws IndiceeAPIConnectorException {
		return GroupParser.createGroups(connector.makeGetCallRtnDocument(GROUPS_URL + JOINED_URL + LIST_URL + DOT_XML));
	}

	@Override
	public void addUserToGroup(Long groupId, Long userId) throws IndiceeAPIConnectorException{
		connector.makePostCallRtnString(GROUPS_URL + URL_SLASH + groupId + MEMBERS_URL + URL_SLASH + userId + CREATE_URL + DOT_XML );
	}

	@Override
	public void removeUserFromGroup(Long groupId, Long userId) throws IndiceeAPIConnectorException{
		connector.makePostCallRtnString(GROUPS_URL + URL_SLASH + groupId + MEMBERS_URL + URL_SLASH + userId + DELETE_URL + DOT_XML );
	}

	@Override
	protected IndiceeAPIService getInstance(IndiceeAPIConnector connector) {
		return new DashboardServiceImpl(connector);
	}

}
