package com.indicee.api.services.impl;

import java.util.HashMap;
import java.util.List;

import com.indicee.api.connector.IndiceeAPIConnector;
import com.indicee.api.connector.exception.IndiceeAPIConnectorException;
import com.indicee.api.objects.identityprovider.IdentityProvider;
import com.indicee.api.parser.IdentityProviderParser;
import com.indicee.api.services.interfaces.IdentityProviderService;

public class IdentityProviderServiceImp extends IndiceeAPIService implements
		IdentityProviderService {

	private static final String IDP_URL = API_URL + "/identityProvider";
	private static final String GET_URL = "/get";
	
	public IdentityProviderServiceImp(IndiceeAPIConnector connector) {
		super(connector);
	}

	@Override
	public List<IdentityProvider> listIdentityProviders() throws IndiceeAPIConnectorException {
		return IdentityProviderParser.createIdentityProviders(connector.makeGetCallRtnDocument(IDP_URL + LIST_URL + DOT_XML));
	}

	@Override
	public void createIdentityProvider(IdentityProvider idp)
			throws IndiceeAPIConnectorException {
		connector.makePostCallRtnString(IDP_URL + CREATE_URL + DOT_XML , idp.toString());		
	}

	@Override
	public void updateIdentityProvider(IdentityProvider idp)
			throws IndiceeAPIConnectorException {
		connector.makePostCallRtnString(IDP_URL + UPDATE_URL + DOT_XML , idp.toString());
	}

	@Override
	public void deleteIdentityProvider(String idpDomain)
			throws IndiceeAPIConnectorException {
		HashMap<String, Object> map = new HashMap<String, Object>();
        map.put("domain", idpDomain);
		connector.makePostCallRtnString(IDP_URL + DELETE_URL , map);
	}

	@Override
	public IdentityProvider getIdentityProvider(String idpDomain)
			throws IndiceeAPIConnectorException {
		return IdentityProviderParser.createIdentityProvider(connector.makeGetCallRtnDocument(IDP_URL + URL_SLASH + idpDomain + GET_URL + DOT_XML));
	}
	
	@Override
	protected IndiceeAPIService getInstance(IndiceeAPIConnector connector) {
		return new IdentityProviderServiceImp(connector);
	}

}
