package com.indicee.api.services.impl;

import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;

import com.indicee.api.connector.IndiceeAPIConnector;
import com.indicee.api.connector.exception.IndiceeAPIConnectorException;

/**
 * This defines an abstract Indicee API Service that all other API services
 * extend.
 * 
 * @author victork, aminy
 * 
 */
public abstract class IndiceeAPIService {

	protected static final String PARAM_PAGE = "rpp";
	protected static final String PARAM_CURSOR = "cursor";
	
    protected static final String API_URL = "/api";
    protected static final String CREATE_URL = "/create";
    protected static final String DELETE_URL = "/delete";
    protected static final String DOT_XML = ".xml";
    protected static final String LIST_URL = "/list";
    protected static final String SET_URL = "/set";
    protected static final String SHOW_URL = "/show";
    protected static final String UPDATE_URL = "/update";
    protected static final String UPLOAD_URL = "/upload";
    protected static final String URL_QUERY_PARAM_START = "?";
    protected static final String URL_SLASH = "/";
    protected static final String VIEW_URL = "/view";

    public static IndiceeAPIService getInstance(Class<? extends IndiceeAPIService> clazz, IndiceeAPIConnector connector) {
        try {
            return clazz.newInstance().getInstance(connector);
        } catch (Exception e) {
            throw new RuntimeException("Unexpected Excpetion", e);
        }
    }

    public static String urlEncode(String string) throws IndiceeAPIConnectorException {
        try {
            return URLEncoder.encode(string, "UTF-8").replaceAll("\\+", "%20");
        } catch (UnsupportedEncodingException e) {
            throw new IndiceeAPIConnectorException("Unexpected error url encoding: " + string);
        }
    }

    protected final IndiceeAPIConnector connector;

    protected IndiceeAPIService() {
        this.connector = null;
    }

    protected IndiceeAPIService(IndiceeAPIConnector connector) {
    	if (connector == null)
    		throw new NullPointerException("connector cannot be null");
        this.connector = connector;
    }

    protected abstract IndiceeAPIService getInstance(IndiceeAPIConnector connector);
}
