package com.indicee.api.services.impl;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.HashMap;
import java.util.List;

import com.indicee.api.connector.IndiceeAPIConnector;
import com.indicee.api.connector.exception.IndiceeAPIConnectorException;
import com.indicee.api.objects.infospace.Infospace;
import com.indicee.api.parser.InfospaceParser;
import com.indicee.api.services.interfaces.InfospaceService;

/**
 * This is the implementation of the {@link InfospaceService}
 * 
 * @author aminy
 * 
 */
public class InfospaceServiceImpl extends IndiceeAPIService implements InfospaceService {

	private static final String INFOSPACES_URL = API_URL + "/infospaces/v1";
	private static final String SET_URL = "/set";
	private static final String DEPLOY_URL = "/deploy";
	private static final String COPY_URL = "/copy";
	private static final String DOWNLOAD_URL = "/download";
	private static final String UPLOAD_URL = "/upload";
	private static final String LIST_URL = "/list";
	private static final String CREATE_URL = "/create";
	private static final String UPDATE_URL = "/update";
	private static final String DELETE_URL = "/delete";
	private static final String SHOW_URL = "/show";
	private static final String APPLY_URL = "/apply";
	
	
	protected InfospaceServiceImpl() {
        super();
    }
	
	public InfospaceServiceImpl(IndiceeAPIConnector connector) {
        super(connector);
    }
	
	@Override
	public void setInfospace(long infospaceId)
			throws IndiceeAPIConnectorException {
		String s = connector.makePostCallRtnString(INFOSPACES_URL + URL_SLASH + infospaceId + SET_URL);
		System.out.println(s);
	}
	
	@Override
	public void applyDepolyments(long userId) throws IndiceeAPIConnectorException {
		HashMap<String, Object> map = new HashMap<String, Object>();
        map.put("user_id", userId);
        connector.makePostCallRtnString(INFOSPACES_URL + APPLY_URL, map);
	}

	@Override
	public void deployInfospace(long infospaceId, long userId,
			boolean forceUpgrade, boolean overwriteData, boolean removeReports)
			throws IndiceeAPIConnectorException {
		deployInfospace(infospaceId, userId, forceUpgrade, overwriteData, removeReports, null);
	}
	
	@Override
	public void deployInfospace(long infospaceId, long userId,
			boolean forceUpgrade, boolean overwriteData, boolean removeReports, Boolean forceDeployToCurrentInfospace)
			throws IndiceeAPIConnectorException {
		HashMap<String, Object> map = new HashMap<String, Object>();
        map.put("user_id", userId);
        map.put("force_upgrade", forceUpgrade);
        map.put("overwrite_data", overwriteData);
        map.put("remove_reports", removeReports);
        if(forceDeployToCurrentInfospace != null){
        	map.put("deploy_to_current", forceDeployToCurrentInfospace);
        }
        
        connector.makePostCallRtnString(INFOSPACES_URL + URL_SLASH + infospaceId + DEPLOY_URL, map);
	}

	@Override
	public void copyInfospace(long infospaceId, long srcInfospaceId,
			boolean includData) throws IndiceeAPIConnectorException {
		HashMap<String, Object> map = new HashMap<String, Object>();
		map.put("source", srcInfospaceId);
        map.put("include_data", includData);
        
        connector.makePostCallRtnString(INFOSPACES_URL + URL_SLASH + infospaceId + COPY_URL, map);
	}

	@Override
	public byte[] downloadInfospace(long infospaceId)
			throws IndiceeAPIConnectorException {
		return connector.makeGetCallRtnByteArray(INFOSPACES_URL + URL_SLASH + infospaceId + DOWNLOAD_URL);
	}
	
	@Override
	public void downloadInfospace(long infospaceId, File file)
			throws IndiceeAPIConnectorException {
		try {
			FileOutputStream fos = new FileOutputStream(file);
			try {
				connector.makeGetCallToStream(INFOSPACES_URL + URL_SLASH + infospaceId + DOWNLOAD_URL, fos);
			} finally {
				fos.close();
			}
		} catch (IOException e) {
			throw new IndiceeAPIConnectorException("Error using file: " + file.getAbsolutePath(), e);
		}
	}

	@Override
	public void uploadInfospace(long infospaceId, File file)
			throws IndiceeAPIConnectorException {
		HashMap<String, Object> map = new HashMap<String, Object>();
        map.put("file", file);
        
        connector.makePostCallRtnString(INFOSPACES_URL + URL_SLASH + infospaceId + UPLOAD_URL, map);
	}

	@Override
	public List<Infospace> listInfospace() throws IndiceeAPIConnectorException {
		return InfospaceParser.createInfospaceList(connector.makeGetCallRtnDocument(INFOSPACES_URL + LIST_URL));
	}

	@Override
	public Infospace createInfospace(Infospace infospace)
			throws IndiceeAPIConnectorException {
		return InfospaceParser.createInfospace(connector.makePostCallRtnDocument(INFOSPACES_URL + CREATE_URL, infospace.toString()));
	}

	@Override
	public Infospace updateInfospace(long infospaceId, Infospace infospace)
			throws IndiceeAPIConnectorException {
		return InfospaceParser.createInfospace(connector.makePostCallRtnDocument(INFOSPACES_URL + URL_SLASH + infospaceId + UPDATE_URL, infospace.toString()));
	}

	@Override
	public void deleteInfospace(long infospaceId)
			throws IndiceeAPIConnectorException {
		connector.makePostCallRtnString(INFOSPACES_URL + URL_SLASH + infospaceId + DELETE_URL);
	}

	/**
	 * @see com.indicee.api.services.interfaces.InfospaceService#showCurrentInfospace()
	 */
	@Override
	public Infospace showCurrentInfospace() throws IndiceeAPIConnectorException {
		return InfospaceParser.createInfospace(connector.makeGetCallRtnDocument(INFOSPACES_URL + SHOW_URL));
	}
	
	@Override
	public Infospace showInfospace(long infospaceId)
			throws IndiceeAPIConnectorException {
		return InfospaceParser.createInfospace(connector.makeGetCallRtnDocument(INFOSPACES_URL + URL_SLASH + infospaceId + SHOW_URL));		
	}

	@Override
	protected IndiceeAPIService getInstance(IndiceeAPIConnector connector) {
		return new InfospaceServiceImpl(connector);
	}

}
