package com.indicee.api.services.interfaces;

import java.util.Map;

import com.indicee.api.connector.exception.IndiceeAPIConnectorException;

/**
 * Service for getting and setting application specific data, applications can store key / value data per user. 
 */
public interface ApplicationService {

    /**
     * Delete a piece of data.
     * @param key
     * @throws IndiceeAPIConnectorException
     */
    public void deleteData(String key) throws IndiceeAPIConnectorException;

    /**
     * Read a piece of data
     * @param key
     * @return
     * @throws IndiceeAPIConnectorException
     */
    public String getData(String key) throws IndiceeAPIConnectorException;

    /**
     * Read all the data.
     * @return
     * @throws IndiceeAPIConnectorException
     */
    public Map<String, String> getData() throws IndiceeAPIConnectorException;
    
    /**
     * Get data that's been stored encrypted for that user.
     * @param key
     * @return
     * @throws IndiceeAPIConnectorException
     */
    public String getSecureData(String key) throws IndiceeAPIConnectorException;

    /**
     * Update a piece of data
     * @param key
     * @param value
     * @throws IndiceeAPIConnectorException
     */
    public void setData(String key, String value) throws IndiceeAPIConnectorException;

	/**
	 * Update a set of data values
	 * @param attributes
	 * @throws IndiceeAPIConnectorException
	 */
	public void setData(Map<String, String> attributes) throws IndiceeAPIConnectorException;
	
	/**
	 * Create or Update a value that's stored in an encrypted format specific to the user.
	 * @param key
	 * @param value
	 * @throws IndiceeAPIConnectorException
	 */
	public void setSecureData(String key, String value) throws IndiceeAPIConnectorException;
	
	public int getQuota() throws IndiceeAPIConnectorException;

}
