package com.indicee.api.services.interfaces;

import java.util.List;

import com.indicee.api.connector.exception.IndiceeAPIConnectorException;
import com.indicee.api.objects.group.Group;

/**
 * Service for working with groups, groups are the mechanism by which content is shared between users. 
 */
public interface GroupService {

	/**
	 * Will create a group. Name, description and color are mandatory.
	 * @param group
	 * @return Group
	 * @throws IndiceeAPIConnectorException
	 */
	Group createGroup(Group group) throws IndiceeAPIConnectorException;
	
	/**
	 * Will delete a group by its id
	 * @param groupId - the group id
	 * @throws IndiceeAPIConnectorException
	 */
	void deleteGroup(Long groupId) throws IndiceeAPIConnectorException;
	
	/**
	 * Will update a group. 
	 * @param group - the group to update with its new values
	 * @return Group
	 * @throws IndiceeAPIConnectorException
	 */
	Group updateGroup(Group group) throws IndiceeAPIConnectorException;
	
	/**
	 * Will get a group by its id
	 * @param groupId  the id of the group
	 * @return
	 * @throws IndiceeAPIConnectorException
	 */
	Group getGroup(Long groupId) throws IndiceeAPIConnectorException;
	
	/**
	 * Will return multiple groups by their ids
	 * @param groupIds - list of group ids
	 * @return List of Group
	 * @throws IndiceeAPIConnectorException
	 */
	List<Group> getGroups(List<Long> groupIds) throws IndiceeAPIConnectorException;
	
	/**
	 * Will return a list of groups owned by the application token's user.
	 * @return List of Group
	 * @throws IndiceeAPIConnectorException
	 */
	List<Group> listOwnedGroups() throws IndiceeAPIConnectorException;
	
	/**
	 * Will return list of groups that the application token's user is a member of.
	 * @return List of Group
	 * @throws IndiceeAPIConnectorException
	 */
	List<Group> listJoinedGroups() throws IndiceeAPIConnectorException;
	
	/** 
	 * Will add a user to a group. caller should be admin of the group
	 * @param groupId - the group id
	 * @param userId - the id of the user to add to the group
	 * @throws IndiceeAPIConnectorException
	 */
	void addUserToGroup(Long groupId, Long userId) throws IndiceeAPIConnectorException;
	
	/**
	 * Will remove a user from a group. caller should be admin of the user
	 * @param groupId - the group id
	 * @param userId - the id of the user to remove from the group
	 * @throws IndiceeAPIConnectorException
	 */
	void removeUserFromGroup(Long groupId, Long userId) throws IndiceeAPIConnectorException;
}
