package com.indicee.api.services.interfaces;

import java.util.List;

import com.indicee.api.connector.exception.IndiceeAPIConnectorException;
import com.indicee.api.objects.identityprovider.IdentityProvider;

/**
 * This service is used to create, delete, update and list Identity Providers.
 * An Identity Provider is a SAML concept for a service that can authenticate a token as user. 
 * 
 */
public interface IdentityProviderService {
	
	/**
	 * List the identity provides owned by a the token's user
	 * @return List of IdentityProvider
	 * @throws IndiceeAPIConnectorException
	 */
	List<IdentityProvider> listIdentityProviders() throws IndiceeAPIConnectorException;
	
	/**
	 * Create an identity provider
	 * @param identityProvider
	 * @throws IndiceeAPIConnectorException
	 */
	void createIdentityProvider(IdentityProvider identityProvider) throws IndiceeAPIConnectorException;
	
	/**
	 * Update an identity provider
	 * @param identityProvider
	 * @throws IndiceeAPIConnectorException
	 */
	void updateIdentityProvider(IdentityProvider identityProvider) throws IndiceeAPIConnectorException;
	
	/**
	 * Delete an identity provider by its domain
	 * @param identityProviderDomain
	 * @throws IndiceeAPIConnectorException
	 */
	void deleteIdentityProvider(String identityProviderDomain) throws IndiceeAPIConnectorException;
	
	/**
	 * Get an identity provider by its domain
	 * @param identityProviderDomain
	 * @return IdentityProvider
	 * @throws IndiceeAPIConnectorException
	 */
	IdentityProvider getIdentityProvider(String identityProviderDomain) throws IndiceeAPIConnectorException;

}
