/**
 * 
 */
package com.indicee.api.xml;

import java.util.Map;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import com.indicee.api.objects.account.Account;
import com.indicee.api.util.XmlHelper;

/**
 * Reads and Writes an Account object from XML DOM
 * @author thaggie
 */
public final class AccountXml {

	public static final String TAG = "account";
	
	private static enum Tag {
        attribute,
        attributes,
        created_time,
        display_name,
        email,
        first_name,
        id,
        industry,
        last_name,
        organization,
        parent,
        password,
        phone,
        status,
        title;
	}
	
	private static final Map<String,Tag> tagLookup = ToXml.createTagLookup(Tag.values());
	
	public static final ToXml<Account> toXml = new ToXml<Account>() {
		@Override
		public Element call(Document document, Account account) {
			
			Element accountElement = createElement(document, TAG);
			append(accountElement, Tag.id,           account.getId()          );
			append(accountElement, Tag.created_time, account.getCreatedDate() );
			append(accountElement, Tag.display_name, account.getDisplayName() );
			append(accountElement, Tag.email,        account.getEmail()       );
			append(accountElement, Tag.first_name,   account.getFirstName()   );
			append(accountElement, Tag.industry,     account.getIndustry()    );
			append(accountElement, Tag.last_name,    account.getLastName()    );
			append(accountElement, Tag.organization, account.getOrganization());
			append(accountElement, Tag.title,        account.getTitle()       );
			append(accountElement, Tag.parent,       account.getParent()      );
			append(accountElement, Tag.password,     account.getPassword()    );
			append(accountElement, Tag.status,       account.getStatus()      );
			append(accountElement, Tag.phone,        account.getPhone()       );
			
			append(accountElement, Tag.attributes,   account.getAttributes(), AccountAttributeXml.toXml);

			return accountElement;
		}
	};
	
	public static final FromXml<Account> fromXml = new FromXml<Account>() {
		
		@Override
		public Account call(Element element) {
			Account account = new Account();
			for (Element childElement : XmlHelper.childElements(element)) {
				String tagName = childElement.getTagName();
				Tag t = tagLookup.get(tagName);
				if (t != null) {
					switch (t) {
					case id:
						account.setId(getLong(childElement));
						break;
					case created_time:
						account.setCreatedDate(getDate(childElement));
						break;
					case display_name:
						account.setDisplayName(getText(childElement));
						break;
					case email:
						account.setEmail(getText(childElement));
						break;
					case first_name:
						account.setFirstName(getText(childElement));
						break;
					case industry:
						account.setIndustry(getText(childElement));
						break;
					case last_name:
						account.setLastName(getText(childElement));
						break;
					case organization:
						account.setOrganization(getText(childElement));
						break;
					case title:
						account.setTitle(getText(childElement));
						break;
					case parent:
						account.setParent(getLong(childElement));
						break;
					case password:
						account.setPassword(getText(childElement));
						break;
					case status:
						account.setStatus(getText(childElement));
						break;
					case phone:
						account.setPhone(getText(childElement));
						break;
					case attributes:
						account.setAttributes(getChildren(childElement, AccountAttributeXml.fromXml));
						break;
	                default:
	                	break;
					}
				}
			}
			return account;
		}
	};
}