package com.indicee.api.xml;

import java.util.Map;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import com.indicee.api.objects.contribution.Ancillary;
import com.indicee.api.util.XmlHelper;

public class AncillaryXml {

	public static final String TAG = "ancillary";
	
	private static enum Tag {
		name,
		double_value,
		date_value,
		string_value
	}
	
	private static final Map<String,Tag> tagLookup = ToXml.createTagLookup(Tag.values());
	
	public static final ToXml<Ancillary> toXml = new ToXml<Ancillary>() {
		
		@Override
		public Element call(Document document, Ancillary ancillary) {
			Element ancillaryElement = createElement(document, TAG);
			
			append(ancillaryElement, Tag.name, ancillary.getName()                );
			append(ancillaryElement, Tag.double_value, ancillary.getDoubleValue() );
			append(ancillaryElement, Tag.date_value, ancillary.getDateValue()     );
			append(ancillaryElement, Tag.string_value, ancillary.getStringValue() );

			return ancillaryElement;
		}
	};
	
	public static final FromXml<Ancillary> fromXml = new FromXml<Ancillary>() {
		@Override
		public Ancillary call(Element element) {
			Ancillary ancillary = new Ancillary();
			
			for (Element childElement : XmlHelper.childElements(element)) {
				String tagName = childElement.getTagName();
				Tag t = tagLookup.get(tagName);
				if (t != null) {
					switch (t) {
					case name:
						ancillary.setName(getText(childElement));
						break;
					case double_value:
						ancillary.setDoubleValue(getDouble(childElement));
						break;
					case date_value:
						ancillary.setDateValue(getText(childElement));
						break;
					case string_value:
						ancillary.setStringValue(getText(childElement));
						break;
					}
				}
			}
			return ancillary;
		}
	};
}
