package com.indicee.api.xml;

import java.util.List;
import java.util.Map;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import com.indicee.api.objects.report.axes.AxisColumn;
import com.indicee.api.util.XmlHelper;

public class AxisColumnXml {
	public static final String TAG = "column";
	
	private static enum Tag {
		type,
		name,
		value,
		sub
	}
	
	private static final Map<String,Tag> tagLookup = ToXml.createTagLookup(Tag.values());
	
	public static final ToXml<AxisColumn> toXml = new ToXml<AxisColumn>() {
		@Override
		public Element call(Document document, AxisColumn axisColumn) {
			
			Element columnElement = ToXml.createElement(document, TAG);
			columnElement.setAttribute(Tag.type.toString(), axisColumn.getType());
			
			ToXml.append(columnElement, Tag.name, axisColumn.getName());
			ToXml.append(columnElement, Tag.value, axisColumn.getValue());
			
			List<AxisColumn> subColumns = axisColumn.getSubColumns();
			if (subColumns != null) {
				Element subColumnsElement = ToXml.createElement(document, Tag.sub);
				for (AxisColumn subColumn : subColumns) {
					Element subColumnElement = call(document, subColumn);
					subColumnsElement.appendChild(subColumnElement);
				}
				columnElement.appendChild(subColumnsElement);
			}
			
			return columnElement;
		}
	};
	
	public static final FromXml<AxisColumn> fromXml = new FromXml<AxisColumn>() {
		@Override
		public AxisColumn call(Element columnElement) {
			AxisColumn axisColumn = new AxisColumn();
			axisColumn.setType(columnElement.getAttribute(Tag.type.toString()));
			
			for (Element child : XmlHelper.childElements(columnElement)) {
				Tag tag = tagLookup.get(child.getTagName());
				if (tag != null) {
					switch (tag) {
					case name:
						axisColumn.setName(getText(child));
						break;
					case value:
						axisColumn.setValue(getText(child));
						break;
					case sub:
						axisColumn.setSubColumns(getChildren(child, this));
	                default:
	                	break;
					}
				}
			}
			return axisColumn;
		}
	};
}
