package com.indicee.api.xml;

import java.util.ArrayList;
import java.util.List;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import com.indicee.api.objects.report.view.attributes.AttributesOrigin;
import com.indicee.api.objects.report.view.axes.AxisSort;
import com.indicee.api.objects.report.view.axes.QueryValue;
import com.indicee.api.util.XmlHelper;

public class AxisSortXml {
	
	public static final String TAG = "axissort";

	private static enum Tag {
		ascending,
		breakhierarchies,
		attributes,
	}
	
	public static final ToXml<AxisSort> toXml = new ToXml<AxisSort>() {
		@Override
		public Element call(Document document, AxisSort axisSort) {
			Element axisSortElement = createElement(document, TAG);
			setAttribute(axisSortElement, Tag.ascending,        axisSort.isAscending()        );
			setAttribute(axisSortElement, Tag.breakhierarchies, axisSort.isBreakHierarchies() );
			
			QueryValue queryValue = axisSort.getQueryValue();
			Element queryValueElement = QueryValueXml.toXml.call(document, queryValue);
			axisSortElement.appendChild(queryValueElement);
			
			List<AttributesOrigin> attributes = axisSort.getAttributes();
			AttributesOriginXml.toXml.append(axisSortElement, attributes);

			return axisSortElement;
		}
	};
	
	public static final FromXml<AxisSort> fromXml = new FromXml<AxisSort>() {
		@Override
		public AxisSort call(Element element) {
			AxisSort axisSort = new AxisSort();
			
			
			axisSort.setAscending       (getBoolean(element, Tag.ascending        ) );
			axisSort.setBreakHierarchies(getBoolean(element, Tag.breakhierarchies ) );
			
			List<AttributesOrigin> attributes = new ArrayList<AttributesOrigin>();
			axisSort.setAttributes(attributes);

			Element queryValueElement = XmlHelper.child(element, QueryValueXml.TAG);
			QueryValue queryValue = QueryValueXml.fromXml.call(queryValueElement);
			axisSort.setQueryValue(queryValue);

			for (Element attributeElement : XmlHelper.childElements(element, AttributesOriginXml.TAG)) {
				AttributesOrigin attribute = AttributesOriginXml.fromXml.call(attributeElement);
				attributes.add(attribute);
			}
			
			return axisSort;
		}
	};
}
