package com.indicee.api.xml;

import java.util.Map;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import com.indicee.api.objects.auth.Consumer;
import com.indicee.api.util.XmlHelper;

public class ConsumerXml {

	public static final String TAG = "consumer";
	
	private static enum Tag {
		name,
		description,
		website,
		key,
		secret
	}
	
	private static final Map<String,Tag> tagLookup = ToXml.createTagLookup(Tag.values());
	
	public static final ToXml<Consumer> toXml = new ToXml<Consumer>() {

		@Override
		public Element call(Document document, Consumer consumer) {
			Element element = createElement(document, TAG);
	
			append(element, Tag.name,        consumer.getName()        );
			append(element, Tag.description, consumer.getDescription() );
			append(element, Tag.website,     consumer.getWebsite()     );
			append(element, Tag.key,         consumer.getKey()         );
			append(element, Tag.secret,      consumer.getSecret()      );
			
			return element;
		}
		
	};
	
	public static final FromXml<Consumer> fromXml = new FromXml<Consumer>() {

		@Override
		public Consumer call(Element element) {
			Consumer consumer = new Consumer();
			for (Element childElement : XmlHelper.childElements(element)) {
				String tagName = childElement.getTagName();
				Tag t = tagLookup.get(tagName);
				if (t != null) {
					switch (t) {
					case name:        consumer.setName        (getText(childElement)); break;
					case description: consumer.setDescription (getText(childElement)); break;
					case website:     consumer.setWebsite     (getText(childElement)); break;
					case key:         consumer.setKey         (getText(childElement)); break;
					case secret:      consumer.setSecret      (getText(childElement)); break;
					}
				}
			}
			return consumer;
		}
	};
}

