/**
 * 
 */
package com.indicee.api.xml;

import java.util.Map;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import com.indicee.api.objects.contribution.Contribution;
import com.indicee.api.objects.contribution.Status;
import com.indicee.api.util.XmlHelper;

/**
 * @author thaggie
 *
 */
public class ContributionXml {

	public static final String TAG = "contribution";
	
	private static enum Tag {
		contribution_id,
		dataset_id,   //TODO missing from the POJO
		dataset_qname,
		name,
		unique_name,
		size,
		owner_id,
		status,
		rows_read,
		rows_processed,
		creation_date,
		ancillaries
		
	}
	
	private static final Map<String,Tag> tagLookup = ToXml.createTagLookup(Tag.values());
	
	public static final ToXml<Contribution> toXml = new ToXml<Contribution>() {

		@Override
		public Element call(Document document, Contribution contribution) {
			Element element = createElement(document, TAG);
	
			append(element, Tag.contribution_id, contribution.getId()            );
			append(element, Tag.name,            contribution.getName()          );
			append(element, Tag.dataset_qname,   contribution.getDatasetQName()  );
			append(element, Tag.unique_name,     contribution.getUniqueName()    );
			append(element, Tag.size,            contribution.getSize()          );
			append(element, Tag.owner_id,        contribution.getOwnerID()       );
			append(element, Tag.status,          contribution.getStatus()        );
			append(element, Tag.rows_read,       contribution.getRowsRead()      );
			append(element, Tag.rows_processed,  contribution.getRowsProcessed() );
			append(element, Tag.creation_date,  contribution.getCreationDate()   );
			append(element, Tag.ancillaries,     contribution.getAncillaries(), AncillaryXml.toXml);
			
			return element;
		}
		
	};
	
	public static final FromXml<Contribution> fromXml = new FromXml<Contribution>() {

		@Override
		public Contribution call(Element element) {
			Contribution contribution = new Contribution();
			for (Element childElement : XmlHelper.childElements(element)) {
				String tagName = childElement.getTagName();
				Tag t = tagLookup.get(tagName);
				if (t != null) {
					switch (t) {
					case contribution_id: contribution.setId            (getLong(childElement)); break;
					case name:            contribution.setName          (getText(childElement)); break;
					case dataset_qname:   contribution.setDatasetQName  (getText(childElement)); break;
					case unique_name:     contribution.setUniqueName    (getText(childElement)); break;
					case size:            contribution.setSize          (getLong(childElement)); break;
					case owner_id:        contribution.setOwnerID       (getLong(childElement)); break;
					case status:          contribution.setStatus        (Status.valueOf(getText(childElement))); break;
					case rows_read:       contribution.setRowsRead      (getInteger(childElement)); break;
					case rows_processed:  contribution.setRowsProcessed (getInteger(childElement)); break;
					case creation_date:   contribution.setCreationDate  (getDate(childElement)); break;
					case ancillaries:     contribution.setAncillaries   (FromXml.getChildren(childElement, AncillaryXml.fromXml)); break;
	                default: break;
 					}
				}
			}
			return contribution;
		}
	};
}
