package com.indicee.api.xml;

import java.util.Map;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import com.indicee.api.objects.dashboard.Dashboard;
import com.indicee.api.util.XmlHelper;

public class DashboardXml {
	public static final String TAG = "dashboard";
	
	private static enum Tag {
		id,
		qname,
		name,
		owner_id,
		description,
		definition,
		//TODO These are missing from Dashboard
		created_time,
		reports
	}
	
	private static final Map<String,Tag> tagLookup = ToXml.createTagLookup(Tag.values());
	
	public static final ToXml<Dashboard> toXml = new ToXml<Dashboard>() {

		@Override
		public Element call(Document document, Dashboard dashboard) {
			Element element = createElement(document, TAG);
	
			append(element, Tag.id,          dashboard.getId()          );
			append(element, Tag.qname,       dashboard.getQname()       );
			append(element, Tag.name,        dashboard.getName()        );
			append(element, Tag.owner_id,    dashboard.getOwnerId()     );
			append(element, Tag.description, dashboard.getDescription() );
			append(element, Tag.definition,  dashboard.getDefinition()  );
			
			return element;
		}
		
	};
	
	public static final FromXml<Dashboard> fromXml = new FromXml<Dashboard>() {

		@Override
		public Dashboard call(Element element) {
			Dashboard dashboard = new Dashboard();
			for (Element childElement : XmlHelper.childElements(element)) {
				String tagName = childElement.getTagName();
				Tag t = tagLookup.get(tagName);
				if (t != null) {
					switch (t) {
					case id:          dashboard.setId          (getLong(childElement)); break;
					case qname:       dashboard.setQname       (getText(childElement)); break;
					case name:        dashboard.setName        (getText(childElement)); break;
					case owner_id:    dashboard.setOwnerId     (getLong(childElement)); break;
					case description: dashboard.setDescription (getText(childElement)); break;
					case definition:  dashboard.setDefinition  (getText(childElement)); break;
	                default: break;
					}
				}
			}
			return dashboard;
		}
	};
}
