package com.indicee.api.xml;

import java.util.ArrayList;
import java.util.List;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import com.indicee.api.objects.report.view.filter.FieldFilter;
import com.indicee.api.objects.report.view.filter.FieldFilterValue;
import com.indicee.api.util.XmlHelper;

public class FieldFilterXml {
	public static final String TAG = "fieldfilter";
	
	private static enum Tag {
		not,
		field,
		inclusive,
		value,
		entity,
		values
	}
	
	public static final ToXml<FieldFilter> toXml = new ToXml<FieldFilter>() {
		
		@Override
		public Element call(Document document, FieldFilter fieldFilter) {
			Element ancillaryElement = createElement(document, TAG);
			Element element;
			if (fieldFilter.getNot()) {
				element = createElement(document, Tag.not);
				element.appendChild(ancillaryElement);
			} else { 
				element = ancillaryElement;
			}
				
			setAttribute(ancillaryElement, Tag.field, fieldFilter.getField());
			setAttribute(ancillaryElement, Tag.inclusive, fieldFilter.isInclusive());
			String entity = fieldFilter.getEntity();
			if (entity != null && entity.length() > 0) {
				setAttribute(ancillaryElement, Tag.entity, fieldFilter.getEntity());
			}
			
			List<FieldFilterValue> values = fieldFilter.getValues();
			if (values != null) {
				if (values.size() == 1) {
					FieldFilterValue value = values.get(0);
					setAttribute(ancillaryElement, Tag.value, value.getValue());		
				} else {
					append(ancillaryElement, Tag.values, values, FieldFilterValueXml.toXml);
				}
			}

			return element;
		}
	};
	
	public static final FromXml<FieldFilter> fromXml = new FromXml<FieldFilter>() {
		@Override
		public FieldFilter call(Element element) {
			FieldFilter fieldFilter = new FieldFilter();
			if (element.getTagName().equals(Tag.not.toString())) {
				fieldFilter.setNot(true);
				element = XmlHelper.child(element, TAG);
			}
			
			fieldFilter.setField(     getText    (element, Tag.field));
			fieldFilter.setInclusive( getBoolean (element, Tag.inclusive));
			fieldFilter.setEntity(    getText    (element, Tag.entity));
			
			String value = getText(element, Tag.value);
			if (value == null || value.length() == 0) {
				Element valuesElement = XmlHelper.child(element, Tag.values.toString());
				if (valuesElement != null) {
					List<FieldFilterValue> values = getChildren(valuesElement, FieldFilterValueXml.fromXml);
					fieldFilter.setValues(values);
				}
			} else {
				List<FieldFilterValue> values  = new ArrayList<FieldFilterValue>(1);
				values.add(new FieldFilterValue(value));
				fieldFilter.setValues(values);
			}
			
			return fieldFilter;
		}
	};
}
