package com.indicee.api.xml;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import org.joda.time.format.DateTimeFormat;
import org.joda.time.format.DateTimeFormatter;
import org.w3c.dom.Element;
import org.w3c.dom.Node;

import com.indicee.api.util.XmlHelper;

public abstract class FromXml<T> {
	public abstract T call(Element element);
	
	public List<T> toList(Element element) {
		List<Element> childElements = XmlHelper.childElements(element);
		List<T> children = new ArrayList<T>(childElements.size());
		for (Element childElement : childElements) {
			T t = call(childElement);
			children.add(t);
		}
		return children;
	}

	private static final DateTimeFormatter dateFormats[] = new DateTimeFormatter[] {
			DateTimeFormat.forPattern("yyyy-MM-dd'T'HH:mm:ss'Z'Z"),
			DateTimeFormat.forPattern("yyyy-MM-dd'T'HH:mm:ss'Z'"),
			DateTimeFormat.forPattern("yyyy-MM-dd'T'HH:mm:ssZZ"),
			DateTimeFormat.forPattern("EEE MMM dd HH:mm:ss z yyyy") };
	
	protected static String getText(Node child) {
		return child.getTextContent();
	}

	protected static Double getDouble(Node child) {
		String text = child.getTextContent();
		return Double.valueOf(text);
	}

	protected static String getText(Element child, Object attributeName) {
		return child.getAttribute(String.valueOf(attributeName));
	}

	protected static boolean getBoolean(Element child, Object attributeName) {
		String b = getText(child, attributeName);
		return Boolean.TRUE.toString().equals(b);
	}
	
	protected static int getInteger(Element child, Object attributeName) {
		String b = getText(child, attributeName);
		return Integer.parseInt(b);
	}

	protected static long getLong(Element child, Object attributeName) {
		String b = getText(child, attributeName);
		return Long.parseLong(b);
	}
	
	protected static boolean getBoolean(Element child) {
		String b = getText(child);
		return Boolean.TRUE.toString().equals(b);
	}

    protected static long getLong(Node child) {
    	return Long.parseLong(child.getTextContent());
    }
    
    protected static int getInteger(Node child) {
    	return Integer.parseInt(child.getTextContent());
    }

	protected static Date getDate(Node child) {
		if (child.getFirstChild() == null) {
			return null;
		}
		Node firstChild = child.getFirstChild();
		String text = firstChild.getNodeValue();
		for (DateTimeFormatter format : dateFormats) {
			try {
				return format.parseDateTime(text).toDate();
			} catch (Exception e) {
				// do nothing. Tries next DateFormat.
			}
		}
		throw new RuntimeException(
				"Unexpected exception parsing date. Unable to parse date: "
						+ child.getFirstChild().getNodeValue());
	}

	protected static <T> List<T> getChildren(Element element, FromXml<T> fromXml) {
		List<T> ts = new ArrayList<T>();
		for (Element child : XmlHelper.childElements(element)) {
			T t = fromXml.call(child);
			ts.add(t);
		}
		return ts;

	}
}
