package com.indicee.api.xml;

import java.util.Map;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import com.indicee.api.objects.report.view.axes.GroupFilter;
import com.indicee.api.util.XmlHelper;

public class GroupFilterXml {
	public static final String TAG = "groupfilter";
	
	private static enum Tag {
		comparator,
		comparatorattributes
	}
	
	private static final Map<String,Tag> tagLookup = ToXml.createTagLookup(Tag.values());
	
	public static final ToXml<GroupFilter> toXml = new ToXml<GroupFilter>() {
		
		@Override
		public Element call(Document document, GroupFilter groupFilter) {
			Element element = createElement(document, TAG);
			
			append(element, Tag.comparator, groupFilter.getComparator() );
			
			element.appendChild(QueryValueXml.toXml.call(document, groupFilter.getQueryValue()));
			
			append(element, Tag.comparatorattributes, groupFilter.getComparatorAttributes(), AttributeXml.toXml);
			return element;
		}
	};
	
	public static final FromXml<GroupFilter> fromXml = new FromXml<GroupFilter>() {
		@Override
		public GroupFilter call(Element element) {
			GroupFilter groupFilter = new GroupFilter();
			
			groupFilter.setComparator(getText(element, Tag.comparator));
			
			for (Element childElement : XmlHelper.childElements(element)) {
				String tagName = childElement.getTagName();
				if (QueryValueXml.TAG.equals(tagName)) {
					groupFilter.setQueryValue(QueryValueXml.fromXml.call(childElement));
				} else {
					Tag t = tagLookup.get(tagName);
					if (t != null) {
						switch (t) {
						case comparatorattributes:
							groupFilter.setComparatorAttributes(AttributeXml.fromXml.toList(childElement));
		                default:
		                	break;
						}
					}
				}
			}
			return groupFilter;
		}
	};
}
