package com.indicee.api.xml;

import java.util.Map;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import com.indicee.api.objects.group.Group;
import com.indicee.api.objects.group.Group.GroupColor;
import com.indicee.api.util.XmlHelper;

public class GroupXml {
public static final String TAG = "group";
	
	private static enum Tag {
		color,deleted,description,id,name,owner_id,_public("public");
		private Tag() {
			this.value = this.name();
		}
		private Tag(String value) {
			this.value = value;
		}
		private final String value;
		
		@Override
		public String toString() {
			return value;
		}
	}
	
	private static final Map<String,Tag> tagLookup = ToXml.createTagLookup(Tag.values());
	
	public static final ToXml<Group> toXml = new ToXml<Group>() {
		@Override
		public Element call(Document document, Group group) {
			
			Element groupElement = createElement(document, TAG);
			append(groupElement, Tag.id,			group.getId()					);
			append(groupElement, Tag.name,			group.getName()					);
			append(groupElement, Tag.description,	group.getDescription()			);
			append(groupElement, Tag.owner_id,		group.getOwnerId()				);
			append(groupElement, Tag.color,			group.getColor().toString()		);
			append(groupElement, Tag.deleted,		group.getDeleted().toString()	);
			append(groupElement, Tag._public,		group.getIsPublic().toString()	);
			
			return groupElement;
		}
	};
	
	public static final FromXml<Group> fromXml = new FromXml<Group>() {
		
		@Override
		public Group call(Element element) {
			Group group = new Group();
			for (Element childElement : XmlHelper.childElements(element)) {
				String tagName = childElement.getTagName();
				Tag t = tagLookup.get(tagName);
				if (t != null) {
	                switch (t) {
	                case id:
	                    group.setId(getLong(childElement));
	                    break;
	                case name:
	                    group.setName(getText(childElement));
	                    break;
	                case deleted:
	                	group.setDeleted(getBoolean(childElement));
	                	break;
	                case _public:
	                	group.setIsPublic(getBoolean(childElement));
	                	break;
	                case owner_id:
	                	group.setOwnerId(getLong(childElement));
	                	break;
	                case color:
	                	group.setColor(GroupColor.values()[getInteger(childElement)]);
	                	break;
	                case description:
	                	group.setDescription(getText(childElement));
	                	break;
	                }
				}
			}
			return group;
		}
	};
}
