package com.indicee.api.xml;

import java.util.List;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import com.indicee.api.objects.report.view.attributes.AttributesOrigin;
import com.indicee.api.objects.report.view.axes.QueryValue;
import com.indicee.api.objects.report.view.axes.QueryValueModel;
import com.indicee.api.objects.report.view.axes.QueryValueReport;
import com.indicee.api.util.XmlHelper;

public class QueryValueXml {

	public static final String TAG = "queryvalue";

	private enum Tag {
		aggregator,
		attributes,
		field,
		entity,
		name,
		formula
	}
	
	public static final ToXml<QueryValue> toXml = new ToXml<QueryValue>() {

		@Override
		public Element call(Document document, QueryValue queryValue) {
			Element element = createElement(document, TAG);
			
			if (queryValue instanceof QueryValueModel) {
				writeQueryValueModel(element, (QueryValueModel) queryValue);
			} else if (queryValue instanceof QueryValueReport) {
				writeQueryValueReport(element, (QueryValueReport) queryValue);
			}
			
			setAttribute(element, Tag.aggregator, queryValue.getAggregator());
			
			List<AttributesOrigin> attributes = queryValue.getAttributes();
			Element attributesElement = createElement(document, Tag.attributes);
			AttributesOriginXml.toXml.append(attributesElement, attributes);
			element.appendChild(attributesElement);
			
			return element;
		}
		
	};
	
	private static void writeQueryValueModel(Element element, QueryValueModel queryValueModel) {
		ToXml.setAttribute(element, Tag.field,  queryValueModel.getField()  );
		ToXml.setAttribute(element, Tag.entity, queryValueModel.getEntity() );
	}
	
	private static void writeQueryValueReport(Element element, QueryValueReport queryValueReport) {
		ToXml.setAttribute(element, Tag.name,    queryValueReport.getName()    );
		ToXml.setAttribute(element, Tag.formula, queryValueReport.getFormula() );
	}

	public static final FromXml<QueryValue> fromXml = new FromXml<QueryValue>() {
		@Override
		public QueryValue call(Element element) {
			
			QueryValue queryValue;
			String formula = element.getAttribute(Tag.formula.toString());
			if (formula != null && formula.length() > 0) {
				QueryValueReport queryValueReport = new QueryValueReport();
				queryValueReport.setFormula(formula);
				String name = getText(element, Tag.name);
				queryValueReport.setName(name);
				queryValue = queryValueReport;
			} else {
				QueryValueModel queryValueModel = new QueryValueModel();

				queryValueModel.setEntity(getText(element, Tag.entity ) );
				queryValueModel.setField (getText(element, Tag.field  ) );
				
				queryValue = queryValueModel;
			}
			
			queryValue.setAggregator(getText(element, Tag.aggregator));
			
			Element attributesElement = XmlHelper.child(element, Tag.attributes.toString());
			List<AttributesOrigin> attributes = AttributesOriginXml.fromXml.toList(attributesElement);
			queryValue.setAttributes(attributes);
			return queryValue;
		}
	};
}
