package com.indicee.api.xml;

import java.util.Map;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import com.indicee.api.objects.report.view.filter.RelativeTimePeriodFilter;
import com.indicee.api.util.XmlHelper;

public class RelativeTimePeriodFilterXml {
	public static final String TAG = "relativetimeperiodfilter";
	
	private static enum Tag {
		field,
		calendar,
		inclusive,
		entity,
		relativetimeperiods
	}
	
	private static final Map<String,Tag> tagLookup = ToXml.createTagLookup(Tag.values());
	
	public static final ToXml<RelativeTimePeriodFilter> toXml = new ToXml<RelativeTimePeriodFilter>() {
		@Override
		public Element call(Document document, RelativeTimePeriodFilter filter) {
			
			Element filterElement = createElement(document, TAG);
			
			setAttribute(filterElement, Tag.field, filter.getField());
			setAttribute(filterElement, Tag.calendar, filter.getCalendar());
			setAttribute(filterElement, Tag.inclusive, filter.isInclusive());
			
			String entity = filter.getEntity();
			if (entity != null && entity.length() > 0) {
				setAttribute(filterElement, Tag.entity, entity);
			}
			
			append(filterElement,Tag.relativetimeperiods, filter.getRelativeTimePeriods(), RelativeTimePeriodXml.toXml);
			return filterElement;
		}
	};
	
	public static final FromXml<RelativeTimePeriodFilter> fromXml = new FromXml<RelativeTimePeriodFilter>() {
		
		@Override
		public RelativeTimePeriodFilter call(Element element) {
			RelativeTimePeriodFilter filter = new RelativeTimePeriodFilter();
			
			filter.setField(getText(element, Tag.field));
			filter.setCalendar(getText(element, Tag.calendar));
			filter.setInclusive(getBoolean(element, Tag.inclusive));
			
			String entity = getText(element, Tag.entity);
			if (entity != null && entity.length() > 0) {
				filter.setEntity(entity);
			}
			
			for (Element childElement : XmlHelper.childElements(element)) {
				String tagName = childElement.getTagName();
				Tag t = tagLookup.get(tagName);
				if (t != null) {
	                switch (t) {
	                case relativetimeperiods:
	                	filter.setRelativeTimePeriods(getChildren(childElement, RelativeTimePeriodXml.fromXml));
	                	break;
	                default:
	                	break;
	                }
				}
			}
			return filter;
		}
	};

}
