package com.indicee.api.xml;

import java.util.Collection;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import com.indicee.api.util.XmlHelper;

public abstract class ToXml<T> {

	public abstract Element call(Document document, T t);

	public void append(Element element, List<T> ts) {
		int nt = ts == null ? 0 : ts.size();
		
		Document document = element.getOwnerDocument();
		for (int i=0; i<nt;++i) {
			Element tElement = call(document, ts.get(i));
			element.appendChild(tElement);
		}
	}
	
	protected static final Element createElement(Document document, Object tag) {
		return document.createElement(String.valueOf(tag));
	}

	protected static final void setAttribute(Element element, Object tag, Object value) {
		if (value != null) {
			element.setAttribute(String.valueOf(tag), String.valueOf(value));
		}
	}

	protected static final Element append(Element element, Object tag, Object value) {
		if (value == null) {
			return null;
		}
		return XmlHelper.appendTextElement(element, String.valueOf(tag), String.valueOf(value));
	}

	protected static final Element append(Element element, Object tag, boolean value) {
		return append(element, tag, Boolean.toString(value));
	}

	protected static final Element append(Element element, Object tag, long value) {
		return append(element, tag, Long.toString(value));
	}

	protected static final Element append(Element element, Object tag, double value) {
		return append(element, tag, Double.toString(value));
	}
	
	protected static final Element append(Element element, Object tag, Date value) {
		if (value == null) {
			return null;
		}
		return append(element, tag, dateToString(value));
	}
	
	
	protected static <T> Element append(Element element, Object tag, Collection<T> ts, ToXml<T> toXml) {
		Document document = element.getOwnerDocument();
		Element containerElement = document.createElement(String.valueOf(tag));
		if (ts != null) {
			for (T t : ts) {
				Element childElement = toXml.call(document, t);
				containerElement.appendChild(childElement);
			}
		}
		element.appendChild(containerElement);
		
		return containerElement;
	}
	
    @SuppressWarnings("deprecation")
	private static String dateToString(Date date) {
        return (date.getYear() + 1900) + "-" + pad(date.getMonth() + 1) + "-" + pad(date.getDate()) + "T"
                + pad(date.getHours()) + ":" + pad(date.getMinutes()) + ":" + pad(date.getSeconds()) + "-"
                + pad(date.getTimezoneOffset() / 60) + ":" + pad(date.getTimezoneOffset() % 60);
    }

    private static String pad(int num) {
        String number = String.valueOf(num);
        return number.length() == 2 ? number : "0" + number;
    }
    
	protected static <T> Map<String,T> createTagLookup(T[] ts) {
		Map<String,T> lookup = new HashMap<String,T>(ts.length);
		
		for (T t : ts) {
			lookup.put(t.toString(), t);
		}
		return lookup;
	}
}
