package com.indicee.api.parser;

import junit.framework.TestCase;

import org.w3c.dom.Document;

import com.indicee.api.objects.Job;
import com.indicee.api.objects.JobStatus;
import com.indicee.api.util.XmlHelper;

public class JobParserTest extends TestCase {

	
	private static final String JOB_XML = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n"+
		"<job>\n"+
		"  <created>2013-01-23T09:11:21-08:00</created>\n"+
		"  <description>Load contrib(81) to dataSet iqn.2://indicee.com/3/0/dataModel/dataSets/FeedItem_v3.csv</description>\n"+
		"  <expires>2013-01-24T09:11:21-08:00</expires>\n"+
		"  <finished>2013-01-23T09:11:21-08:00</finished>\n"+
		"  <id>f4840583-d465-4aa3-a432-3e2c1ee4aa38</id>\n"+
		"  <message>Loading</message>\n"+
		"  <owner>3</owner>\n"+
		"  <percent_complete>100</percent_complete>\n"+
		"  <started>2013-01-23T09:11:21-08:00</started>\n"+
		"  <status>Succeeded</status>\n"+
		"  <type>datauploader</type>\n"+
		"</job>";
	
	
	public static void testParseJobXML() {
		
		Document document = XmlHelper.readXmlDocumentFromString(JOB_XML);
		Job job = JobParser.createJob(document);
		
		assertContent(job);
		
		String jobXml = job.toFormatedString();
		Document document2 = XmlHelper.readXmlDocumentFromString(jobXml);
		Job job2 = JobParser.createJob(document2);
		
		assertContent(job2);
	}


	private static void assertContent(Job job) {
		assertEquals(JobParser.parseDate("2013-01-23T09:11:21-08:00"), job.getCreated());
		assertEquals("Load contrib(81) to dataSet iqn.2://indicee.com/3/0/dataModel/dataSets/FeedItem_v3.csv", job.getDescription());
		assertEquals(JobParser.parseDate("2013-01-24T09:11:21-08:00"), job.getExpires());
		assertEquals(JobParser.parseDate("2013-01-23T09:11:21-08:00"), job.getFinished());
		assertEquals("f4840583-d465-4aa3-a432-3e2c1ee4aa38", job.getId());
		assertEquals("Loading", job.getMessage());
		assertEquals(3L, job.getOwner());
		assertEquals(100, job.getPercentComplete());
		assertEquals(JobParser.parseDate("2013-01-23T09:11:21-08:00"), job.getStarted());
		assertEquals(JobStatus.Succeeded, job.getStatus());
		assertEquals("datauploader", job.getType());
	}
}
