package com.indicee.api.parser;

import java.util.List;

import junit.framework.TestCase;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import com.indicee.api.objects.model.Relationship;
import com.indicee.api.util.XmlHelper;

public class RelationshipParserTest extends TestCase {

	private static final String RELATIONSHIPS_XML = 
		  "<relationships>" +
		  "  <relationship>" +
		  "    <uuid>2869b203-ea4d-4fed-8470-6db2b86e5a89</uuid>" +
		  "    <left_entity>iqn.2://indicee.com/3/0/dataModel/entities/users</left_entity>" +
		  "    <relationship>MANY_TO_ONE</relationship>" +
		  "    <right_entity>iqn.2://indicee.com/3/0/dataModel/entities/User_Div%20%28users%20property%29</right_entity>" +
		  "  </relationship>" +
		  "  <relationship>" +
		  "    <uuid>e27e072f-17c2-4e76-a007-ff857947e3fa</uuid>" +
		  "    <left_entity>iqn.2://indicee.com/3/0/dataModel/entities/opportunities</left_entity>" +
		  "    <relationship>MANY_TO_ONE</relationship>" +
		  "    <right_entity>iqn.2://indicee.com/3/0/dataModel/entities/users</right_entity>" +
		  "  </relationship>" +
		  "</relationships>";
	@SuppressWarnings("deprecation")
	public void testParseRelationship() {
	
		String uuid = "2869b203-ea4d-4fed-8470-6db2b86e5a89";
		String left = "iqn.2://indicee.com/3/0/dataModel/entities/users";
		Relationship.Type type = Relationship.Type.MANY_TO_ONE;
		String right = "iqn.2://indicee.com/3/0/dataModel/entities/User_Div%20%28users%20property%29";
		
		Relationship originalRelationship = new Relationship();
		originalRelationship.setUuid(uuid);
		originalRelationship.setLeftQName(left);
		originalRelationship.setType(type);
		originalRelationship.setRightQName(right);
		
		String xmlString = originalRelationship.toString();
		Document document = XmlHelper.readXmlDocumentFromString(xmlString);
		Element element = document.getDocumentElement();
		Relationship parsedRelationship = RelationshipParser.parseRelationship(element);
		
		assertEquals(uuid, parsedRelationship.getUuid());
		assertEquals(left, parsedRelationship.getLeftQName());
		assertEquals(type, parsedRelationship.getType());
		assertEquals(right, parsedRelationship.getRightQName());
		
	}
	
	public void testParseRelationships() {
		Document document = XmlHelper.readXmlDocumentFromString(RELATIONSHIPS_XML);
		Element element = document.getDocumentElement();
		List<Relationship> relationships = RelationshipParser.parseRelationships(element);
		assertEquals(2, relationships.size());
	}
}
